<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Components.AriSmartContent.Plugins.PluginBase');

class AriJDataTablePlugin extends AriSmartContentPluginBase
{
	var $_idList = array();
	var $_defaultConfig = array(
		'bStateSave' => false,
		'iCookieDuration' => 7200,
		'bPaginate' => true,
		'sPaginationType' => 'two_button',
		'bLengthChange' => true,
		'bFilter' => true,
		'bSort' => true,
		'bInfo' => true,
		'bProcessing' => false,
		'bAutoWidth' => true,
		'iDisplayLength' => 10,
		'sDom' => 'lfrtip',
		'asStripClasses' => 'odd,even',
		'oLanguage' => array(
			'sProcessing' => 'Processing...',
			'sLengthMenu' => 'Show _MENU_ entries',
			'sZeroRecords' => 'No matching records found',
			'sInfo' => 'Showing _START_ to _END_ of _TOTAL_ entries',
			'sInfoEmpty' => 'Showing 0 to 0 of 0 entries',
			'sInfoFiltered' => '(filtered from _MAX_ total entries)',
			'sInfoPostFix' => '',
			'sSearch' => 'Search:',
			'oPaginate' => array(
				'sFirst' => 'First',
				'sPrevious' => 'Previous',
				'sNext' => 'Next',
				'sLast' => 'Last'
			)
		));

	function __construct()
	{
		parent::__construct('arijdatatable');
	}
	
	function processContent($published, &$row, &$params, $page = 0)
	{
		parent::processContent($published, $row, $params, $page);
		
		$this->runtimeJsDeclaration();
	}
	
	function replaceCallback($attrs, $content = '')
	{
		$customPagination = $this->getCustomPagination($attrs);
		$config = $this->getConfig($attrs, $this->_defaultConfig, '', $customPagination);
		
		$id = uniqid('jdt_', false);
		$this->_idList[$id] = $config;

		return sprintf('<div id="%2$s">%1$s</div>',
			$content,
			$id);
	}
	
	function getCustomPagination($attrs)
	{
		$customPagination = array();
		$sCustomPagination = $this->getValueByDefSample('customPagination', $attrs, '');

		if ($sCustomPagination)
		{
			$pagination = explode(',', $sCustomPagination);
			foreach ($pagination as $value)
			{
				$length = @intval($value, 10);
				if ($length < 1) continue;
				
				$customPagination[] = $length;
			}
			
			$customPagination = array_unique($customPagination);
			sort($customPagination, SORT_NUMERIC);
		}
		
		return count($customPagination) > 0 ? $customPagination : null;
	}
	
	function getConfig($attrs, $defConfig, $prefix = '', $customPagination)
	{
		$config = array('bJQueryUI' => $this->getUseJQueryUI());
		foreach ($defConfig as $key => $defValue)
		{
			if ($prefix == 'oLanguage_' && $key == 'sLengthMenu' && $customPagination)
			{ 
				$val = $this->getValueByDefSample($prefix . $key, $attrs, $defValue);
				$pagingMenu = '<select size="1">';
				foreach ($customPagination as $length)
				{
					$pagingMenu .= sprintf('<option value="%1$d">%1$d</option>',
						$length);
				}
				$pagingMenu .= '</select>';
				
				$config[$key] = str_replace('_MENU_', $pagingMenu, $val);

				continue;
			}
			else if ($key == 'asStripClasses')
			{
				$val = $this->getValueByDefSample($prefix . $key, $attrs, $defValue);
				if ($val != $defValue)
				{
					$config[$key] = explode(',', $val);
				}
				continue;
			}
			else if ($key == 'iDisplayLength')
			{
				$val = $this->getValueByDefSample($key, $attrs, $defValue);
				if ($customPagination)
				{
					if (!in_array($val, $customPagination)) $val = $customPagination[0];
					
					$config[$key] = $val;
				}
				else if ($val != $defValue && in_array($val, array(10,25,50,100)))
				{
					$config[$key] = $val;
				}
				continue;
			}
			
			if (is_array($defValue))
			{
				$subConfig = $this->getConfig($attrs, $defValue, $prefix . $key . '_', $customPagination);
				if (!empty($subConfig)) $config[$key] = $subConfig;
			}
			else
			{
				$val = $this->getValueByDefSample($prefix . $key, $attrs, $defValue);
				if ($val != $defValue) $config[$key] = $val;
			}
		}
		
		$config['aaSorting'] = array();

		return $config;
	}
	
	function jsDeclaration()
	{
		$useJQueryUI = $this->getUseJQueryUI();
		
		$jqueryLoader = new AriJQueryLoaderPlugin();
		$jqueryLoader->load();

		$rootPath = $this->getRootPath();
		$jsPath = $rootPath . 'js/';
		$cssPath = $rootPath . 'css/';
		$cssFile = $useJQueryUI ? 'datatable_jui.css' : 'datatable.css';
		
		AriDocumentHelper::includeCssFile(AriUtils::absPath2Url($cssPath . $cssFile));
		AriDocumentHelper::includeJsFile(AriUtils::absPath2Url($jsPath . 'jquery.dataTables.min.js'));
		AriDocumentHelper::includeJsFile(AriUtils::absPath2Url($jsPath . 'jquery.dataTablesExt.min.js'));
		
		if ($useJQueryUI)
		{
			$jqueryUiLoader = new AriJQueryUILoaderPlugin();
			$jqueryUiLoader->loadTheme(null, '1.7.2');
		}
	}
	
	function runtimeJsDeclaration()
	{
		$idList = $this->_idList;
		if (empty($idList)) return ;
		
		$script = '<script type="text/javascript">';
		foreach ($idList as $id => $config)
		{
			$script .= sprintf('jQuery(document).ready(function($) { $("#%1$s > table").addClass("display").dataTable(%2$s); });', 
				$id,
				!empty($config) ? AriJSONHelper::encode($config) : '');
		}
		$script .= '</script>';
		
		AriDocumentHelper::includeCustomHeadTag($script);
	}
	
	function getUseJQueryUI()
	{
		$useJQueryUI = $this->getValueByDefSample('bJQueryUI', null, false);

		return $useJQueryUI;
	}
	
	function getRootPath()
	{
		return dirname(__FILE__) . '/media/';
	}
}
?>