/** ARI Soft copyright
 * Copyright (C) 2008 ARI Soft.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from ARI Soft. This notice must be included on 
 * all copies, modifications and derivatives of this work.
 *
 * ARI Soft products are provided "as is" without warranty of 
 * any kind, either expressed or implied. In no event shall our 
 * juridical person be liable for any damages including, but 
 * not limited to, direct, indirect, special, incidental or 
 * consequential damages or other losses arising out of the use 
 * of or inability to use our products.
 *
**/

YAHOO.namespace('ARISoft.page');

YAHOO.ARISoft.page.pageController = function(config)
{
	this.init(config);
};

YAHOO.ARISoft.page.pageController.prototype =
{
	_actionGroups: {},

	_actions: {},

	MESSAGE_TYPE:
	{
		INFO: 1,
		WARNING: 2,
		ERROR: 4
	},

	init: function(config)
	{
		this.createEvent('beforeAction');
		this.createEvent('afterAction');
		this.createEvent('sendInfoMessage');
		
		config = config || {};
		this._applyConfig(config);
	},
	
	_applyConfig: function(config)
	{
		var defConfig = this.defaultConfig;
		for (var prop in defConfig)
		{
			this[prop] = (!YAHOO.lang.isUndefined(config[prop])) ? config[prop] : defConfig[prop]; 
		}
	},
	
	registerActionGroup: function(groupName, config)
	{
		this._actionGroups[groupName] = config;
	},
	
	registerAction: function(action, config)
	{
		this._actions[action] = config;
	},
	
	triggerAction: function(action, options)
	{
		options = options || {};
		var config = this.getActionConfig(action, options);
		if (config.onAction)
		{
			this.fireEvent('beforeAction', {action: action, config: config});

			config.onAction.call(this, action, config);
			
			this.fireEvent('afterAction', {action: action, config: config});
		}
		else if (this.defaultAction)
		{
			this.fireEvent('beforeAction', {action: action, config: config});
			
			this.defaultAction.call(this, action, config);
			
			this.fireEvent('afterAction', {action: action, config: config});
		}
	},
	
	getActionConfig: function(action, overrideParams)
	{
		var config = {};
		YAHOO.lang.augmentObject(config, this.defaultActionConfig, true);
		var actionConfig = this._actions[action] || {};
		var groupName = actionConfig.group || config.group;

		if (groupName && !YAHOO.lang.isUndefined(this._actionGroups[groupName]))
		{
			var groupConfig = this._actionGroups[groupName];
			YAHOO.lang.augmentObject(config, groupConfig, true);
		}
		
		YAHOO.lang.augmentObject(config, actionConfig, true);
		YAHOO.lang.augmentObject(config, overrideParams, true);

		return config;
	},
	
	sendInfoMessage: function(message, type)
	{
		type = type || this.MESSAGE_TYPE.INFO; 
		this.fireEvent('sendInfoMessage', {message: message, type: type});
	},
	
	defaultConfig: 
	{
		baseUrl: '',
		adminBaseUrl: '',
		formId: 'adminForm',
		option: '',
		defaultAction: null
	},
	
	defaultActionConfig:
	{
		group: null,
		onAction: null,
		enableValidation: false
	}
};

YAHOO.augment(YAHOO.ARISoft.page.pageController, YAHOO.util.EventProvider);

YAHOO.ARISoft.page.actionHandlers = 
{
	simpleDatatableAction: function(action, config)
	{
		var actionUrl = 'index.php?option=' + this.option + '&task=' + action;
		if (config.query)
		{
			for (var name in config.query)
			{
				actionUrl += '&' + name + '=' + config.query[name];  
			}
		};

		var pm = this;

		YAHOO.ARISoft.ajax.ajaxManager.asyncRequest(
			'POST',
			actionUrl,
			{
				cache: false, 
				success: function(oResponse)
				{
					var result = false;
					var args = oResponse.argument;
					var config = args.config || {};
					try
					{
						var responseText = oResponse.responseText;
						eval('result = (' + responseText + ')');				
					}
					catch (e) {};

					this.sendInfoMessage(result ? config.completeMessage : config.errorMessage, this.MESSAGE_TYPE.INFO);
					YAHOO.ARISoft.widgets.dataTable.refresh(config.dataTable);
					
					if (config.onComplete) config.onComplete.call(this);
				},
				failure: function(oResponse) 
				{
					var args = oResponse.argument;
					var config = args.config || {};
				
					this.sendInfoMessage(config.errorMessage, this.MESSAGE_TYPE.ERROR);
					
					if (config.onFailure) config.onFailure.call(this);
				},
				argument:
				{
					config: config
					/*
					completeMessage: config.completeMessage,
					errorMessage: config.errorMessage,
					dataTable: config.dataTable*/
				},
				scope: pm
			},
			null,
			config.formId || this.formId,
			{
				containerId: (config.dataTable.getContainerEl()),
				loadingMessage: config.loadingMessage,
				overlayCfg:
				{ 
					visible:false, 
					constraintoviewport:true, 
					close: false,
					draggable: false,
					autofillheight: 'body',
					zIndex: 10000
				}
			});
	}
};