<?php
/*
 * ARI Framework Lite
 *
 * @package		ARI Framework Lite
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2009 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Articles.ArticleSearchStruct');
AriKernel::import('Mambot.MambotProcessHelper');
AriKernel::import('Joomla.JoomlaUtils');

require_once JPATH_ROOT . DS . 'components' . DS . 'com_content' . DS . 'helpers' . DS . 'route.php';

class AriJoomlaArticleController
{
	function getSections() 
	{
		$database =& JFactory::getDBO();

		$query = 'SELECT id, title' .
			' FROM #__sections S' .
			' WHERE published = 1 AND' .
			' scope = "content"' .
			' ORDER BY title ASC';

		$database->setQuery($query);
		$sections = $database->loadObjectList();
		if ($database->getErrorNum()) 
		{
			return null; 
		}

		return $sections;
	}
	
	function getCategories() 
	{
		$database =& JFactory::getDBO();

		$query = 'SELECT C.id, CONCAT_WS("/", S.title, C.title) AS title' .
			' FROM #__categories AS C' .
			' LEFT JOIN #__sections AS S ON S.id = C.section' .
			' WHERE C.published = 1 AND' .
			' S.scope = "content"' .
			' ORDER BY S.title ASC, C.title ASC';
		
		$database->setQuery($query);
		$categories = $database->loadObjectList();
		if ($database->getErrorNum()) 
		{
			return null; 
		}
		
		return $categories;
	}
	
	function getArticleList($searchStruct, $processTag = true)
	{
		$my =& JFactory::getUser();
		$database =& JFactory::getDBO();
		
		$userId = $my->get('id');
		
		$config =& JFactory::getConfig();
    	$nullDate = $database->getNullDate();
		$now =& JFactory::getDate('now', $config->getValue('config.offset'));
		
		$excFrontend = $searchStruct->getExcludeFrontend();
		$categoryIdList = $searchStruct->getCategoryIdList();
		$sectionIdList = $searchStruct->getSectionIdList();
		$creatorIdList = $searchStruct->getCreatorIdList();
		$incArticleIdList = $searchStruct->getIncludeArticleIdList();
		$excArticleIdList = $searchStruct->getExcludeArticleIdList();

		$query = 'SELECT C.*,CC.title AS category_title,CS.title AS section_title,U.name AS author_name,U.username AS author_username,' .
			' CASE WHEN CHAR_LENGTH(C.alias) THEN CONCAT_WS(":", C.id, C.alias) ELSE C.id END AS Slug,'.
			' CASE WHEN CHAR_LENGTH(CC.alias) THEN CONCAT_WS(":", CC.id, CC.alias) ELSE CC.id END AS CatSlug'. 
			' FROM #__content C';

		if ($excFrontend)
			$query .= ' LEFT JOIN #__content_frontpage CF ON C.id = CF.content_id';

		$query .= ' LEFT JOIN #__categories CC ON C.catid = CC.id';
		$query .= ' LEFT JOIN #__sections CS ON C.sectionid = CS.id';
		$query .= ' LEFT JOIN #__users U ON U.id = C.created_by';

		$query .= sprintf(' WHERE (%4$s(C.state = 1) AND (IFNULL(CC.published,1) = 1) AND (IFNULL(CS.published,1) = 1)' .
			' AND (C.publish_up = %1$s OR C.publish_up <= %2$s) AND (C.publish_down = %1$s OR C.publish_down >= %2$s)' .
			' AND (%3$s > 0 OR (IFNULL(C.access,0) = 0 AND IFNULL(CC.access,0) = 0 AND IFNULL(CS.access,0) = 0)))',
			$database->Quote($nullDate),
			$database->Quote($now->toMySQL()),
			$userId,
			$excFrontend 
				? $excFrontend == 'only' 
					? 'CF.content_id IS NOT NULL AND'
					: 'CF.content_id IS NULL AND ' 
				: ''
		);

		$whereArticle = array();
		if ($categoryIdList)
		{
			$whereArticle[] = sprintf('(C.catid %s IN (%s))',
				$searchStruct->getExcludeCategory() ? 'NOT' : '',
				join(',', $this->_quoteValues($categoryIdList)));
		}
		
		if ($sectionIdList)
		{
			$whereArticle[] = sprintf('(C.sectionid %s IN (%s))',
				$searchStruct->getExcludeSection() ? 'NOT' : '',
				join(',', $this->_quoteValues($sectionIdList)));
		}
		
		if ($creatorIdList)
		{
			$whereArticle[] = sprintf('(C.created_by %s IN (%s))',
				$searchStruct->getExcludeCreator() ? 'NOT' : '',
				join(',', $this->_quoteValues($sectionIdList)));
		}
		
		if ($excArticleIdList)
		{
			$whereArticle[] = sprintf('(C.id NOT IN (%s))',
					join(',', $this->_quoteValues($excArticleIdList)));
		}
		
		$whereArticle = join(' AND ', $whereArticle);
		if ($incArticleIdList)
		{
			if ($whereArticle)
				$whereArticle = '(' . $whereArticle . ') OR ';
					
			$whereArticle .= sprintf('(C.id IN (%s))',
				join(',', $this->_quoteValues($incArticleIdList)));
		}
		
		if ($whereArticle)
			$query .= ' AND (' . $whereArticle . ')';

		if ($searchStruct->getIsRandom())
		{
			$query .= ' ORDER BY RAND()';
		}
		else
		{		
			$sort = $searchStruct->getSort();
			if (is_array($sort) && count($sort) > 0)
			{
				$sortPart = array();
				foreach ($sort as $sortItem)
				{
					$sortPart[] = 'C.`' . $sortItem['field'] . '` ' . $sortItem['dir'];
				}

				$query .= ' ORDER BY ' . join(',', $sortPart);
			}
		}
		
		$itemCount = $searchStruct->getItemCount();
		if ($itemCount > 0)
		{
			$query .= ' LIMIT 0,' . $itemCount;
		}

		$database->setQuery($query);
		$articleList = $database->loadObjectList();
		if ($database->getErrorNum())
		{
			return null;
		}

		if ($articleList)
		{
			for ($i = 0, $c = count($articleList); $i < $c; $i++)
			{
				$article = $articleList[$i]; 
				if ($processTag) $article = AriMambotProcessHelper::processArticle($article);
				$article->readmore_link = JRoute::_(ContentHelperRoute::getArticleRoute($article->Slug, $article->CatSlug, $article->sectionid));
				
				$articleList[$i] = $article;
			}
		}
		
		return $articleList;
	}
	
	function getArticleLink($article)
	{
		$link = 'index.php?option=com_content&view=article&id=' . $article->id;
		if ($article->catid)
		{
			$link .= '&catid=' . $article->catid;  
		}

		return $link;
	}
	
	function _quoteValues($arr)
	{
		$database =& JFactory::getDBO();
		if (!empty($arr))
		{
			foreach ($arr as $key => $value)
				$arr[$key] = $database->Quote($value);
		}

		return $arr;
	}
}
?>