<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Components.AriSmartContent.Plugins.PluginBase');
AriKernel::import('Components.AriSmartContent.Plugins.SimpleContentPlugin');
AriKernel::import('SimpleTemplate.SimpleTemplate');

class AriSmartHtmlTablePlugin extends AriSmartContentPluginBase
{
	var $_data;
	var $_columnsTag;
	var $_columnTag;
	var $_spanTag;
	var $_defaultConfig = array(
		'header' => true,
		'footer' => false,
		'cellPadding' => 0,
		'cellSpacing' => 0,
		'width' => '',
		'className' => '',
		'inputEncoding' => '',
		'advancedStyling' => true);
	
	function __construct($tag, $columnsTag, $columnTag, $spanTag, $paramKey)
	{
		$this->setParametersKey($paramKey);
		
		$this->_columnsTag = $columnsTag;
		$this->_columnTag = $columnTag;
		$this->_spanTag = $spanTag;
		
		parent::__construct($tag);
	}
	
	function getColumnsTag()
	{
		return $this->_columnsTag;
	}
	
	function getColumnTag()
	{
		return $this->_columnTag;
	}
	
	function getSpanTag()
	{
		return $this->_spanTag;
	}

	function getDataKeys($data)
	{
		$keys = array();
		if (is_array($data) && count($data) > 0)
		{
			$tKeys = array_keys($data);
			$dataItem = $data[$tKeys[0]];
			$keys = array_keys($dataItem);
		}
		
		return $keys;
	}
	
	function replaceCallback($data, $attrs, $content = '')
	{
		$config = $this->getConfig($attrs);
		$colsPlugin = new AriSmartHtmlTableColumnsPlugin($this->getColumnsTag(), $this->getColumnTag(), $this->getParametersKey());
		$colsPlugin->processContent(true, $content, $this->_processParams, $this->_page);
		$columns = $colsPlugin->getColumns();

		$colsSpanPlugin = new AriSmartHtmlTableColumnSpanPlugin($this->getSpanTag(), $this->getParametersKey());
		$colsSpanPlugin->processContent(true, $content, $this->_processParams, $this->_page);
		$spanInfo = $colsSpanPlugin->getDefinition();
		
		$data = $this->modifyData($data, $columns);
		$keys = $this->getDataKeys($data);

		$tableAttrs = array(
			'style' => array('width' => trim($config['width'])), 
			'class' => !empty($config['className']) ? $config['className'] : null, 
			'cellpadding' => $config['cellPadding'],
			'cellspacing' => $config['cellSpacing']);
		if (empty($tableAttrs['style']['width'])) unset($tableAttrs['style']['width']);

		$colsCount = count($keys);
		$table = sprintf('<table%1$s>', 
			AriHtmlHelper::getAttrStr($tableAttrs));

		// Create header
		if ($config['header'])
		{
			$table .= '<thead><tr>';
			
			$header = '';
			$i = 0;
			$rowSpan = isset($spanInfo[-1]) ? $spanInfo[-1] : array();
			
			$spanCnt = 0;
			foreach ($keys as $key)
			{
				--$spanCnt;
				if ($spanCnt > 0)
				{ 
					$i++;
					continue;
				}
				
				$label = $key;
				$colHeadAttrs = array('class' => '');
				if ($config['advancedStyling']) $colHeadAttrs['class'] = 'actCell_' . $i;
				if (isset($rowSpan[$i]))
				{
					$endIndex = $rowSpan[$i]['endIndex'];
					if ($endIndex == 0 || $endIndex >= $colsCount) $endIndex = $colsCount - 1;
					$spanCnt = $endIndex - $i + 1;
					$colHeadAttrs['colspan'] = $spanCnt;
				}
				$cellTag = 'th';
				if (isset($columns[$key]))
				{
					$column = $columns[$key];
					if ($column['alias'])
					{ 
						$label = $column['alias'];
					}
					if ($column['headerClassName'])
					{
						$colHeadAttrs['class'] .= ' ' . $column['headerClassName'];
					}
					if ($column['width'])
					{
						$colHeadAttrs['style']['width'] = $column['width'];
					}
					if ($column['attrWidth'])
					{
						$colHeadAttrs['width'] = $column['attrWidth'];
					}
					
					$cellTag = $column['headCellTag'];
				}
				 
				$header .= sprintf('<%1$s%2$s>%3$s</%1$s>',
					$cellTag,
					AriHtmlHelper::getAttrStr($colHeadAttrs),
					$this->translateValue($config['inputEncoding'], $label));
				$i++;
			}
	
			$table .= $header;
			$table .= '</tr></thead>';
			
			if ($config['footer']) $table .= '<tfoot><tr>' . $header . '</tr></tfoot>';
		}
		
		// Create body
		$table .= '<tbody>';
		$rowIndex = 0;
		foreach ($data as $resultItem)
		{	
			$table .= sprintf('<tr%s>',
				$config['advancedStyling']
					? sprintf(' class="actRow_%d actRow_%s"',
						$rowIndex,
						$rowIndex % 2 ? 'even' : 'odd')
					: '');
			$colIndex = 0;
			$rowSpan = isset($spanInfo[$rowIndex]) ? $spanInfo[$rowIndex] : array();
			$spanCnt = 0;
			foreach ($resultItem as $key => $value)
			{
				--$spanCnt;
				if ($spanCnt > 0)
				{ 
					++$colIndex;
					continue;
				}
				
				$tdAttrs = array('class' => '');
				if ($config['advancedStyling']) $tdAttrs['class'] = sprintf('actCell_' . $colIndex);
				if (isset($rowSpan[$colIndex]))
				{
					$endIndex = $rowSpan[$colIndex]['endIndex'];
					if ($endIndex == 0 || $endIndex >= $colsCount) $endIndex = $colsCount - 1;
					$spanCnt = $endIndex - $colIndex + 1;
					$tdAttrs['colspan'] = $spanCnt;
				}
				$cellTag = 'td';
				if (isset($columns[$key]))
				{
					$column = $columns[$key];
					
					if ($column['className']) $tdAttrs['class'] .= ' ' . $column['className'];
					if ($column['cellTag']) $cellTag = $column['cellTag']; 
				}
				$table .= sprintf('<%1$s%2$s>%3$s</%1$s>',
					$cellTag,
					AriHtmlHelper::getAttrStr($tdAttrs),
					$this->translateValue($config['inputEncoding'], $value));
				++$colIndex;
			}
			$table .= '</tr>';
			++$rowIndex;
		}
		
		$table .= '</tbody>';
			
		$table .= '</table>';

		return $table;
	}
	
	function modifyData($data, $columns)
	{
		$virtualColumns = $this->getVirtualColumns($columns);
		$formattedColumns = $this->getFormattedColumns($columns);
		$hiddenColumns = $this->getHiddenColumns($columns);
		$indexOrder = $this->getColumnsIndex($columns);
		
		if (count($virtualColumns) == 0 && 
			count($formattedColumns) == 0 &&
			count($hiddenColumns) == 0 &&
			count($indexOrder) == 0)
		{ 
			return $data;
		}
		
		$modifyData = array();
		foreach ($data as $key => $dataItem)
		{
			$modifyItem = $dataItem;
			foreach ($virtualColumns as $vcKey => $virtualColumn)
			{
				$modifyItem[$vcKey] = AriSimpleTemplate::parse($virtualColumn['format'], $dataItem);
			}
			
			foreach ($formattedColumns as $fmKey => $formattedColumn)
			{
				$modifyItem[$fmKey] = AriSimpleTemplate::parse($formattedColumn['format'], $dataItem);
			}

			foreach ($hiddenColumns as $hdKey => $hiddenColumn)
			{
				unset($modifyItem[$hdKey]);
			}

			if ($indexOrder)
			{
				$sortedModifyItem = array();
				foreach ($indexOrder as $orderKey => $value)
				{
					$sortedModifyItem[$value] = $modifyItem[$value];
				}
				
				$modifyItem = $sortedModifyItem;
			}
			
			$modifyData[$key] = $modifyItem;
		}
		
		return $modifyData;
	}
	
	function getColumnsIndex($columns)
	{
		$hasPos = false;
		$indexOrder = array();
		$tempIndexOrder = array();
		$unindexed = array();
		foreach ($columns as $key => $column)
		{
			if ($column['hidden']) continue ;

			$index = $column['pos'];
			if ($index > -1)
			{
				$hasPos = true;
				if (!isset($tempIndexOrder[$index])) $tempIndexOrder[$index] = array();
				
				$tempIndexOrder[$index][] = $key;
			}
			else
			{
				$unindexed[] = $key;
			}
		}
		
		if (!$hasPos) return $indexOrder;
		
		ksort($tempIndexOrder);
		foreach ($tempIndexOrder as $orderItems)
		{
			foreach ($orderItems as $orderItem) 
			{
				$indexOrder[] = $orderItem;
			}
		}
		
		$indexOrder = array_merge($indexOrder, $unindexed);
		
		return $indexOrder;
	}
	
	function getHiddenColumns($columns)
	{
		$hiddenColumns = array();
		foreach ($columns as $key => $column)
		{
			if ($column['hidden']) $hiddenColumns[$key] = $column;
		}
		
		return $hiddenColumns;
	}
	
	function getFormattedColumns($columns)
	{
		$formattedColumns = array();
		foreach ($columns as $key => $column)
		{
			if ($column['format']) $formattedColumns[$key] = $column;
		}
		
		return $formattedColumns;
	}
	
	function getVirtualColumns($columns)
	{
		$virtualColumns = array();
		foreach ($columns as $key => $column)
		{
			if ($column['virtual']) $virtualColumns[$key] = $column;
		}
		
		return $virtualColumns;
	}
	
	function getConfig($attrs)
	{
		$config = array();
		
		foreach ($this->_defaultConfig as $key => $defValue)
		{
			$config[$key] = $this->getValueByDefSample($key, $attrs, $defValue);
		}
		
		$config['inputEncoding'] = strtoupper($config['inputEncoding']);

		return $config;
	}
	
	function translateValue($inputEncoding, $value)
	{
		if ($inputEncoding)
		{
			$value = AriString::translateParam($inputEncoding, AriGlobalPrefs::getDbCharset(), $value);
		}
		
		return $value;
	}
}

class AriSmartHtmlTableColumnsPlugin extends AriSmartContentPluginBase
{
	var $_childTag;
	var $_columns = array();
	
	function __construct($tag, $childTag, $paramKey)
	{
		$this->_childTag = $childTag;
		$this->setParametersKey($paramKey);

		parent::__construct($tag);
	}
	
	function getColumns()
	{
		$cols = array();
		foreach ($this->_columns as $colSet)
		{
			if (is_array($colSet))
			{
				foreach ($colSet as $id => $colDef)
				{
					$cols[$id] = $colDef;
				}
			}
		}
		
		return $cols;
	}
	
	function replaceCallback($attrs, $content = '')
	{
		$columnPlugin = new AriSmartHtmlTableColumnPlugin($this->_childTag, $this->getParametersKey());
		$columnPlugin->processContent(true, $content, $this->_processParams, $this->_page);
		$cols = $columnPlugin->getColumns();
		if (!empty($cols)) $this->_columns[] = $cols;
		
		return '';
	}
}

class AriSmartHtmlTableColumnPlugin extends AriSmartContentPluginBase
{
	var $_columns;
	var $_idx = 0;
	var $_defColumn = array(
		'id' => null,
		'className' => null,
		'headerClassName' => null,
		'attrWidth' => null,
		'width' => null,
		'alias' => null,
		'headCellTag' => 'th',
		'cellTag' => 'td',
		'virtual' => false,
		'hidden' => false,
		'pos' => -1,
		'format' => '');
	
	function __construct($tag, $paramKey)
	{
		$this->setParametersKey($paramKey);

		parent::__construct($tag);
	}
	
	function getColumns()
	{
		return $this->_columns;
	}
	
	function processContent($published, &$row, &$params, $page = 0)
	{
		$this->_idx = 0;
		
		return parent::processContent($published, $row, $params, $page);
	}
	
	function replaceCallback($attrs, $content = '')
	{
		$colDef = array();
		foreach ($this->_defColumn as $key => $value)
		{
			$colDef[$key] = AriUtils::parseValueBySample(AriUtils::getParam($attrs, $key, $value), !is_null($value) ? $value: '');
		}
		
		$availCellTags = array('td', 'th');
		if (!in_array(strtolower($colDef['headCellTag']), $availCellTags)) $colDef['headCellTag'] = 'th';
		if (!in_array(strtolower($colDef['cellTag']), $availCellTags)) $colDef['cellTag'] = 'td';

		$colFormat = new AriSmartContentSimpleContentPlugin('coltemplate', $this->getParametersKey());
		$colFormat->processContent($this->_published, $content, $this->_processParams, $this->_page);
		$colDef['format'] = @trim($colFormat->getContent());

		if ($colDef['virtual'] && !$colDef['id'])
		{
			$colDef['id'] = 'VC_' . $this->_idx;
			++$this->_idx;
		}

		if ($colDef['id'] || $colDef['id'] == '0')
		{
			$this->_columns[$colDef['id']] = $colDef;
		}
		
		return '';
	}
}

//{aricsvtablecolumnspan row="0,2,4" startIndex="0" endIndex="4"}
class AriSmartHtmlTableColumnSpanPlugin extends AriSmartContentPluginBase
{
	var $_def = array();
	
	function __construct($tag, $paramKey)
	{
		$this->setParametersKey($paramKey);

		parent::__construct($tag);
	}

	function getDefinition()
	{
		return $this->_normalizeDefinition($this->_def);
	}
	
	function _normalizeDefinition($def)
	{
		$nDef = array();
		
		foreach ($def as $rowIndex => $rowData)
		{
			$nDef[$rowIndex] = array();
			foreach ($rowData as $cellData)
			{
				$startIndex = $cellData['startIndex'];
				$nDef[$rowIndex][$startIndex] = $cellData;
			}
		}
		
		return $nDef;
	}
	
	function replaceCallback($attrs, $content = '')
	{
		$row = AriUtils::getParam($attrs, 'row', '');
		$row = explode(',', $row);
		$rows = array();
		foreach ($row as $rowIndex)
		{
			$index = 0;
			if ($rowIndex !== '0')
			{
				$index = @intval($rowIndex, 10);
				if ($index != $rowIndex) continue;
			}
			
			$rows[] = $index;
		}
		
		if (count($rows) == 0) return '';
		
		$startIndex = AriUtils::getParam($attrs, 'startIndex', 0);
		$endIndex = AriUtils::getParam($attrs, 'endIndex', 0);
		
		$indexInfo = array('startIndex' => $startIndex, 'endIndex' => $endIndex);
		foreach ($rows as $rowIndex)
		{
			if (!array_key_exists($rowIndex, $this->_def)) $this->_def[$rowIndex] = array();

			$this->_def[$rowIndex][] = $indexInfo;
		}

		return '';
	}
}
?>