<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Document.DocumentHelper');
AriKernel::import('File.FileManager');
AriKernel::import('Controllers.AriSmartContent.PluginController');

class AriLoaderPluginBase extends AriObject
{
	var $_systemName = 'sys';
	var $_paramKey = null;
	var $_params = null;
	
	function setParametersKey($key)
	{
		$this->_paramKey = $key;
	}
	
	function getParametersKey()
	{
		$paramKey = $this->_paramKey;
		if (is_null($paramKey))
		{
			$paramKey = get_class($this);
		}
		
		return $paramKey;
	}
	
	function getParameters()
	{
		if (!$this->_params)
		{
			$pc = new AriSmartContentPluginController();
			$this->_params = $pc->call('getClientPluginParams', 
				$this->getParametersKey(), 
				AriConstantsManager::getVar('Type.System', AriSmartContentPluginControllerConstants::getClassName()));
		}
		
		return $this->_params;
	}
	
	function load($module, $version = null)
	{
		static $loadedModule = array();

		$jsPath = $this->getRootPath();
		$version = $this->findAppropriateVersion($jsPath, $version);
		if (empty($version)) return false;
		
		$sysName = $this->_systemName;
		$currentLoaded = isset($loadedModule[$sysName][$version]) ? $loadedModule[$sysName][$version] : null;
		if ($currentLoaded && in_array($module, $currentLoaded)) return true;
				
		$jsPath .= $version . '/';
		$filePath = $jsPath .  $this->getFileModuleName($module);
		
		if (!@file_exists($filePath)) return false;

		AriDocumentHelper::includeJsFile(AriUtils::absPath2Url($filePath));

		if (!isset($loadedModule[$sysName])) $loadedModule[$sysName] = array(); 
		if (!isset($loadedModule[$sysName][$version])) $loadedModule[$sysName][$version] = array();
		$loadedModule[$sysName][$version][] = $module;
		
		return true;
	}
	
	function getRootPath()
	{
		return dirname(__FILE__);
	}
	
	function getFileModuleName($module)
	{
		return $module;
	}
	
	function findAppropriateVersion($path, $version)
	{
		$folders = AriFileManager::getFolderList($path);
		if (empty($folders) || ($version && !in_array($version, $folders))) return null;
		
		if ($version && in_array($version, $folders)) return $version;
		
		$version = $folders[0];
		for ($i = 1, $cnt = count($folders); $i < $cnt; $i++)
		{
			if (version_compare($version, $folders[$i]) < 0)
			{
				$version = $folders[$i];
			}
		}
		
		return $version;
	}
}
?>