<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('I18N.I18N');
AriKernel::import('Components.Constants');

class AriSmartContentComponent extends AriObject
{
	function getClientCacheDir()
	{
		static $cacheDir = null;
		
		if (is_null($cacheDir))
		{
			$cacheDir = JPATH_ROOT . '/components/' . AriSmartContentComponent::getCodeName() . '/cache/files/';
		}
		
		return $cacheDir;
	}
	
	function getPluginFolder($plugin)
	{
		return AriSmartContentComponent::getPluginRootDir() . $plugin->Type . '/' . $plugin->Folder . '/';
	}
	
	function getPluginFilePath($plugin)
	{
		return AriSmartContentComponent::getPluginFolder($plugin) . $plugin->FileName;
	}

	function getPluginRootDir()
	{
		static $rootDir = null;
		
		if (is_null($rootDir))
			$rootDir = JPATH_ROOT . '/plugins/content/arismartcontent/';
		
		return $rootDir;
	}
	
	function getCodeName()
	{
		return 'com_arismartcontent';
	}
	
	function &instance()
	{
		static $instance;
		
		if (!isset($instance))
		{
			$c = __CLASS__;
			$instance = new $c();
		}
		
		return $instance;
	}
	
	function init($loadI18N = true)
	{		
		$codeName = $this->getCodeName();
		AriGlobalPrefs::setOption($codeName);
		AriGlobalPrefs::setConfigGroup($codeName);
		AriGlobalPrefs::setCacheDir(AriConstantsManager::getVar('CacheDir', $codeName));
		AriGlobalPrefs::setConfigTable(AriConstantsManager::getVar('ConfigTable', $codeName));
		AriGlobalPrefs::setFileTable(AriConstantsManager::getVar('FileTable', $codeName));
		AriGlobalPrefs::setPersistanceTable(AriConstantsManager::getVar('PersistanceTable', $codeName));
		AriGlobalPrefs::setDbCharset(AriConstantsManager::getVar('DbCharset', $codeName));
		AriGlobalPrefs::setEntityGroup(AriConstantsManager::getVar('EntityGroup', $codeName));
		
		if ($loadI18N)
		{
			$i18n =& $this->_getI18N();
			AriGlobalPrefs::setI18N($i18n);
		}
	}
	
	function &_getI18N()
	{
		$codeName = $this->getCodeName();
		$configKey = 'Config.BackendLang';
		$configKey = AriConstantsManager::getVar($configKey, $codeName);
		$fileGroup = 'FileGroup.BackendLang';
		$fileGroup = AriConstantsManager::getVar($fileGroup, $codeName);
		
		$i18n = $this->_createI18N($configKey, $fileGroup); 
		return $i18n;
	}
	
	function _createI18N($configKey, $fileGroup)
	{
		AriKernel::import('Cache.FileCache');
		AriKernel::import('Config.ConfigWrapper');
		
		$cacheDir = AriGlobalPrefs::getCacheDir();
		$useLang = AriConfigWrapper::getConfigKey($configKey, 'en'); 
		AriFileCache::cacheFile($cacheDir, $fileGroup, $useLang, 'xml');

		return new ArisI18N($cacheDir . $fileGroup, $useLang, $cacheDir . 'i18n/' . $fileGroup, $this->getCodeName(), 'en');
	}
}

class AriSmartContentConstants extends AriComponentConstants
{
	var $Option = null;
	var $PersistanceTable = '#__arisc_persistance';
	var $FileTable = '#__arisc_file';
	var $ConfigTable = '#__arisc_config';
	var $DbCharset = 'UTF-8';
	var $EntityGroup = '_AriSmartContentEntity';
	var $Config = array(
		'Version' => 'Version',
		'BackendLang' => 'BLang',
		'FrontendLang' => 'FLang');
	var $FileGroup = array(
		'BackendLang' => 'lbackend',
		'Thumb' => 'thumb',
		'CSS' => 'css',
		'Plugins' => 'plugins');
	
	function __construct()
	{		
		$this->Option = AriSmartContentComponent::getCodeName();
		$this->CacheDir = JPATH_ROOT . '/administrator/components/' . $this->Option . '/cache/files/';
		
		parent::__construct();
	}
}

new AriSmartContentConstants();
?>