<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Controllers.ControllerBase');
AriKernel::import('Controllers.FileController');
AriKernel::import('File.FileManager');
AriKernel::import('Cache.FileCache');

class AriSmartContentCssController extends AriControllerBase
{
	var $_fileController;
	var $_cacheController;
	
	function __construct($fileTable = null)
	{
		$this->_fileController = new AriFileController($fileTable);
		$this->_cacheController = new AriSmartContentCssCacheController($this);
	}
	
	function createCssInstance()
	{
		return AriEntityFactory::createInstance('AriSmartContentCssEntity', AriGlobalPrefs::getEntityGroup());
	}
	
	function getCssCount($filter = null)
	{
		$database =& JFactory::getDBO();
		
		$query = sprintf('SELECT C.CssId,C.FileId,C.Ordering,C.`Status`,F.ShortDescription FROM #__arisc_css C INNER JOIN %s F ON C.FileId = F.FileId WHERE F.`Group` = %s',
			$this->_fileController->_table,
			$database->Quote($this->getGroup()));
		$query = $this->_applyDbCountFilter($query, $filter);

		$database->setQuery($query);
		$count = $database->loadObjectList();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get css lists count.', E_USER_ERROR);
			return 0;
		}

		return $count;
	}
	
	function getCssList($filter = null, $fullLoad = false)
	{
		$database =& JFactory::getDBO();
		
		$query = sprintf('SELECT C.CssId,C.FileId,C.Ordering,C.`Status`,%sF.ShortDescription FROM #__arisc_css C INNER JOIN %s F ON C.FileId = F.FileId WHERE F.`Group` = %s',
			$fullLoad ? 'F.`Content`,' : '',
			$this->_fileController->_table,
			$database->Quote($this->getGroup()));
		$query = $this->_applyFilter($query, $filter);

		$database->setQuery($query);
		$cssList = $database->loadObjectList();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get css lists.', E_USER_ERROR);
			return null;
		}

		return $cssList;
	}
	
	function getCssFile($cssId)
	{
		$cssFile = $this->createCssInstance();
		$cssId = @intval($cssId, 10);
		if ($cssId > 0)
		{
			if (!$cssFile->load($cssId))
			{
				trigger_error('ARI: Couldnt get CSS file.', E_USER_ERROR);
				return null;
			}
		}
		
		return $cssFile;
	}
	
	function saveCssFile($cssId, $cssFields, $fileFields, $userId)
	{
		$errorMessage = 'ARI: Couldnt save CSS file.';
		$cssId = intval($cssId, 10);
		$isUpdate = ($cssId > 0);
		$row = $isUpdate
			? $this->getCssFile($cssId)
			: $this->createCssInstance();

		$fileFields['Group'] = $this->getGroup();
		$file = $this->_fileController->saveFile($row->FileId, $fileFields, $userId);
		if ($this->_isError(true, false) || !$file)
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}

		$row->Status = !empty($fields['Status']) ? 1 : 0;
		$cssFields['FileId'] = $file->FileId;
		if (!$row->bind($cssFields))
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}

		if (!$row->store())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}
		$row->File = $file;
		
		$this->_cacheController->deleteCacheFiles($row->CssId);
		$this->_cacheController->createCacheFile($row->CssId, $file->Content, $row->Status);

		return $row;
	}
	
	function deleteCssFiles($idList)
	{
		$database =& JFactory::getDBO();
		
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return true;
		
		$error = 'ARI: Couldnt delete css files.';
		
		$cssIdStr = join(',', $this->_quoteValues($idList));
		$query = 'SELECT FileId FROM #__arisc_css WHERE CssId IN (' . $cssIdStr . ')';
		$database->setQuery($query);
		$fileIdList = $database->loadResultArray();
		if ($database->getErrorNum())
		{
			trigger_error($error, E_USER_ERROR);
			return false;
		}
		
		$this->_fileController->deleteFile($fileIdList, $this->getGroup());
		if ($this->_isError(true, false))
		{
			trigger_error($error, E_USER_ERROR);
			return false;
		}

		$query = 'DELETE FROM #__arisc_css WHERE CssId IN (' . $cssIdStr . ')';
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error($error, E_USER_ERROR);
			return false;
		}
		
		$this->_cacheController->deleteCacheFiles($idList);
		
		return true;
	}

	function activateCssFile($idList)
	{
		$res = $this->setStatusCssFile(1, $idList);
		if ($this->_isError(true, false))
		{
			trigger_error('ARI: Couldnt activate CSS templates.', E_USER_ERROR);
			return false;
		}
		
		return $res;
	}
	
	function deactivateCssFile($idList)
	{
		$res = $this->setStatusCssFile(0, $idList);
		if ($this->_isError(true, false))
		{
			trigger_error('ARI: Couldnt deactivate CSS templates.', E_USER_ERROR);
			return false;
		}
		
		return $res;
	}
	
	function setStatusCssFile($status, $idList)
	{
		$database =& JFactory::getDBO();
		
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return true;
		
		$status = @intval($status, 10);
		$cssIdStr = join(',', $this->_quoteValues($idList));
		
		$query = sprintf('UPDATE #__arisc_css SET `Status` = %d WHERE CssId IN (%s)',
			$status,
			$cssIdStr);
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt change CSS template status.', E_USER_ERROR);
			return false;
		}
		
		$this->_cacheController->renameCacheFiles($idList, $status);
		
		return true;
	}
	
	function syncCssFiles()
	{
		return $this->_cacheController->syncCssFiles();
	}
	
	function getGroup()
	{
		return AriConstantsManager::getVar('FileGroup.CSS', AriSmartContentComponent::getCodeName());
	}
}

class AriSmartContentCssCacheController extends AriControllerBase
{
	var $_cssController;
	
	function __construct(&$cssController)
	{
		$this->_cssController =& $cssController;
	}
	
	function getCssFileName($cssId, $status)
	{
		return $cssId . ($status ? '.css' : '._cs');
	}
	
	function getCachePath()
	{
		return AriSmartContentComponent::getClientCacheDir() . AriConstantsManager::getVar('FileGroup.CSS', AriSmartContentComponent::getCodeName()) . '/';
	}
	
	function syncCssFiles()
	{
		$ret = true;
		$path = $this->getCachePath();
		$cssFileList = AriFileManager::getFileList($path, false, array('css', '_cs'));
		if (!empty($cssFileList))
		{
			foreach ($cssFileList as $cssFile)
			{
				@AriFileManager::deleteFile($path . $cssFile);
			}
		}
		
		$cssList = $this->_cssController->getCssList(null, true);
		if (!empty($cssList))
		{
			foreach ($cssList as $cssFile)
			{
				if (!$this->createCacheFile($cssFile->CssId, $cssFile->Content, $cssFile->Status))
				{
					$ret = false;
				}
			}
		}
		
		return $ret;
	}
	
	function createCacheFile($cssId, $content, $status)
	{
		return @AriFileCache::saveBinaryFile($content, $this->getCachePath() . $this->getCssFileName($cssId, $status));
	}
	
	function deleteCacheFiles($idList)
	{
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return ;
		
		$cachePath = $this->getCachePath();
		foreach ($idList as $cssId)
		{
			if (!$this->deleteCacheFile($cssId, 1))
			{
				$this->deleteCacheFile($cssId, 0);
			}
		}
	}
	
	function deleteCacheFile($cssId, $status = 1)
	{
		$path = $this->getCachePath() . $this->getCssFileName($cssId, $status);
		if (@file_exists($path))
		{
			return @AriFileManager::deleteFile($path);
		}
		
		return false;
	}
	
	function renameCacheFiles($idList, $status)
	{
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return ;
		
		$invStatus = (!$status ? 1 : 0);
		
		$cachePath = $this->getCachePath();
		foreach ($idList as $cssId)
		{
			$path = $cachePath . $this->getCssFileName($cssId, $invStatus);
			if (!@file_exists($path)) continue ;
			
			@AriFileManager::move($path, $cachePath . $this->getCssFileName($cssId, $status));
		}
	}
}
?>