<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Controllers.ControllerBase');
AriKernel::import('File.FileManager');
AriKernel::import('Controllers.AriSmartContent._Templates.PluginTemplates');

class AriSmartContentPluginControllerConstants extends AriClassConstants
{
	var $Status = array('Active' => 1, 'Inactive' => 2);
	var $Type = array('Content' => 'content', 'System' => 'system');
	var $ParamType = array(
		'Text' => 'text', 
		'LongText' => 'longtext', 
		'Integer' => 'int', 
		'Float' => 'float',
		'List' => 'list',
		'Folder' => 'folder');
	
	function getClassName()
	{
		return strtolower('AriSmartContentPluginControllerConstants');
	}
}

new AriSmartContentPluginControllerConstants();

class AriSmartContentPluginParams extends AriObject
{
	var $_params;

	function get($key, $defValue)
	{
		$value = $defValue;
		$param = AriUtils::getParam($this->_params, $key, null);
		if ($param)
		{
			$value = $param->Value;
		}
		
		return $value;
	}
	
	function bind($params)
	{
		$this->_params = $params;
	}
}

class AriSmartContentPluginController extends AriControllerBase
{
	var $_cacheController;
	
	function __construct()
	{
		$this->_cacheController = new AriSmartContentPluginCacheController($this);
	}
	
	function _applyPluginFilter($query, $filter)
	{
		if ($filter != null)
		{
			$filterFields = $filter->getConfigValue('filter');
			if (is_array($filterFields))
			{
				$needAnd = false;
				foreach ($filterFields as $key => $value)
				{
					if ($needAnd) $query .= ' AND ';
					$query .= sprintf(' `%s` = %s', $key, $value);
					
					$needAnd = true;
				}
			}
		}
		
		return $query;
	}
	
	function _fixPluginFilter($filter)
	{
		$database =& JFactory::getDBO();
		
		if ($filter != null)
		{
			$filterFields = $filter->getConfigValue('filter');
			$fixFilterFields = array();
			if (!empty($filterFields))
			{
				$statusList = AriConstantsManager::getVar('Status', AriSmartContentPluginControllerConstants::getClassName());
				$typeList = AriConstantsManager::getVar('Type', AriSmartContentPluginControllerConstants::getClassName());
				$applyFields = array('Status', 'Type');
				
				foreach ($filterFields as $key => $value)
				{
					if (is_null($value) || !in_array($key, $applyFields)) continue;
					
					switch ($key)
					{
						case 'Type':
							$value = $database->Quote($value);
							break;
						case 'Status':
							$value = intval($value, 10);
							break;
					}
					
					$fixFilterFields[$key] = $value;
				}
			}
			
			if (count($fixFilterFields) == 0) $fixFilterFields = null;
			
			$filter->setConfigValue('filter', $fixFilterFields);
		}
		
		return $filter;
	}
	
	function createPluginInstance()
	{
		return AriEntityFactory::createInstance('AriSmartContentPluginEntity', AriGlobalPrefs::getEntityGroup());
	}
	
	function createPluginParamInstance()
	{
		return AriEntityFactory::createInstance('AriSmartContentPluginParamEntity', AriGlobalPrefs::getEntityGroup());
	}

	function getPluginIdByName($name)
	{
		$database =& JFactory::getDBO();
		
		$query = sprintf('SELECT PluginId FROM #__arisc_plugins WHERE PluginName=%s LIMIT 0,1',
			$database->Quote($name));
		$database->setQuery($query);
		$pluginId = $database->loadResult();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldt get plugin id by name.', E_USER_ERROR);
			return 0;
		}

		return $pluginId;
	}
	
	function deletePlugins($idList, $dir, $strick = true)
	{
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return true;
		
		$complete = true;
		foreach ($idList as $id)
		{
			if (!$this->deletePlugin($id, $dir, $strick, false))
			{
				$complete = false;
			}
		}
		
		$this->syncPlugins();
		
		return $complete;
	}
	
	function deletePlugin($pluginId, $dir, $strick = true, $needSync = true)
	{
		$database =& JFactory::getDBO();
		
		$pluginId = @intval($pluginId, 10);
		
		$plugin = $this->getPlugin($pluginId, false);
		if (!$plugin) return false;

		$systemType = AriConstantsManager::getVar('Type.System', AriSmartContentPluginControllerConstants::getClassName());
		if ($strick && $plugin->Type == $systemType) return false;
		
		if (!empty($plugin->Folder)) AriFileManager::deleteFiles(AriFileManager::ensureEndWithSlash($dir) . $plugin->Type . '/' . $plugin->Folder . '/');		
		$query = sprintf('DELETE FROM P,PPD,PPDA,PPDV,PP' .
			' USING #__arisc_plugins P LEFT JOIN #__arisc_plugin_params_def PPD' .
			' 	ON P.PluginId = PPD.PluginId' .
			' LEFT JOIN #__arisc_plugin_params_def_attrs PPDA' .
			'	ON PPD.ParamId = PPDA.ParamId' .
			' LEFT JOIN #__arisc_plugin_params_def_values PPDV' .
			'	ON PPD.ParamId = PPDV.ParamId' .
			' LEFT JOIN #__arisc_plugin_params PP' .
			'	ON PPD.ParamId = PP.ParamId' .
			' WHERE P.PluginId = %d',
			$pluginId);

		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldt delete plugin.', E_USER_ERROR);
			return false;
		}
		
		if ($needSync) $this->syncPlugins();
		
		return true;
	}
	
	function createPlugin($manifest, $pluginId, $userId, $sync = true)
	{
		$database =& JFactory::getDBO();

		$errorMessage = 'ARI: Couldnt create plugin.';
		
		$statusList = AriConstantsManager::getVar('Status', AriSmartContentPluginControllerConstants::getClassName());
		$isExists = !empty($pluginId);
		$plugin = $this->getPlugin($pluginId, false);
		$plugin->bind($manifest);
		$now = ArisDate::getDbUTC();

		if ($isExists)
		{
			if ($plugin->ModifiedBy)
			{
				$plugin->Modified = $now;
				$plugin->ModifiedBy = $userId;
			}
			else
			{
				$plugin->Created = $now;
				$plugin->CreatedBy = $userId;
			}
			++$plugin->Revision;
		}
		else
		{
			$plugin->PluginName = AriUtils::getParam($manifest, 'Name', '');
			$plugin->Status = $statusList['Active'];
			$plugin->Folder = uniqid($this->getSystemNameByName($plugin->PluginName), false);
			$plugin->Created = $now;
			$plugin->CreatedBy = $userId;
			$plugin->Modified = null;
			$plugin->ModifiedBy = null;
			$plugin->Revision = 0;
		}

		if (!$plugin->store())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}

		if (!$this->createPluginParams($manifest['params'], $plugin))
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}
		
		if ($sync) $this->syncPlugins();

		return $plugin;
	}
	
	function createPluginParams($params, $plugin)
	{
		$database =& JFactory::getDBO();
		
		if (empty($params)) return true;

		$errorMessage = 'ARI: Couldnt create plugin params.';
		
		$pluginId = $plugin->PluginId;
		$revision = $plugin->Revision;
		$paramDefQueryPart = array();
		foreach ($params as $param)
		{
			$paramDefQueryPart[] = sprintf('(NULL,%s,%s,%s,%d,%s,%d)',
				$database->Quote($param['ParamName']),
				$database->Quote($param['DisplayName']),
				$database->Quote($param['Description']),
				$pluginId,
				$database->Quote($param['ParamType']),
				$revision);			
		}

		$query = 'INSERT INTO #__arisc_plugin_params_def (ParamId,ParamName,DisplayName,Description,PluginId,ParamType,Revision) VALUES' . join(',', $paramDefQueryPart) . 
			' ON DUPLICATE KEY UPDATE DisplayName=VALUES(DisplayName),Description=VALUES(Description),ParamType=VALUES(ParamType),Revision=VALUES(Revision)';
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return false;
		}
		
		$query = sprintf('DELETE FROM PPD,PP' .
			' USING #__arisc_plugin_params_def PPD LEFT JOIN #__arisc_plugin_params PP' .
			' 	ON PPD.ParamId = PP.ParamId' .
			' WHERE PPD.PluginId = %d AND PPD.Revision <> %d',
			$pluginId,
			$revision);
		$database->setQuery($query);
		$database->query();

		$query = sprintf('DELETE FROM PPDA,PPDV' .
			' USING #__arisc_plugin_params_def PPD LEFT JOIN #__arisc_plugin_params_def_attrs PPDA' .
			'	ON PPD.ParamId = PPDA.ParamId' .
			' LEFT JOIN #__arisc_plugin_params_def_values PPDV' .
			'	ON PPD.ParamId = PPDV.ParamId' .
			' WHERE PPD.PluginId = %d',
			$pluginId);
		$database->setQuery($query);
		$database->query();

		$query = sprintf('SELECT ParamId,ParamName FROM #__arisc_plugin_params_def WHERE PluginId = %d', $pluginId);
		$database->setQuery($query);
		$paramsDef = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return false;
		}
		
		$paramsDefHash = array();
		foreach ($paramsDef as $paramDef)
		{
			$paramsDefHash[$paramDef['ParamName']] = $paramDef['ParamId'];
		}
		
		$paramAttrQueryPart = array();
		$paramValueQueryPart = array();
		foreach ($params as $param)
		{
			$paramId = $paramsDefHash[$param['ParamName']];
			
			
			$value = $param['Value'];
			if (!is_array($value)) $value = array($value);
			
			foreach ($value as $valueItem)
			{
				$paramValueQueryPart[] = sprintf('(%d,%s)', $paramId, $database->Quote($valueItem));
			}
			
			$attrs = $param['Attrs'];
			foreach ($attrs as $attrName => $attrValue)
			{
				$paramAttrQueryPart[] = sprintf('(%d,%s,%s)', $paramId, $database->Quote($attrName), $database->Quote($attrValue));
			}
		}
		
		$query = 'INSERT INTO #__arisc_plugin_params_def_values (ParamId,ParamValue) VALUES' . join(',', $paramValueQueryPart);
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return false;
		}

		if (!empty($paramAttrQueryPart))
		{
			$query = 'INSERT INTO #__arisc_plugin_params_def_attrs (ParamId,AttrName,AttrValue) VALUES' . join(',', $paramAttrQueryPart);
			$database->setQuery($query);
			$database->query();
			if ($database->getErrorNum())
			{
				trigger_error($errorMessage, E_USER_ERROR);
				return false;
			}
		}
		
		return true;
	}
	
	function savePlugin($pluginId, $fields, $params, $userId)
	{
		$database =& JFactory::getDBO();
		
		$errorMessage = 'ARI: Couldnt save plugin.';
		
		$plugin = $this->getPlugin($pluginId);
		if ($this->_isError(true, false))
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}
		
		if (empty($plugin->PluginId)) return null;
		
		$statusList = AriConstantsManager::getVar('Status', AriSmartContentPluginControllerConstants::getClassName());
		$typeList = AriConstantsManager::getVar('Type', AriSmartContentPluginControllerConstants::getClassName());
		
		if ($plugin->Type != $typeList['System'])
		{
			$plugin->Status = !empty($fields['Status']) ? $statusList['Active'] : $statusList['Inactive'];
		}

		$plugin->Modified = ArisDate::getDbUTC();
		$plugin->ModifiedBy = $userId;
		
		if (!$plugin->store())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}
		
		$this->savePluginParams($pluginId, $params, $plugin->Params);

		$this->syncPlugins();
		
		return $plugin;
	}
	
	function savePluginParams($pluginId, $params, $defParams)
	{
		$database =& JFactory::getDBO();
		
		$pluginId = @intval($pluginId, 10);
		if ($pluginId < 1) return true;
		
		$errorMessage = 'ARI: Couldnt save plugin params.';
		
		$query = sprintf('DELETE FROM PP'.
			' USING #__arisc_plugin_params PP INNER JOIN #__arisc_plugin_params_def PPD' .
			'	ON PP.ParamId = PPD.ParamId' .
			' WHERE PPD.PluginId = %d',
			$pluginId);
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return false;
		}

		if (empty($defParams)) return true;

		$savedParams = array();
		foreach ($defParams as $name => $defParam)
		{
			$savedParams[$defParam->ParamId] = isset($params[$name]) ? $params[$name] : $defParam->Value; 
		}
		
		$query = 'INSERT INTO #__arisc_plugin_params (ParamId,ParamValue) VALUES';
		$queryParts = array();
		foreach ($savedParams as $id => $value)
		{
			if (!is_array($value)) $value = array($value);
			
			foreach ($value as $valueItem)
			{
				$queryParts[] = sprintf('(%d,%s)', $id, $database->Quote($valueItem));
			}
		}
		
		$query .= join(',', $queryParts);
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return false;
		}
	}
	
	function getPlugin($pluginId, $fullLoad = true)
	{
		$pluginId = @intval($pluginId, 10);
		
		$errorMessage = 'ARI: Couldnt get plugin.';
		$plugin = $this->createPluginInstance();
		if ($pluginId > 0)
		{
			if (!$plugin->load($pluginId))
			{
				trigger_error($errorMessage, E_USER_ERROR);
				return null;
			}
			
			$plugin->Params = array();
			if ($fullLoad)
			{
				$plugin->Params = $this->getPluginParams($pluginId);
			}
		}

		return $plugin;
	}
	
	function getPluginParams($pluginId)
	{
		$database =& JFactory::getDBO();

		$pluginId = @intval($pluginId, 10);
		$errorMessage = 'ARI: Couldnt get plugin parameters.';

		$params = array();
		$query = sprintf('SELECT PPD.ParamId,PPD.ParamType,PPD.Description,PPD.DisplayName,PPD.ParamName,PPDV.ParamValue' .
			' FROM #__arisc_plugin_params_def PPD LEFT JOIN #__arisc_plugin_params_def_values PPDV' .
			'	ON PPD.ParamId = PPDV.ParamId' .
			' WHERE PPD.PluginId = %d' .
			' ORDER BY PPD.ParamId',
				$pluginId);
		$database->setQuery($query);
		$defParams = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}

		$query = sprintf('SELECT PPD.ParamId,PPD.ParamName,PP.ParamValue' .
			' FROM #__arisc_plugin_params_def PPD LEFT JOIN #__arisc_plugin_params PP' .
			'	ON PPD.ParamId = PP.ParamId' .
			' WHERE PPD.PluginId = %d AND PP.ParamId IS NOT NULL' .
			' ORDER BY PPD.ParamId',
				$pluginId);
		$database->setQuery($query);
		$overrideParamsList = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}

		$overrideParams = array();
		if ($overrideParamsList)
		{
			foreach ($overrideParamsList as $overrideParam)
			{
				$paramName = $overrideParam['ParamName'];
				$value = $overrideParam['ParamValue'];
				if (!isset($overrideParams[$paramName]))
				{
					$overrideParams[$paramName] = $value;
				}
				else
				{
					if (!is_array($overrideParams[$paramName]))
					{
						$overrideParams[$paramName] = array($overrideParams[$paramName]);
					}
					
					$overrideParams[$paramName][] = $value;
				}
			}
		}

		if ($defParams)
		{
			$paramsAttrs = $this->getPluginParamsAttrs($pluginId);
			
			foreach ($defParams as $param)
			{
				$patamEntity = null;
				$value = $param['ParamValue'];
				$paramName = $param['ParamName'];
				if (isset($params[$paramName]))
				{
					$paramEntity = $params[$paramName];
					if (!is_array($paramEntity->Value))
					{
						$paramEntity->Value = array($paramEntity->Value);
					}
					
					$paramEntity->Value[] = $value;
				}
				else
				{
					$type = $param['ParamType'];
					$paramEntity = $this->createPluginParamInstance();
					$paramEntity->bind($param, AriUtils::getParam($paramsAttrs, $paramName, array()));
				}

				if (isset($overrideParams[$paramName])) $paramEntity->Value = $overrideParams[$paramName];
				$params[$paramName] = $paramEntity;
			}
		}
		
		return $params;
	}
	
	function getPluginParamsAttrs($pluginId)
	{
		$database =& JFactory::getDBO();
		
		$pluginId = @intval($pluginId, 10);
		if ($pluginId < 1) return null;
		
		$query = sprintf('SELECT PPD.ParamName,PPD.ParamId,PPDA.AttrName,PPDA.AttrValue'.
			' FROM #__arisc_plugins P INNER JOIN #__arisc_plugin_params_def PPD' .
			'	ON P.PluginId = PPD.PluginId' .
			' INNER JOIN #__arisc_plugin_params_def_attrs PPDA' .
			'	ON PPD.ParamId = PPDA.ParamId' .
			' WHERE P.PluginId = %d' .
			' ORDER BY PPD.ParamId',
			$pluginId);
		$database->setQuery($query);
		$paramList = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get plugin params attributes.', E_USER_ERROR);
			return null;
		}

		$params = array();
		if ($paramList)
		{
			$prevParamId = -1;
			foreach ($paramList as $param)
			{
				if ($prevParamId != $param['ParamId'])
				{
					$params[$param['ParamName']] = array();
				}
				
				$params[$param['ParamName']][$param['AttrName']] = $param['AttrValue'];
				
				$prevParamId = $param['ParamId'];
			}
		}
		
		return $params;
	}

	function getPluginCount($filter = null)
	{
		$database =& JFactory::getDBO();
		
		$filter = $this->_fixPluginFilter($filter);
		
		$query = 'SELECT COUNT(*) FROM #__arisc_plugins';
		$query = $this->_applyPluginFilter($query, $filter);
		$query = $this->_applyDbCountFilter($query, $filter);
		
		$database->setQuery($query);
		$count = $database->loadResult();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get plugins count.', E_USER_ERROR);
			return 0;
		}

		return $count;
	}
	
	function getPluginList($filter = null)
	{
		$database =& JFactory::getDBO();
		
		$filter = $this->_fixPluginFilter($filter);

		$query = 'SELECT PluginId,PluginName,`Type`,Ordering,`Status`,`Version`,`Folder`,`FileName` FROM #__arisc_plugins';		
		$query = $this->_applyPluginFilter($query, $filter);
		$query = $this->_applyFilter($query, $filter);

		$database->setQuery($query);
		$pluginList = $database->loadObjectList();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get plugins.', E_USER_ERROR);
			return null;
		}

		return $pluginList;
	}
	
	function getClientPluginParams($className, $type)
	{
		$params = new AriSmartContentPluginParams();
		
		$plugin = $this->getClientPlugin($className, $type);
		$params->bind(AriUtils::getParam($plugin, 'Params', null));

		return $params;
	}
	
	function getClientPluginAttrs($className, $type)
	{
		$attrs = null;
		$plugin = $this->getClientPlugin($className, $type);
		if (isset($plugin->PluginId))
		{
			$attrs = $this->getPluginParamsAttrs($plugin->PluginId);
			if ($this->_isError(true, false))
			{
				$attrs = null;
			}
		}
		
		return $attrs;
	}

	function getClientPlugin($className, $type)
	{
		$plugin = null;
		$plugins = $this->getClientPluginList();
		$className = strtolower($className);
		if (isset($plugins[$type][$className]))
		{
			$plugin = $plugins[$type][$className];
		}
		
		return $plugin;
	}
	
	function getClientPluginsData($useCache = true)
	{
		if ($useCache)
		{
			$pluginsData = $this->_cacheController->getCachedPlugins(true);
			if (!is_null($pluginsData))
			{
				return $pluginsData;
			}
		}

		$database =& JFactory::getDBO();
		$pluginsData = array('plugins' => array(), 'params' => array());
		
		$errorMessage = 'ARI: Couldnt get client plugins.';
		
		$filter = new AriDataFilter(
			array('startOffset' => null, 'limit' => null, 'sortField' => 'Ordering', 'dir' => 'asc'));
		
		$query = 'SET SESSION SQL_BIG_SELECTS=1';
		$database->setQuery($query);
		$database->query();
		if ($database->getErrorNum())
		{
			// Log error
		}

		//$query = 'SELECT * FROM #__arisc_plugins';		
		$query = 'SELECT PluginId,PluginName,Type,Folder,FileName,ClassName,Ordering,Status,Revision,Created,CreatedBy,Modified,ModifiedBy FROM #__arisc_plugins';
		$query = $this->_applyPluginFilter($query, $filter);
		$query = $this->_applyFilter($query, $filter);
		
		$database->setQuery($query);
		$plugins = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return $pluginsData;
		}
		
		// get params
		$query = 'SELECT PPD.PluginId,PPD.ParamId,PPD.ParamType,PPD.ParamName,PPDV.ParamValue'.
			' FROM #__arisc_plugins P INNER JOIN #__arisc_plugin_params_def PPD' .
		    '   ON P.PluginId = PPD.PluginId' .
			' LEFT JOIN #__arisc_plugin_params_def_values PPDV' .
			'	ON PPD.ParamId = PPDV.ParamId' .
			' ORDER BY PPD.PluginId ASC';
		$database->setQuery($query);
		$paramList = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return $pluginsData;
		}
		
		// user defined params
		$query = 'SELECT PPD.ParamId,PPD.ParamName,PP.ParamValue' .
			' FROM #__arisc_plugin_params_def PPD LEFT JOIN #__arisc_plugin_params PP' .
			'	ON PPD.ParamId = PP.ParamId' .
			' WHERE PP.ParamId IS NOT NULL' .
			' ORDER BY PPD.ParamId';
		$database->setQuery($query);
		$overrideParamsList = $database->loadAssocList();
		if ($database->getErrorNum())
		{
			trigger_error($errorMessage, E_USER_ERROR);
			return null;
		}
		
		$overrideParams = array();
		if ($overrideParamsList)
		{
			foreach ($overrideParamsList as $overrideParam)
			{
				$paramId = $overrideParam['ParamId'];
				$value = $overrideParam['ParamValue'];
				if (!isset($overrideParams[$paramId]))
				{
					$overrideParams[$paramId] = $value;
				}
				else
				{
					if (!is_array($overrideParams[$paramId]))
					{
						$overrideParams[$paramId] = array($overrideParams[$paramId]);
					}
					
					$overrideParams[$paramId][] = $value;
				}
			}
		}
		
		$params = array();
		if ($paramList)
		{
			$prevPluginId = -1;
			foreach ($paramList as $param)
			{
				$pluginId = $param['PluginId'];
				$paramName = $param['ParamName'];
				$paramId = $param['ParamId'];
				if ($prevPluginId != $pluginId)
				{
					$params[$pluginId] = array();
				}
				
				$paramEntity = null;
				$value = $param['ParamValue'];
				if (isset($params[$pluginId][$paramName]))
				{
					$paramEntity = $params[$pluginId][$paramName];
					if (!is_array($paramEntity->Value))
					{
						$paramEntity->Value = array($paramEntity->Value);
					}
							
					$paramEntity->Value[] = $value;
				}
				else
				{
					$type = $param['ParamType'];
					$paramEntity = $this->createPluginParamInstance();
					$paramEntity->bind($param, array());
				}
				
				if (isset($overrideParams[$paramId])) $paramEntity->Value = $overrideParams[$paramId];
				$params[$pluginId][$paramName] = $paramEntity;
				
				$prevPluginId = $pluginId;
			}
		}
		
		$pluginsData['plugins'] = $plugins;
		$pluginsData['params'] = $params;
		
		return $pluginsData;
	}
	
	function getClientPluginList($reload = false, $useCache = true)
	{
		static $plugins = null;
		
		if (!is_null($plugins) && !$reload)
		{
			return $plugins;
		}
		
		$pluginsData = $this->getClientPluginsData($useCache);
		$pluginList = $pluginsData['plugins'];
		$params = $pluginsData['params'];
		if ($pluginList)
		{
			foreach ($pluginList as $pluginItem)
			{
				$plugin = $this->createPluginInstance();
				$plugin->bind($pluginItem);
				
				if (empty($plugin->Type)) continue;
				
				$type = $plugin->Type;
				if (isset($params[$plugin->PluginId])) $plugin->Params = $params[$plugin->PluginId]; 
				if (!isset($plugins[$type])) $plugins[$type] = array();
				
				$plugins[$type][strtolower($plugin->ClassName)] = $plugin; 
			}
		}

		return $plugins;
	}
	
	function getSystemNameByName($name)
	{
		$systemName = str_replace(array(' ', '-'), array('_', '_'), strtolower($name));
		return $systemName;
	}

	function activatePlugins($idList)
	{
		$complete = $this->changePluginsStatus($idList, AriConstantsManager::getVar('Status.Active', AriSmartContentPluginControllerConstants::getClassName()));
		if ($this->_isError(true, false))
		{
			trigger_error('ARI: Couldnt activate plugins.', E_USER_ERROR);
			return false;
		}
		
		return $complete;
	}
	
	function deactivatePlugins($idList)
	{
		$complete = $this->changePluginsStatus($idList, AriConstantsManager::getVar('Status.Inactive', AriSmartContentPluginControllerConstants::getClassName()));
		if ($this->_isError(true, false))
		{
			trigger_error('ARI: Couldnt deactivate plugins.', E_USER_ERROR);
			return false;
		}
		
		return $complete;
	}
	
	function changePluginsStatus($idList, $status)
	{
		$idList = $this->_fixIdList($idList);
		if (empty($idList)) return true;
		
		$database =& JFactory::getDBO();
		
		$systemType = AriConstantsManager::getVar('Type.System', AriSmartContentPluginControllerConstants::getClassName());
		$status = intval($status);
		$query = sprintf('UPDATE #__arisc_plugins SET Status = %d WHERE PluginId IN (%s) AND Type <> %s', 
			$status, 
			join(',', $this->_quoteValues($idList)),
			$database->Quote($systemType));
		$database->setQuery($query);
		$database->query();

		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt change plugins status.', E_USER_ERROR);
			return false;
		}
		
		$this->syncPlugins();
		
		return true;
	}
	
	function syncPlugins()
	{
		return $this->_cacheController->syncPlugins();
	}
}

class AriSmartContentPluginCacheController extends AriControllerBase
{
	var $_pluginController;
	
	function __construct(&$pluginController)
	{
		$this->_pluginController =& $pluginController;
	}
	
	function getCachePath()
	{
		return AriGlobalPrefs::getCacheDir() . AriConstantsManager::getVar('FileGroup.Plugins', AriSmartContentComponent::getCodeName()) . '/';
	}
	
	function syncPlugins()
	{
		$clientPluginsData = $this->_pluginController->getClientPluginsData(false);		
		$content = sprintf(ARI_SC_CACHE_PLUGINS_TEMPLATE, str_replace('\'', '\\\'', serialize($clientPluginsData)));
		
		return @AriFileCache::saveBinaryFile($content, $this->getCachePath() . 'plugins.php');
	}
	
	function getCachedPlugins($needCaching = true)
	{
		$file = $this->getCachePath() . 'plugins.php';
		if (@!file_exists($file) && $needCaching)
		{
			$this->syncPlugins();
		}
		
		if (!file_exists($file)) return null;
		
		// hack
		$this->_pluginController->createPluginParamInstance();
		require_once $file;
		
		return isset($GLOBALS[ARI_ROOT_NAMESPACE][AriSmartContentComponent::getCodeName()]['ClientPlugins'])
			? $GLOBALS[ARI_ROOT_NAMESPACE][AriSmartContentComponent::getCodeName()]['ClientPlugins']
			: null;
	}
}
?>