<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Controllers.ControllerBase');
AriKernel::import('Controllers.AriSmartContent.PluginController');
AriKernel::import('File.FileManager');
AriKernel::import('Xml.SimpleXml');

class AriSmartContentPluginInstallerConstants extends AriClassConstants
{
	var $Tags = array(
		'Root' => 'ariplugin', 
		'Files' => 'files', 
		'File' => 'file',
		'Params' => 'params', 
		'Param' => 'param',
		'PluginClassName' => 'ClassName',
		'ParamListOption' => 'option');
	var $Attributes = array(
		'PluginClass' => 'class',
		'Folder' => 'folder',
		'ParamName' => 'name',
		'ParamDisplayName' => 'label',
		'ParamDescription' => 'description',
		'ParamValue' => 'default',
		'ParamType' => 'type',
		'ListOptionValue' => 'value',
		'PluginType' => 'group');
	
	function getClassName()
	{
		return strtolower('AriSmartContentPluginInstallerConstants');
	}
}

new AriSmartContentPluginInstallerConstants();

class AriSmartContentPluginInstaller extends AriControllerBase 
{
	function installFromDir($dir, $destDir, $userId, $overwrite = false, $sync = true)
	{
		if (empty($dir) || !@is_dir($dir)) return false;

		$dir = AriFileManager::ensureEndWithSlash($dir);
		$destDir = AriFileManager::ensureEndWithSlash($destDir);
		$manifestFile = $this->findAppropriateManifestFile($dir);
		if (empty($manifestFile)) return false;

		$manifest = $this->parseManifestFile($manifestFile, $dir);

		$pluginController = new AriSmartContentPluginController();
		$pluginId = $pluginController->call('getPluginIdByName', $manifest['Name']);
		if ($pluginId)
		{
			if (!$overwrite)
			{
				return false;
			}
		}

		$plugin = $pluginController->call('createPlugin', $manifest, $pluginId, $userId, $sync);

		if (empty($plugin) || empty($plugin->PluginId)) return false;

		$files = $manifest['files'];
		if (!empty($files))
		{			
			$destDir .= $manifest['Type'] . '/' . $plugin->Folder . '/';
			if (!@file_exists($destDir))
			{
				AriFileManager::createFolder($destDir, 0777);
			}

			foreach ($files as $file)
			{
				$destFile = $destDir . $file;
				AriFileManager::ensureDirExists($file, $destDir);
				
				if (@file_exists($destFile)) @AriFileManager::deleteFile($destFile);
				//@AriFileManager::move($dir . $file, $destFile);
				@AriFileManager::copy($dir . $file, $destFile);
			}
		}
		
		return true;
	}
	
	function parseManifestFile($file, $dir, $simpleMode = false)
	{
		$info = array();
		
		$manifestXml = new AriSimpleXML();
		$manifestXml->loadFile($file);

		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);
		
		$childs = $manifestXml->document->children();
		foreach ($childs as $child)
		{
			$name = $child->name();
			$value = null;
			switch ($name)
			{
				case $tags['Files']:
					if (!$simpleMode)
					{
						$value = $this->parseFilesNode($child, $dir);
						$pluginInfo  = $this->getPluginClassName($child);
						$info['FileName'] = $pluginInfo['FileName'];
						$info['ClassName'] = $pluginInfo['ClassName'];
					}
					break;
					
				case $tags['Params']:
					if (!$simpleMode)
					{
						$value = $this->parseParamsNode($child);
					}
					break;
					
				default:
					$subChilds = $child->children();
					
					if (empty($subChilds)) 
					{
						if (!$simpleMode || $name != $attrs['ParamDescription'])
						{
							$name = ucfirst($name);
							$value = $child->data();
						}
					}
			}
			
			if (!is_null($value)) $info[$name] = $value;
		}
		
		$pluginTypeList = AriConstantsManager::getVar('Type', AriSmartContentPluginControllerConstants::getClassName());
		$pluginType = $manifestXml->document->attributes($attrs['PluginType']);
		if (!in_array($pluginType, $pluginTypeList))
		{
			$pluginType = $pluginTypeList['Content'];
		}
		$info['Type'] = $pluginType;
		$info['Path'] = $dir;
		
		return $info;
	}

	function getPluginClassName($node)
	{
		$className = array('ClassName' => null, 'FileName' => null);

		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);
		
		$fileTag = $tags['File'];
		$files = $node->$fileTag;
		foreach ($files as $file)
		{
			$class = $file->attributes($attrs['PluginClass']);
			if (!empty($class) && strlen(trim($class)) > 0)
			{
				$className['ClassName'] = trim($class);
				$className['FileName'] = $file->data();
				break;
			}
		}
		
		return $className;
	}
	
	function parseFilesNode($node, $dir)
	{
		$files = array();

		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);

		$folderAttr = $attrs['Folder'];
		$fileTag = $tags['File'];
		$fileList = $node->$fileTag;
		foreach ($fileList as $file)
		{
			$folder = $file->attributes($folderAttr);
			if (!is_null($folder))
			{
				$folder = AriFileManager::ensureEndWithSlash($folder);
				$folderFiles = AriFileManager::getFileList($dir . $folder, true);
				if (!empty($folderFiles))
				{
					foreach ($folderFiles as $file)
					{
						$files[] = $folder . $file;
					}
				}
			}
			else
			{
				$files[] = $file->data();
			}
		}

		$files = array_unique($files);

		return $files;
	}
	
	function parseParamsNode($node)
	{
		$params = array();
		
		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);
		$typeList = AriConstantsManager::getVar('ParamType', AriSmartContentPluginControllerConstants::getClassName());		

		$paramTag = $tags['Param'];
		if (empty($node->$paramTag)) return $params;		

		$paramList = $node->$paramTag;
		foreach ($paramList as $param)
		{			
			$paramDef = array(
				'ParamName' => '', 
				'DisplayName' => '', 
				'Description' => '', 
				'ParamType' => $typeList['Text'], 
				'Value' => '', 
				'Attrs' => array());
			
			$paramAttrs = $param->attributes();
			if ($paramAttrs)
			{
				foreach ($paramAttrs as $attrName => $attrValue)
				{
					switch ($attrName)
					{
						case $attrs['ParamType']:
							if (in_array($attrValue, $typeList))
							{
								$paramDef['ParamType'] = $attrValue;
							}
							break;
							
						case $attrs['ParamName']:
							$paramDef['ParamName'] = $attrValue;
							break;
							
						case $attrs['ParamDisplayName']:
							$paramDef['DisplayName'] = $attrValue;
							break;

						case $attrs['ParamDescription']:
							$paramDef['Description'] = $attrValue;
							break;
							
						case $attrs['ParamValue']:
							$paramDef['Value'] = $attrValue;
							break;
							
						default:
							$paramDef['Attrs'][$attrName] = $attrValue;
					}
				}
			}
			
			if ($paramDef['ParamType'] == $typeList['List'])
			{
				$optionAttrs = $this->parseParamListNode($param);
				if (!empty($optionAttrs))
				{
					$paramDef['Attrs'] = array_merge($paramDef['Attrs'], $optionAttrs);
				}
			}
			
			if ($paramDef['ParamName']) $params[$paramDef['ParamName']] = $paramDef;
		}
		
		return $params;
	}
	
	function parseParamListNode($node)
	{
		$listAttrs = array();
		
		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);
		
		$optionTag = $tags['ParamListOption'];
		if (empty($node) || empty($node)) return $listAttrs;

		$valueAttr = $attrs['ListOptionValue'];
		$optionList = $node->$optionTag;
		$i = 0;
		foreach ($optionList as $option)
		{
			$value = $option->attributes($valueAttr);
			$text = $option->data();

			if (is_null($text) || $text == '') $text = $value;
			else if (is_null($value)) $value = $text;
			
			$listAttrs['value_' . $i] = $value;
			$listAttrs['option_' . $i] = $text;
			++$i;
		}
		
		return $listAttrs;
	}
	
	function findAppropriateManifestFile($dir)
	{
		$file = null;
		if (empty($dir) || !@is_dir($dir)) return $file;

		$xmlFiles = AriFileManager::getFileList($dir, false, array('xml'));
		$dir = AriFileManager::ensureEndWithSlash($dir);
		foreach ($xmlFiles as $xmlFile)
		{
			if ($this->isManifestFile($dir . $xmlFile))
			{
				$file = $dir . $xmlFile;
				break;
			}
		}
		
		return $file;
	}
	
	function isManifestFile($file)
	{
		$isManifest = false;
		if (empty($file) || !@file_exists($file)) return $isManifest;
		
		$ns = AriSmartContentPluginInstallerConstants::getClassName();
		$tags = AriConstantsManager::getVar('Tags', $ns);
		$attrs = AriConstantsManager::getVar('Attributes', $ns);
		$filesTag = $tags['Files'];
		$fileTag = $tags['File'];

		$xml = new AriSimpleXML();
		if (!@$xml->loadFile($file) || 
			empty($xml->document) || 
			$xml->document->name() != $tags['Root'] ||
			empty($xml->document->$filesTag))
		{
			return $isManifest;
		}

		$files = $xml->document->$filesTag;
		if (empty($files[0]->$fileTag)) return $isManifest;
		
		$files = $files[0]->$fileTag;
		if (!is_array($files)) $files = array($files);
		
		foreach ($files as $file)
		{
			$class = $file->attributes($attrs['PluginClass']);
			if (!empty($class) && strlen(trim($class)) > 0)
			{
				$isManifest = true;
				break;
			}
		}
		
		return $isManifest;
	}
	
	function findPlugingsInDir($dir, $simpleMode = true, $filter = null)
	{
		$plugins = $this->_findPlugingsInDir($dir, $simpleMode, $filter);
		$totalCnt = count($plugins);

		if ($filter && $totalCnt > 0)
		{
			$firstRow = $plugins[0];
			$sort = $filter->getConfigValue('sortField');
			if (!is_null($sort) && isset($firstRow[$sort]))
			{
				$dir = $filter->getConfigValue('sortDirection');
				$dir = !is_null($dir) ? strtolower($dir) : 'asc';
				$plugins = AriUtils::sortAssocArray($plugins, $sort, $dir);
			}

			$filter->fixFilter($totalCnt);
			$offset = $filter->getConfigValue('startOffset');
			$limit = $filter->getConfigValue('limit');
			
			if (!is_null($offset))
			{
				$plugins = array_slice($plugins, $offset, $limit);
			}
		}

		return array('totalCnt' => $totalCnt, 'plugins' => $plugins);
	}
	
	function _findPlugingsInDir($dir, $simpleMode = true)
	{
		$plugins = array();
		if (empty($dir) || !@file_exists($dir) || !@is_dir($dir)) return $plugins;

		$plgManifestFile = $this->findAppropriateManifestFile($dir);
		if (!is_null($plgManifestFile))
		{
			$plugins[] = $this->parseManifestFile($plgManifestFile, $dir, $simpleMode);
		}
		else
		{
			$subDirs = AriFileManager::getFolderList($dir, false, true);
			if (!empty($subDirs))
			{
				foreach ($subDirs as $subDir)
				{
					$subDirPlugins = $this->_findPlugingsInDir($subDir, $simpleMode);
					if (count($subDirPlugins) > 0)
					{
						$plugins = array_merge($plugins, $subDirPlugins);
					}
				}
			}
		}
		
		return $plugins;
	}
}
?>