<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

class AriControllerBase extends AriObject
{
	function call($method)
	{
		$numArgs = func_num_args();
		$args = func_get_args();
		if ($numArgs > 0)
		{
			array_shift($args);
		}
		
		set_error_handler(array(&$this, 'errorHandler'));

		$retVal = call_user_func_array(array(&$this, $method), $args);

		restore_error_handler();
		
		$this->_isError();
		
		return $retVal;
	}
	
	function _raiseError($error)
	{
		trigger_error($error->error, E_USER_ERROR);
	}

	function _applyFilter($query, $filter)
	{
		if ($filter)
		{
			$query .= $this->_getOrder($filter);
			$query .= $this->_getLimit($filter->getConfigValue('startOffset'), $filter->getConfigValue('limit'));
		}

		return $query;
	}
	
	function _applyDbCountFilter($query, $filter)
	{
		if ($filter)
		{
			$filter = $this->_getDbCountFilter($filter);
			$query = $this->_applyFilter($query, $filter);
		}

		return $query;
	}
	
	function _getFilter($filterInfo)
	{
		$database =& JFactory::getDBO();
		
		$filter = '';
		if (!empty($filterInfo) && is_array($filterInfo))
		{
			$filterParts = array();
			foreach ($filterInfo as $field => $value)
			{
				$filterParts[] = sprintf('%s = %s',
					$field, 
					$database->Quote($value));
			}
			$filter = join(' AND ', $filterParts);
		}
		
		return $filter;
	}
	
	function _getOrder($sortInfo)
	{
		$query = '';
		if (!empty($sortInfo))
		{
			$sortField = $sortInfo->getConfigValue('sortField');
			if (!empty($sortField))
			{
				$query = sprintf(' ORDER BY %s %s ', $sortField, $sortInfo->getConfigValue('sortDirection'));
				$secondarySorting = $sortInfo->getConfigValue('secondarySorting');

				if ($secondarySorting)
				{
					foreach ($secondarySorting as $sortingItem)
					{
						$sortField = AriUtils::getParam($sortingItem, 'sortField', null);
						if (empty($sortField)) continue;
						$sortDir = strtolower(AriUtils::getParam($sortingItem, 'sortDirection', ''));
						if ($sortDir && $sortDir != 'asc' && $sortDir != 'desc') $sortDir = '';
						
						$query .= sprintf(',%s %s', $sortField, $sortDir);
					}
				}
			}
		}
		
		return $query;
	}
	
	function _getLimit($limitStart, $limit)
	{
		$query = '';
		if (!is_null($limitStart))
		{
			$query .= ' LIMIT ' . intval($limitStart);
			$query .= ',' . (!is_null($limit) ? intval($limit) : '18446744073709551615');
		}
		
		return $query;
	}
	
	function _fixIdList($idList)
	{
		if (empty($idList))
			return array();

		if (!is_array($idList))
		{
			$idList = array($idList);
		}
		
		return $idList;
	}
	
	function _quoteValues($arr)
	{
		$database =& JFactory::getDBO();
		
		if (!empty($arr))
		{
			foreach ($arr as $key => $value)
				$arr[$key] = $database->Quote($value);
		}
		
		return $arr;
	}
	
	function _normalizeValue($val)
	{
		$database =& JFactory::getDBO();
		
		return $val === null ? 'NULL' : $database->Quote($val);
	}
	
	// predicateFields array(fieldName => value, ..)
	function _getRecordCount($tableName, $predicateFields = array())
	{
		$database =& JFactory::getDBO();
		
		$query = 'SELECT COUNT(*) FROM ' . $tableName;
		if (!empty($predicateFields))
		{
			$query .= ' WHERE ';
			$predQuery = array();
			foreach ($predicateFields as $field => $value)
			{
				 $predQuery[] = sprintf('`%s` = %s',
				 	$field,
				 	$database->Quote($value)); 
			}
			
			$query .= join($predQuery, ' AND ');
		}

		$database->setQuery($query);
		$count = $database->loadResult();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt get count.', E_USER_ERROR);
			return 0;
		}
		
		return $count;
	}

	function _isUniqueField($tableName, $field, $value, $keyField = null, $key = null)
	{
		$database =& JFactory::getDBO();
		
		$query = sprintf('SELECT COUNT(%s) FROM %s WHERE %s = %s', $field, $tableName, $field, $database->Quote($value));
		if (!empty($keyField) && $key != null)
		{
			$query .= sprintf(' AND %s <> %s', $keyField, $database->Quote($key));
		}
		$database->setQuery($query);
		$count = $database->loadResult();
		if ($database->getErrorNum())
		{
			trigger_error('ARI: Couldnt check unique field.', E_USER_ERROR);
			return false;
		}
		
		return $count < 1;
	}

	function _getDbCountFilter($filter)
	{
		if ($filter)
		{
			$filter = clone($filter);
			$filter->setConfigValue('startOffset', null);
			$filter->setConfigValue('limit', null);
			$filter->setConfigValue('sortField', null);
		}
		
		return $filter;
	}
	
	function _getDbCount($query, $filter)
	{
		
	}
}

AriKernel::import('Data.DataFilter');
?>