<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Web.Page.Specific.AdminPageBase');
AriKernel::import('File.FileManager');
AriKernel::import('Controllers.AriSmartContent.PluginController');
AriKernel::import('Joomla.JoomlaHelper');
AriKernel::import('Install.Installer');

jimport('joomla.client.helper');

class diagnosticsAriPage extends AriAdminPageBase
{
	function _init()
	{
		$ftp =& JClientHelper::setCredentialsFromRequest('ftp');
		$this->addVar('ftp', $ftp);
	}
	
	function execute()
	{
		$folders = $this->getFolders();
		$plugins = $this->getPlugins();
		$isPluginInstalled = AriJoomlaHelper::isPluginInstalled('arismartcontent', 'system');
		$isModuleInstalled = AriJoomlaHelper::isModuleInstalled('mod_arismartcontent');
		$isFoldersCorrect = $this->isFoldersCorrect($folders);
		$requireFtp = AriJoomlaHelper::isRequireFtp();
		$activeTab = @intval(AriRequest::getParam('hidDiagActiveTab', 0), 10);
		
		$this->setResTitle('Label.Diagnostics');
		
		$this->addVar('folders', $folders);
		$this->addVar('plugins', $plugins);
		$this->addVar('isPluginInstalled', $isPluginInstalled);
		$this->addVar('isModuleInstalled', $isModuleInstalled);
		$this->addVar('isFoldersCorrect', $isFoldersCorrect);
		$this->addVar('requireFtp', $requireFtp);
		$this->addVar('activeTab', $activeTab);

		parent::execute();
	}
	
	function isFoldersCorrect($folders)
	{
		foreach ($folders as $folder)
		{
			if (!$folder['exists'] || !$folder['writable']) return false;
		}
		
		return true;
	}
	
	function getFolders()
	{
		$foldersInfo = array();
		$typeList = AriConstantsManager::getVar('Type', AriSmartContentPluginControllerConstants::getClassName());

		$basePath = JPATH_ROOT . '/administrator/components/com_arismartcontent/cache/files';
		$folders = AriFileManager::getFolderList($basePath, true, true);
		
		$pluginDir = 'plugins';
		$folders[] = JPATH_ROOT . '/' . $pluginDir . '/content/arismartcontent';
		
		foreach ($typeList as $type)
		{
			$folders[] = JPATH_ROOT . '/' . $pluginDir . '/content/arismartcontent/' . $type;
		}

		foreach ($folders as $key => $folder)
		{
			$exists = @file_exists($folder);
			$foldersInfo[$folder] = array(
				'shortPath' => AriUtils::absPath2Relative($folder),
				'exists' => $exists,
				'writable' => $exists ? @is_writable($folder) : false
				);
		}
		
		return $foldersInfo;
	}
	
	function getPlugins()
	{
		$pc = new AriSmartContentPluginController();
		$plugins = $pc->call('getPluginList');
		
		return $plugins;
	}
	
	function _registerEventHandlers()
	{
		$this->_registerEventHandler('fixDirPermissions', 'clickFixDirPermissions');
		$this->_registerEventHandler('fixPlugin', 'clickFixPlugin');
		$this->_registerEventHandler('fixModule', 'clickFixModule');
		$this->_registerEventHandler('cleanPlugin', 'clickCleanPlugin');
		$this->_registerEventHandler('cleanModule', 'clickCleanModule');
	}
	
	function clickCleanPlugin()
	{
		$ret = AriJoomlaHelper::deletePlugin('arismartcontent');
		$this->setInfoMessage(AriWebHelper::translateResValue($ret ? 'Message.DiagActionSuccess' : 'Message.DiagActionFail'));
	}
	
	function clickCleanModule()
	{
		$ret = AriJoomlaHelper::deleteModule('mod_arismartcontent');
		$this->setInfoMessage(AriWebHelper::translateResValue($ret ? 'Message.DiagActionSuccess' : 'Message.DiagActionFail'));
	}
	
	function clickFixDirPermissions()
	{
		$folders = $this->getFolders();
		foreach ($folders as $path => $info)
		{
			if (!$info['exists'])
			{
				AriFileManager::ensureDirExists(AriFileManager::ensureEndWithSlash($info['shortPath']) . 'fake.txt', JPATH_ROOT, 0777);
			}
			else if (!$info['writable'])
			{
				AriFileManager::setPermissions($path, 0777);
			}
		}
		
		$folders = $this->getFolders();
		$isFolderCorrect = $this->isFoldersCorrect($folders);
		
		$this->setInfoMessage(AriWebHelper::translateResValue($isFolderCorrect ? 'Message.DiagActionSuccess' : 'Message.DiagActionFail'));
	}
	
	function clickFixPlugin()
	{
		jimport( 'joomla.installer.installer' );

		$installer = new JInstaller();

		$installer->setOverwrite(true);
		$ret = $installer->install(JPATH_ROOT . DS . 'administrator' . DS . 'components' . DS . 'com_arismartcontent' . DS . 'plugins' . DS . 'arismartcontent');

		$this->setInfoMessage(AriWebHelper::translateResValue($ret ? 'Message.DiagActionSuccess' : 'Message.DiagActionFail'));
	}
	
	function clickFixModule()
	{
		jimport( 'joomla.installer.installer' );

		$installer = new JInstaller();

		$installer->setOverwrite(true);
		$ret = $installer->install(JPATH_ROOT . DS . 'administrator' . DS . 'components' . DS . 'com_arismartcontent' . DS . 'modules' . DS . 'mod_arismartcontent');

		$this->setInfoMessage(AriWebHelper::translateResValue($ret ? 'Message.DiagActionSuccess' : 'Message.DiagActionFail'));
	}
}
?>