<?php
/*
 * ARI Colorbox
 *
 * @package		ARI Colorbox
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Module.Providers.PicasaProvider');
AriKernel::import('Module.Lightbox.Models.GalleryModel');
AriKernel::import('Colorbox.Models.Templates.SimpleGalleryTemplate');

class AriColorboxPicasaModel extends AriGalleryModel
{
	var $_prefix = 'AriColorbox';
	
	function execute($modelParams, $params, $templatePath)
	{
		if (!$this->checkCompatibility())
			return ;
		
		$picasaProvider = new AriPicasaProvider($modelParams, null, 'mod_aricolorbox');
		$picasaData = $this->normalizeData(
			$picasaProvider->getData($modelParams)
		);
		$cssClass = uniqid('acb', false);
		$jsOptions = AriUtils2::getParam($params, 'opt', array());
		
		if ($modelParams['type'] == 'customtext')
		{
			$cId = uniqid('ct_', false);
			$modelParams['customtemplate'] = '<div id="' . $cId . '" style="cursor: pointer;">' . $modelParams['customtemplate'] . '</div>';
			$document =& JFactory::getDocument();
			$document->addScriptDeclaration(sprintf('(window["jQueryACB"] || jQuery)(document).ready(function($){ $("#%1$s").click(function(event) { $("#%1$s").closest(".ari_lightbox_container").find("a[class^=\'%2$s\']").eq(0).click(); }); });',
				$cId,
				$cssClass));			
		}
		else if ($modelParams['type'] == 'picasaalbums')
		{
			$albumThumbs = array();
			$albumsPhotos = array();

			foreach ($picasaData['albums'] as $album)
			{
				$group = uniqid('cb');
				$cssClass = uniqid('acb', false);
				$albumData = $album['album'];
				$firstPhoto = $album['photo'][0];
				$img = $firstPhoto['image'];
				$thumb = !empty($albumData['thumb']['url']) ? $albumData['thumb'] : $firstPhoto['thumb'];
				$title = $albumData['Caption'] = $albumData['Title'] = AriUtils2::getParam($albumData, 'title', $firstPhoto['title']);

				$aAttrs = array('href' => $img['url'], 'title' => $firstPhoto['summary'] ? $firstPhoto['summary'] : $firstPhoto['title'], 'class' => $cssClass, 'rel' => $group);
				$imgAttrs = array('src' => $thumb['url'], 'border' => '0', 'alt' => $title, 'width' => $thumb['w'], 'height' => $thumb['h']);

				$albumData['colorboximage'] = sprintf('<a%1$s><img%2$s/></a>',
					AriHtmlHelper::getAttrStr($aAttrs),
					AriHtmlHelper::getAttrStr($imgAttrs));

				$albumThumbs[] = $albumData;
				
				$isFirst = true;
				foreach ($album['photo'] as $dataItem)
				{		
					if ($isFirst)
					{
						$isFirst = false;
						continue ;
					}
					
					$title = $dataItem['Title'] = ($dataItem['summary'] ? $dataItem['summary'] : $dataItem['title']);
					$phImg = $dataItem['image'];
					$phAncAttrs = array('href' => $phImg['url'], 'title' => $title, 'class' => $cssClass, 'rel' => $group);
					$dataItem['colorboximage'] = sprintf('<a%1$s></a>',
						AriHtmlHelper::getAttrStr($phAncAttrs)
					);
					$albumsPhotos[] = $dataItem;
				}
				
				$jsOptions['rel'] = $group;
				AriColorboxHelper::includeSpecificAssets('a.' . $cssClass, $jsOptions);
			}

			$showTitle = (bool)$modelParams['picasaalbums']['showTitle'];
			$rptParams = $modelParams['picasaalbums'];
			$rptParams['GalleryCaption'] = $showTitle ? '{$data:Caption}' : '';
			$repeater = new AriRepeaterWebControl(
				AriSimpleTemplate::parse(
					ACOLORBOX_SIMPLEGALLERYTEMPLATE,
					$rptParams
				), $albumThumbs);
			AriTemplate::display(
				$templatePath . 'picasaalbums.html.php', 
				array(
					'repeater' => $repeater,
					'photos' => $albumsPhotos
				)
			);
			return ;
		}
		
		AriColorboxHelper::includeSpecificAssets('a.' . $cssClass, $jsOptions);
		
		parent::execute(
			$this->getData(
				$picasaData,
				$modelParams, 
				$params,
				$cssClass), 
			array(
				'simpleGallery' => ACOLORBOX_SIMPLEGALLERYTEMPLATE,
				'singleGallery' => ACOLORBOX_SINGLEIMAGEGALLERYTEMPLATE,
				'hiddenItems' => ACOLORBOX_HIDDENITEMSTEMPLATE),
			$modelParams,
			$params,
			$templatePath);
	}
	
	function normalizeData($picasaData)
	{
		if (empty($picasaData))
			return $picasaData;
			
		if (!array_key_exists('albums', $picasaData))
		{
			$picasaData['albums'] = array(
				0 => array(
					'album' => array(),
					'photo' => $picasaData
				)
			);
		}
		
		return $picasaData;
	}
	
	function getData($picasaData, $modelParams, $params, $groupCssClass)
	{
		$data = array();
		if (empty($picasaData))
			return $data;
			
		$showCaption = (bool)AriUtils2::getParam($modelParams, 'showCaption');
		$captionTemplate = $showCaption ? AriUtils2::getParam($modelParams, 'caption', '{$Title}') : '';
		$needParseTemplate = $showCaption && strpos($captionTemplate, '{$') !== false;
		if (!$needParseTemplate && $captionTemplate)
			$captionTemplate = str_replace('"', '&qout;', $captionTemplate);

		$group = $params['opt']['rel'];
		foreach ($picasaData['albums'] as $album)
		{
			foreach ($album['photo'] as $key => $value)
			{
				$dataItem = $value;

				$dataItem['Title'] = ($dataItem['summary'] ? $dataItem['summary'] : $dataItem['title']);
				$title = $needParseTemplate
					? str_replace('"', '&qout;', AriSimpleTemplate::parse($captionTemplate, $dataItem, true))
					: $captionTemplate;
				
				$img = $dataItem['image'];
				$thumb = $dataItem['thumb'];
				$aAttrs = array('href' => $img['url'], 'title' => $title, 'class' => $groupCssClass, 'rel' => $group);
				$imgAttrs = array('src' => $thumb['url'], 'border' => '0', 'alt' => $title, 'width' => $thumb['w'], 'height' => $thumb['h']);
	
				$dataItem['colorboximage'] = sprintf('<a%1$s><img%2$s/></a>',
						AriHtmlHelper::getAttrStr($aAttrs),
						AriHtmlHelper::getAttrStr($imgAttrs));
						
				$data[] = $dataItem;
			}
		}

		return $data;
	}
	
	function checkCompatibility()
	{
		if (version_compare(PHP_VERSION, '5.0.0') >= 0)
			return true;

		echo '<b style="color: red;">Picasa provider requires PHP v. 5.0.0 or above.</b>';
		
		return false;
	}
}
?>