<?php
/*
 * ARI Colorbox
 *
 * @package		ARI Colorbox
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Document.DocumentHelper');
jimport('joomla.filter.filterinput');

class AriColorboxHelper
{
	function includeAssets($loadJQuery, $noConflict, $jQueryVer = '1.4.4', $theme, $options = array(), $params = null)
	{
		static $loaded;

		if ($loaded)
			return ;

		$baseUri = JURI::root(true) . '/modules/mod_aricolorbox/includes/js/';
		if ($loadJQuery) 
		{
			$loadJQueryMethod = $params->get('loadJQueryMethod', 'google_cdn');

			if ($loadJQueryMethod == 'local')
			{
				AriDocumentHelper::includeJsFile($baseUri . 'jquery.min.js');
			}
			else
			{
				AriDocumentHelper::includeJsFile('http://ajax.googleapis.com/ajax/libs/jquery/' . $jQueryVer . '/jquery.min.js');			
			}

			if ($noConflict)
			{
				AriDocumentHelper::includeJsFile($baseUri . 'jquery.noconflict.js');
			}
		}
			
		AriDocumentHelper::includeJsFile($baseUri . 'jquery.colorbox-min.js');
		AriDocumentHelper::includeCssFile($baseUri . 'colorbox.css');
		AriDocumentHelper::includeCustomHeadTag(sprintf('<!--[if lt IE 7]><link rel="stylesheet" href="%s" type="text/css" /><![endif]-->',
			$baseUri . 'colorbox.ie6.css'));
		
		$theme = JFilterInput::clean($theme, 'WORD');
		AriDocumentHelper::includeCssFile($baseUri . 'themes/' . $theme . '/colorbox.css');

		$loaded = true;
	}
	
	function includeSpecificAssets($cssSelector, $options)
	{
		$jsOptions = AriColorboxHelper::getJsOptions($options);
		$document =& JFactory::getDocument();
		$document->addScriptDeclaration(sprintf('(window["jQueryACB"] || jQuery)(document).ready(function($){ $("%1$s").colorbox(%2$s); });',
			$cssSelector,
			!empty($jsOptions) ? AriJSONHelper::encode($jsOptions) : ''));
	}
	
	function activateAutoShow($group, $autoShow = true, $uniqueKey = false)
	{
		if ($autoShow != 'first')
			$autoShow = AriUtils2::parseValueBySample($autoShow, false);
		
		if (!$autoShow)
			return ;

		static $loaded;
		
		if ($loaded)
			return ;
			
		if ('first' === $autoShow && !empty($uniqueKey))
		{
			$cookieKey = 'acbox_' . $uniqueKey;
			if (!empty($_COOKIE[$cookieKey]))
				return ;

			setcookie($cookieKey, '1', time() + 365 * 24 * 60 * 60);
		}
		
		$document =& JFactory::getDocument();
		$document->addScriptDeclaration(sprintf('(window["jQueryACB"] || jQuery)(document).ready(function($){ $("a[rel=\'%1$s\']").eq(0).click(); });',
			$group));

		$loaded = true;
	}

	function getJsOptions($overrideOptions = array())
	{
		$defOptions = array(
			'transition' => 'elastic',
			'href' => '',
			'rel' => '',
			'inline' => false,
			'iframe' => false,
			'speed' => 350,
			'width' => '',
			'height' => '',
			'innerWidth' => '',
			'innerHeight' => '',
			'initialWidth' => 300,
			'initialHeight' => 100,
			'maxWidth' => '',
			'maxHeight' => '',
			'scalePhotos' => true,
			'scrolling' => true,
			'opacity' => 0.85,
			'preloading' => true,
			'overlayClose' => true,
			'slideshow' => false,
			'slideshowSpeed' => 2500,
			'slideshowAuto' => true,
			'slideshowStart' => 'start slideshow',
			'slideshowStop' => 'stop slideshow',
			'current' => '{current} of {total}',
			'previous' => 'previous',
			'next' => 'next',
			'close' => 'close',
			'loop' => true
		);

		$jsOptions = AriParametersHelper::getUniqueOverrideParameters($defOptions, $overrideOptions);

		return $jsOptions;
	}
	
	function executeModel($model, $params, $templatePath)
	{
		$modelParams = isset($params[$model]) ? $params[$model] : null;
		$model = ucfirst(JFilterInput::clean($model, 'WORD'));
		if (empty($model))
		{
			AriColorboxHelper::executeErrorModel($params, $templatePath);
			return ;
		}
		else
		{
			$modelName = 'Colorbox' . $model . 'Model';
			$modelPath = dirname(__FILE__) . DS . 'Models' . DS . 'class.' . $modelName . '.php';

			if (!@file_exists($modelPath))
			{
				AriColorboxHelper::executeErrorModel($params, $templatePath);
				return ;
			}
			else
			{
				AriKernel::import('Colorbox.Models.' . $modelName);

				$modelClass = 'Ari' . $modelName;
				$oModel = new $modelClass();
				$oModel->execute($modelParams, $params, $templatePath);
			}
		}
	}
	
	function executeErrorModel($params, $templatePath)
	{
		AriKernel::import('Colorbox.Models.ColorboxErrorModel');
		
		$errorModel = new AriColorboxErrorModel();
		$errorModel->execute(null, $params, $templatePath);
	}
}
?>