<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('DataTables.Models.DataTablesDataModel');
AriKernel::import('CSV.CSVParser');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Utils.Utils');
AriKernel::import('Web.JSON.JSONHelper');

class AriDataTablesDataCsvModel extends AriDataTablesDataModel
{
	var $CACHE_TIME = 10;
	var $_defaultConfig = array(
		'file' => '',
		'sortBy' => '',
		'sortDir' => '',
		'sortType' => '',
		'heading' => true,
		'autoDetect' => false,
		'delimiter' => ',',
		'enclosure' => '"',
		'offset' => 0,
		'limit' => 0);

	function getData($modelParams, $params)
	{
		$csvParser = new AriCSVParser();
		$file = trim($modelParams['path']);
		if (empty($file)) return null;
		
		if (AriUtils::isRemoteResource($file))
		{
			AriKernel::import('Remote.Remote');
			
			$cacheFolder = AriDataTablesHelper::getCacheFolder();
			$remote = AriRemote::getInstance();
			$cachedFileName = $remote->saveRemoteFile($file, $cacheFolder, $this->CACHE_TIME, AriUtils::getParam($modelParams, 'remoteMethod', 'auto'));
			if (empty($cachedFileName))
				return null;
				
			$file = $cacheFolder . DS . $cachedFileName;
		}

		if ($file[0] != '/' && $file[0] != '\\' && (strlen($file) < 2 || $file[1] != ':'))
			$file = JPATH_ROOT . DS . $file;

		if (!@file_exists($file) || !@is_readable($file)) return null;

		$csvParser->strictMode = (bool)AriUtils::getParam($modelParams, 'strictMode');
		if (!empty($modelParams['sortBy'])) 
		{
			$csvParser->sort_by = $modelParams['sortBy'];
			if ($modelParams['sortDir'] && strtoupper($modelParams['sortDir']) == 'DESC') 
				$csvParser->sort_reverse = true;
			if (!empty($modelParams['sortType']) && defined($modelParams['sortType'])) 
				$csvParser->sort_type = constant($modelParams['sortType']);
		}
		
		if (isset($modelParams['heading'])) 
			$csvParser->heading = $modelParams['heading'];
		if (isset($modelParams['offset']) && $modelParams['offset'] > 0) 
			$csvParser->offset = $modelParams['offset'];
		if (isset($modelParams['limit']) && $modelParams['limit'] > 0) 
			$csvParser->limit = $modelParams['limit'];
		if (!$modelParams['autoDetect'])
		{
			if ($modelParams['delimiter']) 
				$csvParser->delimiter = $modelParams['delimiter'];
			if ($modelParams['enclosure']) 
				$csvParser->enclosure = $modelParams['enclosure'];
		}
		
		$csvParser->conditions = $this->getConditions(AriUtils::getParam($modelParams, 'filters'), AriUtils::getParam($modelParams, 'inputEncoding')) ;//'Date2 equals 40664 OR Date2 equals "40665 Man" OR Start Time equals 12:30:0';

		if ($modelParams['autoDetect'])
			$csvParser->auto($file);
		else
			$csvParser->parse($file);

		return $csvParser->data;
	}

	function getConditions($filters, $inputEncoding)
	{
		$conditions = null;
		if (empty($filters))
			return $conditions;
			
		$filters = AriJSONHelper::decode($filters);
		if (!is_array($filters) || count($filters) == 0)
			return $conditions;
		
		if ($inputEncoding)
			AriKernel::import('String.String');
			
		$conditions = array();
		foreach ($filters as $filter)
		{
			$colName = trim($filter['col']);
			if (empty($colName))
				continue ;
				
			$op = trim($filter['op']);
			if (empty($op))
				continue ;
				
			$value = trim($filter['value']);
			if (empty($value))
				continue ;

			$condition = $colName . ' ' . $op . ' "' . $this->prepareCondition($value) . '"';
			if ($inputEncoding)
				$condition = AriString::translateParam('UTF-8', $inputEncoding, $condition);
			
			$conditions[] = $condition;
		}

		return count($conditions) > 0 ? join(' OR ', $conditions) : null;
	}

	function prepareCondition($condition)
	{
		$user =& JFactory::getUser();

		$params = array(
			'UserId' => $user->get('id'),
			'UserGroupId' => $user->get('gid'),
			'UserType' => $user->get('usertype'),
			'UserName' => $user->get('name'),
			'UserLogin' => $user->get('username'),
			'UserBlock' => $user->get('block')
		);
		
		if (strpos($condition, '{$REQUEST:') !== false)
		{
			$request = $_REQUEST;			
			$params['REQUEST'] = $request;
		}

		$condition = AriSimpleTemplate::parse($condition, $params, true);

		return $condition;
	}

	function postProcessing($content, $modelParams)
	{
		if (!empty($modelParams['inputEncoding']))
		{
			AriKernel::import('String.String');

			$content = AriString::translateParam($modelParams['inputEncoding'], 'UTF-8', $content);
		}
			
		return $content;
	}
	
	function isSupportAjax()
	{
		return false;
	}
	
	function getColumns($modelParams, $params, $resolveAlias = false)
	{
		$offset = $modelParams['offset'];
		$limit = $modelParams['limit'];

		$modelParams['offset'] = 0;
		$modelParams['limit'] = 1;

		$data = $this->getData($modelParams, $params);
		$columns = null;
		if ($data)
		{
			$columns = array_keys($data);
			$columns = $data[$columns[0]];
			$columns = array_keys($columns);			
		}		

		$modelParams['offset'] = $offset;
		$modelParams['limit'] = $limit;
		
		return $columns;
	}
	
	function resolveColumnsAlias($columns, $modelParams)
	{
		return $columns;
	}
	
	function initAjax($modelParams, $params)
	{
		$sessionKey = md5(var_export($modelParams, true));
		$_SESSION[$sessionKey] = array();
	}
	
	function getCount($modelParams, $params)
	{
		$count = 0;
		$sessionKey = md5(var_export($modelParams, true));
		if (isset($_SESSION[$sessionKey]['count']))
		{
			$count = @intval($_SESSION[$sessionKey]['count'], 10);
			if ($count > 0)
				return $count;
		}

		$data = $this->getData($modelParams, $params);
		$count = count($data);

		if (isset($_SESSION[$sessionKey]))
			$_SESSION[$sessionKey]['count'] = $count;

		return $count;
	}

	function getAjaxData($modelParams, $params, $filters, $aColumns = null)
	{
		if (empty($aColumns))
			$aColumns = $this->getColumns($modelParams, $params);

		$totalRecords = $this->getCount($modelParams, $params);
	
		$aSearch = $filters['search'];
		$sOrderField = $filters['orderField'];
		$sOrderDir = $filters['orderDir'];

		$start = $filters['start'];
		$limit = $filters['limit'];
		
		$totalFilteredRecords = $totalRecords;
		if (is_array($aSearch) && count($aSearch) > 0)
		{
			$filters = array();
			foreach ($aSearch as $search)
			{
				$filters[] = array(
					'col' => $search['col'], 
					'op' => 'contains',
					'value' => trim($search['search'], '"')
				);
			}
			$modelParams['filters'] = AriJSONHelper::encode($filters);

			$totalFilteredRecords = $this->getCount($modelParams, $params);
		}

		if ($sOrderField)
		{
			$modelParams['sortBy'] = $sOrderField;
			$modelParams['sortDir'] = $sOrderDir;
			$modelParams['sortType'] = 'SORT_REGULAR';
		}

		if ($start > -1 && $limit > 0)
		{
			$modelParams['limit'] = $limit;
			
			if ($start && $modelParams['heading'])
				$modelParams['offset'] = $start + 1;
			else
				$modelParams['offset'] = $start;
		}

		if ($modelParams['heading'])
			$modelParams['heading'] = empty($start);

		$data = $this->getData($modelParams, $params);

		return array(
			'totalRecords' => $totalRecords,
			'totalFilteredRecords' => $totalFilteredRecords,
			'data' => $data
		);	
	}
}