﻿<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('DataTables.Models.DataTablesDataModel');
AriKernel::import('DB.ADODBLite');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Utils.ArrayHelper');

class AriDataTablesDataSqlModel extends AriDataTablesDataModel
{	
	function getData($modelParams, $params)
	{
		$data = null;
		if ($modelParams['connection'] == 'external')
			$data = $this->getDataFromExternalSource($modelParams);
		else
			$data = $this->getDataFromDefaultSource($modelParams);

		$revertData = (bool)AriUtils::getParam($modelParams, 'revertData', false);
		if ($revertData)
			$data = $this->revertData($data);
			
		$graphTransform = AriUtils::getParam($modelParams, 'graphTransform');
		$graphTransformEnabled = (bool)AriUtils::getParam($graphTransform, 'enable');
		if ($graphTransformEnabled)
			$data = $this->graphTransform($data, $graphTransform);

		return $data;
	}
	
	function graphTransform($data, $graphTransform)
	{
		if (empty($data))
			return $data;
			
		$xAxis = trim(AriUtils::getParam($graphTransform, 'xAxis', ''));
		$yAxis = trim(AriUtils::getParam($graphTransform, 'yAxis', ''));
		$valueCol = trim(AriUtils::getParam($graphTransform, 'value', ''));
		$defValue = AriUtils::getParam($graphTransform, 'defValue');

		if (empty($xAxis) || empty($yAxis) || empty($valueCol) || $xAxis == $yAxis)
			return $data;
		
		$prepareData = array();
		$yValues = array();
		foreach ($data as $dataItem)
		{
			$xVal = $dataItem[$xAxis];
			$yVal = $dataItem[$yAxis];
			$val = $dataItem[$valueCol];

			if (!isset($prepareData[$xVal]))
				$prepareData[$xVal] = array();

			$yValues[$yVal] = $yVal;
			$prepareData[$xVal][$yVal] = $val;
		}

		$newData = array();
		foreach ($yValues as $yVal)
		{
			$row = array();
			$row[] = $yVal;
			foreach ($prepareData as $key => $val)
			{
				$row[$key] = isset($val[$yVal]) ? $val[$yVal] : $defValue;
			}

			$newData[] = $row;
		}

		return $newData;
	}
	
	function revertData($data)
	{
		if (empty($data))
			return $data;
	
		$revertedData = array();
		$colKeys = array_keys($data[0]);
		$keyField = array_shift($colKeys);
		$colCount = count($colKeys);

		for ($i = 0; $i < $colCount; $i++)
		{
			$revertedData[] = array('&nbsp;' => $colKeys[$i]);
		}
		
		$rowIdx = 0;
		foreach ($data as $dataItem)
		{
			$colIdx = 0;
			foreach ($colKeys as $key)
			{
				$revertedData[$colIdx][$dataItem[$keyField]] = $dataItem[$key];

				++$colIdx;
			}
			++$rowIdx;
		}

		return $revertedData;
	}
	
	function getDataFromDefaultSource($params)
	{
		$debug = (bool)AriUtils::getParam($params, 'debug', false);
		$query = $this->prepareQuery($params['query']);
		
		$db =& JFactory::getDBO();
		if ($debug)
			$db->debug($debug);
		$db->setQuery($query);
		$data = $db->loadAssocList();
		if ($db->getErrorNum())
		{
			$data = null;
		}
		
		return $data;
	}
	
	function getDataFromExternalSource($params)
	{
		$debug = (bool)AriUtils::getParam($params, 'debug', false);
		$dbHost = AriUtils::getParam($params, 'dbhost', 'localhost');
		$dbName = AriUtils::getParam($params, 'dbname', '');
		$dbUser = AriUtils::getParam($params, 'dbuser', 'root');
		$dbPass = AriUtils::getParam($params, 'dbpass', '');
		$dbType = AriUtils::getParam($params, 'dbtype', 'mysql');
		$persistent = (bool)AriUtils::getParam($params, 'persistent', false);
		$charsetName = AriUtils::getParam($params, 'charset', '');

		$conn = false;
		if (AriADODBLite::isAllowedDbType($dbType) && !empty($dbHost) && !empty($dbName))
		{
			$conn = $persistent
				? @AriADODBLite::createPConnection($dbHost, $dbUser, $dbPass, $dbName, null, $dbType)
				: @AriADODBLite::createNConnect($dbHost, $dbUser, $dbPass, $dbName, null, $dbType);
			if (!$conn || !$conn->IsConnected()) 
				$conn = false;
			else
			{
				if ($debug)
					$conn->debug = $debug;
				if ($charsetName)
				{
					$db =& JFactory::getDBO();
					$conn->Execute('SET NAMES ' . $db->Quote($charsetName));
				}
			}
		}
		
		if (!$conn)
		{
			if ($debug)
				JError::raiseWarning(0, JText::_("ARI Data Tables: Couldn't connect to external database."));
			return null;
		}

		global $ADODB_FETCH_MODE;
		
		$query = $this->prepareQuery($params['query']);
		$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
		$data = $conn->GetArray($query);
		if ($conn->ErrorNo())
		{
			$data = null;
		}

		if ($conn && $conn->IsConnected()) 
			$conn->Close();

		return $data;
	}
	
	function prepareQuery($query)
	{
		$query = str_replace(array('&gt;', '&lt;', '&#039;'), array('>', '<', '\''), $query);
		if (strpos($query, '{$') === false)
			return $query;
			
		$db =& JFactory::getDBO();
		$user =& JFactory::getUser();


$leagueid = ($_GET['leagueid']);
                $memberteamid = ($_GET['teamid']);

$db->setQuery("SELECT teamid FROM fs_teams WHERE leagueid ='".$leagueid."' AND userid = '".$user->id."' AND sport = 'Baseball'");
$b_userteamid = $db->loadResult();

$db->setQuery("SELECT teamid FROM fs_teams WHERE leagueid ='".$leagueid."' AND userid = '".$user->id."' AND sport = 'Football'");
$f_userteamid = $db->loadResult();

$db->setQuery("SELECT teamid FROM fs_teams WHERE leagueid ='".$leagueid."' AND userid = '".$user->id."' AND sport = 'Hockey'");
$h_userteamid = $db->loadResult();


                $playerid = ($_GET['playerid']);
                if ($user->guest) {
                $tz = 'UTC';
                }
                else { $tz = ($user->getParam('timezone'));
                }
                $dateTimeZoneUTC = new DateTimeZone(UTC);
                $dateTimeZoneUser = new DateTimeZone($tz);
                $dateTimeUTC = new DateTime("now", $dateTimeZoneUTC);   
                $dateTimeUser = new DateTime("now", $dateTimeZoneUser);                             
                $dateTimeUser = new DateTime("now", $dateTimeZoneUser);
                
                $offset = $dateTimeZoneUser->getOffset($dateTimeUTC);
                $offsetHours = round(($offset)/3600); 
                $offsetMinutes = round(($offset)/ 60); 

 $db->setquery("SELECT TIMER FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
                $b_TIMER = $db->loadresult();
                
 $db->setquery("SELECT TIMER FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
                $f_TIMER = $db->loadresult();  

$db->setquery("SELECT TIMER FROM fs_hockey_leagues WHERE leagueid ='".$leagueid."'");
                $h_TIMER = $db->loadresult();               


                $db->setQuery("SELECT catid FROM jml_content WHERE id ='".$articleID."'");
                $catID = $db->loadResult();
                
                $db->setQuery("SELECT teams FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_teams = $db->loadResult();

                $db->setQuery("SELECT teams FROM fs_hockey_leagues WHERE leagueid ='".$leagueid."'");
$h_teams = $db->loadResult();

                $db->setQuery("SELECT teams FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_teams = $db->loadResult();


               
              


$db->setquery("SELECT SSINGLES FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SSINGLES = $db->loadresult();

$db->setquery("SELECT SDOUBLES FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SDOUBLES = $db->loadresult();

$db->setquery("SELECT STRIPLES FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_STRIPLES = $db->loadresult();

$db->setquery("SELECT SHR FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SHR = $db->loadresult();

$db->setquery("SELECT SRBI FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SRBI = $db->loadresult();

$db->setquery("SELECT SSB FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SSB = $db->loadresult();

$db->setquery("SELECT SBB FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SBB = $db->loadresult();

$db->setquery("SELECT SR FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SR = $db->loadresult();

$db->setquery("SELECT SIP FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SIP = $db->loadresult();

$db->setquery("SELECT SER FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SER = $db->loadresult();

$db->setquery("SELECT SW FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SW = $db->loadresult();

$db->setquery("SELECT SL FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SL = $db->loadresult();

$db->setquery("SELECT SSV FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SSV = $db->loadresult();

$db->setquery("SELECT SK FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SK = $db->loadresult();

$db->setquery("SELECT SH FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SH = $db->loadresult();

$db->setquery("SELECT SPBB FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SPBB = $db->loadresult();

$db->setquery("SELECT SHD FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SHD = $db->loadresult();

$db->setquery("SELECT SQS FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$b_SQS = $db->loadresult();

$db->setQuery("SELECT DATE_FORMAT(max(DATE_ADD(bidtime,INTERVAL ".$b_TIMER." MINUTE)),'%m/%d/%Y %h:%i:%s %p') FROM fs_baseball_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$endtime = $db->loadResult();

$db->setQuery("SELECT DATE_FORMAT(max(DATE_ADD(bidtime,INTERVAL ".$f_TIMER." MINUTE)),'%m/%d/%Y %h:%i:%s %p') FROM fs_football_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$f_endtime = $db->loadResult();

$db->setQuery("SELECT DATE_FORMAT(max(DATE_ADD(bidtime,INTERVAL ".$f_TIMER." MINUTE)),'%m/%d/%Y %h:%i:%s %p') FROM fs_hockey_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$h_endtime = $db->loadResult();



$db->setQuery("SELECT (max(DATE_ADD(bidtime,INTERVAL ".$b_TIMER." MINUTE))) FROM fs_baseball_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$endtime2 = $db->loadResult();

$db->setQuery("SELECT (max(DATE_ADD(bidtime,INTERVAL ".$f_TIMER." MINUTE))) FROM fs_football_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$f_endtime2 = $db->loadResult();

$db->setQuery("SELECT (max(DATE_ADD(bidtime,INTERVAL ".$f_TIMER." MINUTE))) FROM fs_hockey_auction_".$leagueid." WHERE playerid ='".$playerid."'");
$h_endtime2 = $db->loadResult();

$db->setQuery("SELECT cap FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$leaguecap = $db->loadResult();


$db->setQuery("SELECT cap FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguecap = $db->loadResult();

$db->setQuery("SELECT cap FROM fs_hockey_leagues WHERE leagueid ='".$leagueid."'");
$h_leaguecap = $db->loadResult();

$db->setQuery("SELECT (C + FIRSTBASE + SECONDBASE + THIRDBASE + SS + OF + CF + LF + RF + SP + RP + PITCHERS + UTIL + BE) FROM fs_baseball_leagues WHERE leagueid ='".$leagueid."'");
$leaguerostersize = $db->loadResult();

$db->setQuery("SELECT (QB + RB + WR + TE + OP + DST + K + BE) FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize = $db->loadResult();

$db->setQuery("SELECT (C + LW + RW + D + BE + G + UTIL) FROM fs_hockey_leagues WHERE leagueid ='".$leagueid."'");
$h_leaguerostersize = $db->loadResult();

$f_totalplayers = $f_teams * $f_leaguerostersize;

$f_marginalbudget = $f_leaguecap - $f_leaguerostersize;

$f_totalmarginaldollars = $f_teams * $f_marginalbudget;

$f_limit = $f_totalplayers - 1;


$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." ORDER by FPts desc LIMIT $f_limit,1");
$f_nthfpts = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." ORDER by test desc LIMIT $f_totalplayers) AS subquery");
$f_sumfpts = $db->loadResult();

$f_totalmarginalfpts = $f_sumfpts - $f_totalplayers*$f_nthfpts;

$f_marginalratio = $f_totalmarginaldollars / $f_totalmarginalfpts;


$f_totaldollars = $f_teams * $f_leaguecap;

$db->setQuery("SELECT sum(salary) FROM fs_football_players_".$leagueid." WHERE status !='FA'");
$f_totalsalaries = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts.")*".$f_marginalratio.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' HAVING VAL>0) AS subquery");
$f_totalvalue = $db->loadResult();




$db->setQuery("SELECT OP FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_op = $db->loadResult();

$f_totalplayers_op = $f_teams * $f_leaguerostersize_op;

$f_totalplayers_op_cont = round($f_totalplayers_op/3);





$db->setQuery("SELECT BE FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_be = $db->loadResult();

$db->setQuery("SELECT count(slot) FROM fs_football_players_".$leagueid."  WHERE status='".$memberteamid."' AND slot='bench' AND leagueid ='".$leagueid."'");
$f_slot_bench = $db->loadResult();


if ($f_slot_bench>=$f_leaguerostersize_be){
$f_show_bench_number = $f_slot_bench;
}
else{
$f_show_bench_number = $f_leaguerostersize_be;
}


$db->setQuery("SELECT QB FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_qb = $db->loadResult();

$db->setQuery("SELECT RB FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_rb = $db->loadResult();

$db->setQuery("SELECT WR FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_wr = $db->loadResult();

$db->setQuery("SELECT TE FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_te = $db->loadResult();

$db->setQuery("SELECT K FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_k = $db->loadResult();

$db->setQuery("SELECT DST FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$f_leaguerostersize_dst = $db->loadResult();

$f_totalplayers_qb = 30;

$f_totalplayers_rb = 45;

$f_totalplayers_wr = 45;

$f_totalplayers_te = 20;

$f_totalplayers_k = 10;

$f_totalplayers_dst = 10;

$f_limit_qb = $f_totalplayers_qb;

$f_limit_qb_nth = $f_totalplayers_qb - 1;

$f_limit_rb = $f_totalplayers_rb;

$f_limit_rb_nth = $f_totalplayers_rb - 1;

$f_limit_wr = $f_totalplayers_wr;

$f_limit_wr_nth = $f_totalplayers_wr - 1;

$f_limit_te = $f_totalplayers_te;

$f_limit_te_nth = $f_totalplayers_te - 1;

$f_limit_k = $f_totalplayers_k;

$f_limit_k_nth = $f_totalplayers_k - 1;

$f_limit_dst = $f_totalplayers_dst;

$f_limit_dst_nth = $f_totalplayers_dst - 1;


$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='QB' ORDER by FPts desc LIMIT $f_limit_qb_nth,1");
$f_nthfpts_qb = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='QB' ORDER by test desc LIMIT $f_limit_qb) AS subquery");
$f_sumfpts_qb = $db->loadResult();

$f_totalmarginalfpts_qb = $f_sumfpts_qb - $f_totalplayers_qb*$f_nthfpts_qb;

$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='RB' ORDER by FPts desc LIMIT $f_limit_rb_nth,1");
$f_nthfpts_rb = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='RB' ORDER by test desc LIMIT $f_limit_rb) AS subquery");
$f_sumfpts_rb = $db->loadResult();

$f_totalmarginalfpts_rb = $f_sumfpts_rb - $f_totalplayers_rb*$f_nthfpts_rb;

$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='WR' ORDER by FPts desc LIMIT $f_limit_wr_nth,1");
$f_nthfpts_wr = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='WR' ORDER by test desc LIMIT $f_limit_wr) AS subquery");
$f_sumfpts_wr = $db->loadResult();

$f_totalmarginalfpts_wr = $f_sumfpts_wr - $f_totalplayers_wr*$f_nthfpts_wr;

$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='TE' ORDER by FPts desc LIMIT $f_limit_te_nth,1");
$f_nthfpts_te = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='TE' ORDER by test desc LIMIT $f_limit_te) AS subquery");
$f_sumfpts_te = $db->loadResult();

$f_totalmarginalfpts_te = $f_sumfpts_te - $f_totalplayers_te*$f_nthfpts_te;

$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='K' ORDER by FPts desc LIMIT $f_limit_k_nth,1");
$f_nthfpts_k = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='K' ORDER by test desc LIMIT $f_limit_k) AS subquery");
$f_sumfpts_k = $db->loadResult();

$f_totalmarginalfpts_k = $f_sumfpts_k - $f_totalplayers_k*$f_nthfpts_k;

$db->setQuery("SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE pos ='DST' ORDER by FPts desc LIMIT $f_limit_dst_nth,1");
$f_nthfpts_dst = $db->loadResult();

$db->setQuery("SELECT SUM(test) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS test FROM fs_football_players_".$leagueid." WHERE pos ='DST' ORDER by test desc LIMIT $f_limit_dst) AS subquery");
$f_sumfpts_dst = $db->loadResult();

$f_totalmarginalfpts_dst = $f_sumfpts_dst - $f_totalplayers_dst*$f_nthfpts_dst;





$db->setQuery("SELECT (QB + RB + WR + TE + OP + DST + K +BN) FROM fs_football_leagues WHERE leagueid ='".$leagueid."'");
$leaguerostersize_vbd = $db->loadResult();

$f_marginalbudget_vbd = $f_leaguecap - $leaguerostersize_vbd;

$f_totalmarginaldollars_vbd = $f_teams * $f_marginalbudget_vbd;

$f_totalmarginalfpts_vbd = $f_totalmarginalfpts_qb + $f_totalmarginalfpts_rb + $f_totalmarginalfpts_wr + $f_totalmarginalfpts_te + $f_totalmarginalfpts_k + $f_totalmarginalfpts_dst;

$f_marginalratio_vbd = $f_totalmarginaldollars_vbd / $f_totalmarginalfpts_vbd;




$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_qb.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='QB' HAVING VAL>0) AS subquery");
$f_totalvalue_qb = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_qb.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos ='QB' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_qb = $db->loadResult();


$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_rb.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='RB' HAVING VAL>0) AS subquery");
$f_totalvalue_rb = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_rb.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos = 'RB' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_rb = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_wr.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='WR' HAVING VAL>0) AS subquery");
$f_totalvalue_wr = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_wr.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos = 'WR' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_wr = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_te.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='TE' HAVING VAL>0) AS subquery");
$f_totalvalue_te = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_te.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos = 'TE' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_te = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_k.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='K' HAVING VAL>0) AS subquery");
$f_totalvalue_k = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_k.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos = 'K' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_k = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_dst.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status !='FA' AND pos ='DST' HAVING VAL>0) AS subquery");
$f_totalvalue_dst = $db->loadResult();

$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts_dst.")*".$f_marginalratio_vbd.")) AS VAL FROM fs_football_players_".$leagueid." WHERE pos = 'DST' AND status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue_dst = $db->loadResult();



$f_freesalary = $f_totaldollars - $f_totalsalaries;

$f_freevalue = $f_totaldollars - $f_totalvalue;

$f_inflation = $f_freesalary / $f_freevalue;

$f_inflationrounded = round($f_inflation,2);



$f_totalvalue_vbd = $f_totalvalue_dst + $f_totalvalue_te + $f_totalvalue_k + $f_totalvalue_wr + $f_totalvalue_rb + $f_totalvalue_qb;

$f_freevalue_vbd = $f_totaldollars - $f_totalvalue_vbd;

$f_inflation_vbd = $f_freesalary / $f_freevalue_vbd;

$f_inflationrounded_vbd = round($f_inflation_vbd,2);

$f_teamtotalvalue_vbd = $f_teamtotalvalue_dst + $f_teamtotalvalue_k + $f_teamtotalvalue_te + $f_teamtotalvalue_wr + $f_teamtotalvalue_rb + $f_teamtotalvalue_qb;

$f_teamtotalvaluerounded_vbd = round($f_teamtotalvalue_vbd,0);

$db->setQuery("SELECT sum(salary) FROM fs_football_players_".$leagueid." WHERE status ='".$memberteamid."'");
$f_teamtotalsalaries = $db->loadResult();

$f_teamtotalprofit_vbd = $f_teamtotalvaluerounded_vbd - $f_teamtotalsalaries;

$f_etv_vbd = $capleft/$f_inflation_vbd + $f_teamtotalvalue_vbd;

$f_etvrounded_vbd = round($f_etv_vbd);






$db->setQuery("SELECT SUM(VAL) FROM (SELECT (1+(((((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt)-".$f_nthfpts.")*".$f_marginalratio.")) AS VAL FROM fs_football_players_".$leagueid." WHERE status ='".$memberteamid."' HAVING VAL>0) AS subquery");
$f_teamtotalvalue = $db->loadResult();

$f_teamtotalvaluerounded = round($f_teamtotalvalue,0);




$f_teamtotalprofit = $f_teamtotalvaluerounded - $f_teamtotalsalaries;

$f_etv = $capleft/$f_inflation + $f_teamtotalvalue;

$f_etvrounded = round($f_etv);

$db->setQuery("SELECT SUM(FPts) FROM (SELECT (((dst_int*2)+(dst_dtd*6)+(dst_dfr*2)+(dst_sack)+(dst_sty*2))+pass_yds/25+pass_td*4+pass_int*-2+rush_yds/10+rush_td*6+rec_yds/10+rec_td*6+fl*-2+fg*3+xpt) AS FPts FROM fs_football_players_".$leagueid." WHERE status ='".$memberteamid."') AS subquery");
$f_teamsumfpts = $db->loadResult();

$f_teamsumfptsrounded = round($f_teamsumfpts,2);		









		$params = array(
			'UserId' => $user->get('id'),
			'UserGroupId' => $user->get('gid'),
			'UserType' => $db->Quote($user->get('usertype')),
			'UserName' => $db->Quote($user->get('name')),
			'UserLogin' => $db->Quote($user->get('username')),
			'UserBlock' => $user->get('block'),
			'UserEmail' => $db->Quote($user->get('email')),
	 'leagueid' => $leagueid,
                 'memberteamid' => $memberteamid,
                 'b_userteamid' => $b_userteamid,
                 'f_userteamid' => $f_userteamid,
                 'h_userteamid' => $h_userteamid,


                 

                        'offsetHours' => $offsetHours,
                        'offsetMinutes' => $offsetMinutes,
                        'offset' => $offset,
                        'b_SSINGLES' => $b_SSINGLES,
                        'b_SDOUBLES' => $b_SDOUBLES,
                        'b_STRIPLES' => $b_STRIPLES,
                        'b_SHR' => $b_SHR,
                        'b_SRBI' => $b_SRBI,
                        'b_SSB' => $b_SSB,
                        'b_SBB' => $b_SBB,
                        'b_SR' => $b_SR,
                        'b_SIP' => $b_SIP,
                        'b_SER' => $b_SER,
                        'b_SW' => $b_SW,
                        'b_SL' => $b_SL,
                        'b_SSV' => $b_SSV,
                        'b_SK' => $b_SK,
                        'b_SH' => $b_SH,
                        'b_SPBB' => $b_SPBB,
                        'b_SHD' => $b_SHD,
                        'b_SQS' => $b_SQS,
                        'b_TIMER' => $b_TIMER,
                        'h_TIMER' => $h_TIMER,
                        'f_TIMER' => $f_TIMER,
                        'catID' => $catID,
                        'b_teams' => $b_teams,
                        'f_teams' => $f_teams,
                        'h_teams' => $h_teams,

                        'leaguecap' => $leaguecap,
                        'f_leaguecap' => $f_leaguecap,
                        'h_leaguecap' => $h_leaguecap,
                        'leaguerostersize' => $leaguerostersize,
                        'f_leaguerostersize' => $f_leaguerostersize,
                        'h_leaguerostersize' => $h_leaguerostersize,

                        'f_totalplayers' => $f_totalplayers,
                        'f_totalplayers' => $f_totalplayers,
                        'f_totalplayers_qb' => $f_totalplayers_qb,


                        'f_marginalbudget' => $f_marginalbudget,
                        'f_totalmarginaldollars' => $f_totalmarginaldollars,
                        'f_limit' => $f_limit,
                        'f_nthfpts' => $f_nthfpts,
                        'f_sumfpts' => $f_sumfpts,
                        'f_totalmarginalfpts' => $f_totalmarginalfpts,
                        'f_marginalratio' => $f_marginalratio,
                        'f_marginalratio_vbd' => $f_marginalratio_vbd,
                        'f_totalmarginalfpts_vbd' => $f_totalmarginalfpts_vbd,
                        'f_nthfpts_qb' => $f_nthfpts_qb,
                        'f_nthfpts_rb' => $f_nthfpts_rb,
                        'f_nthfpts_wr' => $f_nthfpts_wr,
                        'f_nthfpts_te' => $f_nthfpts_te,
                        'f_nthfpts_k' => $f_nthfpts_k,
                        'f_nthfpts_dst' => $f_nthfpts_dst,
                        'f_leaguerostersize_qb' => $f_leaguerostersize_qb,
                        'f_leaguerostersize_be' => $f_leaguerostersize_be,
                        'f_slot_bench' => $f_slot_bench,
                        'f_show_bench_number' => $f_show_bench_number,
                        'f_inflation_vbd' => $f_inflation_vbd,
                        'f_inflation' => $f_inflation


		);
		
		if (strpos($query, '{$REQUEST:') !== false)
		{
			$request = $_REQUEST;
			$db =& JFactory::getDBO();
			
			AriArrayHelper::walkRecursive($request, array(&$db, 'Quote'));
			$params['REQUEST'] = $request;
		}

		if (strpos($query, '{$VAR:') !== false && isset($GLOBALS['ADT_VARS']))
		{
			$vars = $GLOBALS['ADT_VARS'];
			$db =& JFactory::getDBO();
			
			AriArrayHelper::walkRecursive($vars, array(&$db, 'Quote'));
			$params['VAR'] = $vars;
		}

		$query = AriSimpleTemplate::parse($query, $params, true);

		return $query;
	}
	
	function isSupportAjax()
	{
		return true;
	}
	
	function getColumns($modelParams, $params, $replaceAlias = false)
	{
		$query = $modelParams['query'];
		
		$modelParams['query'] = $query . ' LIMIT 0,1';

		$data = $this->getData($modelParams, $params);
		$columns = null;
		if ($data)
		{
			$columns = array_keys($data);
			$columns = $data[$columns[0]];
			$columns = array_keys($columns);
			
			if ($replaceAlias)
				$columns = $this->resolveColumnsAlias($columns, $modelParams);
		}		

		$modelParams['query'] = $query;
		
		return $columns;
	}
	
	function resolveColumnsAlias($columns, $modelParams)
	{
		if (empty($columns))
			return $columns;
		
		$query = $modelParams['query'];
		$aliasList = $this->getColumnsAlias($query);
				
		if (count($aliasList > 0))
		{
			$tColumns = array();
			foreach ($columns as $column)
			{
				$tColumns[] = isset($aliasList[$column])
					? $aliasList[$column]
					: $column;
			}

			$columns = $tColumns;
		}
		
		return $columns;
	}
	
	function getAjaxData($modelParams, $params, $filters, $aColumns = null)
	{
		if (empty($aColumns))
			$aColumns = $this->getColumns($modelParams, $params);

		$query = $modelParams['query'];
		$leagueid = ($_GET['leagueid']);
		$aliasList = $this->getColumnsAlias($query);
		$totalRecords = $this->getCount($query, $modelParams, $params);
	
		$sWhere = $filters['where'];
		$sOrder = $filters['order'];
		if ($sOrder)
			$sOrder = 'ORDER BY ' . $sOrder;

		$start = $filters['start'];
		$limit = $filters['limit'];
		
		$totalFilteredRecords = $totalRecords;
		if ($sWhere)
		{
			$query = $this->addWhereClause($query, $sWhere);
			$totalFilteredRecords = $this->getCount($query, $modelParams, $params);
		}

		if ($sOrder)
			$query .= ' ' . $sOrder;

		if ($start > -1 && $limit > 0)
			$query .= sprintf(' LIMIT %d,%d',
				$start,
				$limit
			); 

		$modelParams['query'] = $query;
		$data = $this->getData($modelParams, $params);

		return array(
			'totalRecords' => $totalRecords,
			'totalFilteredRecords' => $totalFilteredRecords,
			'data' => $data
		);	
	}
	
	function getColumnsAlias($query)
	{
		$aliasList = array();
		
		$matches = null;
		$query = trim(str_replace(array("\r", "\n"), array(' ', ' '), $query));
		if (!preg_match('/^\s*SELECT(.+) FROM/i', $query, $matches) || empty($matches[1]))
			return $aliasList;
			
		$columns = explode(',', $matches[1]);
		foreach ($columns as $column)
		{
			@list($colName, $colAlias) = preg_split('/ AS /i', $column);
			
			if (empty($colAlias))
				continue ;
				
			$aliasList[trim($colAlias, '` ')] = trim($colName, '` ');
		}

		return $aliasList;
	}
	
	function addWhereClause($query, $where, $glue = 'AND')
	{
		$query2 = strtolower($query);
		$query2 = preg_replace('#\(.*(WHERE|ORDER|GROUP|LIMIT).*\)#i', '^^^^^', $query2);

		$posWhere = strpos($query2, 'where');
		$posEnd = -1;
		$endOps = array('group by', 'order by');
		foreach ($endOps as $op)
		{
			$opPos = strpos($query2, $op, $posWhere !== false ? $posWhere : 0);
			if ($opPos !== false)
			{
				$posEnd = $opPos;
				break;
			}
		}

		if ($posWhere === false)
		{
			$where = 'WHERE ' . $where;
			if ($posEnd == -1)
			{
				$query .= ' ' . $where;
			}
			else
			{
				$query = substr($query, 0, $posEnd) . ' ' . $where . ' ' . substr($query, $posEnd);
			}
		}
		else
		{
			if ($posEnd == -1)
				$query .= ' ' . $glue . ' ' . $where;
			else
			{
				$query = substr($query, 0, $posEnd) . ' ' . $glue . ' ' . $where . ' ' . substr($query, $posEnd);
			}
		}

		return $query;
	}
	
	function getCount($query, $modelParams, $params)
	{
		$query = trim(str_replace(array("\r", "\n"), array(' ', ' '), $query));
		$query = preg_replace('/^\s*SELECT(.+) FROM/i', 'SELECT COUNT(*) AS DT_CNT FROM', $query, 1);
		
		$originalQuery = $modelParams['query'];
		$modelParams['query'] = $query;

		$data = $this->getData($modelParams, $params);

		$modelParams['query'] = $originalQuery;

		$query2 = strtolower($query);
		$query2 = preg_replace('#\(.*(GROUP).*\)#i', '', $query2);
		$groupedResult = (strpos($query2, 'group by') !== false);

		$count = 0;
		if (is_array($data) && count($data) > 0)
		{
			if ($groupedResult || count($data) > 1)
				$count = count($data);
			else 
				$count = $data[0]['DT_CNT'];
		}

		return $count;
	}
}


