<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('DataTables.Models.DataTablesWidgetModel');
AriKernel::import('Document.DocumentHelper');
AriKernel::import('Utils.Utils');
AriKernel::import('Web.JSON.JSONHelper');
jimport('joomla.filter.filterinput');

class AriDataTablesWidgetTablesorterModel extends AriDataTablesWidgetModel
{
	var $_prefix = 'DataTablesWidget';
	
	function init($params, $mParams, $hiddenColumns)
	{
		$this->includeAssets($params, $mParams['_default']);
		$this->initClientInstance($mParams['table']['id'], $params, $hiddenColumns);
	}
	
	function includeAssets($params, $cfg)
	{
		static $loaded;

		if ($loaded)
			return ;

		AriDataTablesHelper::includeJQuery((bool)$cfg['includeJQuery'], (bool)$cfg['noConflict'], $cfg);

		$baseUri = JURI::root(true) . '/modules/mod_aridatatables/includes/js/widgets/tablesorter/';
		$theme = JFilterInput::clean(AriUtils::getParam($params, 'theme', 'blue'), 'WORD');

		AriDocumentHelper::includeJsFile($baseUri . 'jquery.tablesorter.min.js');
		AriDocumentHelper::includeJsFile($baseUri . 'addons/pager/jquery.tablesorter.pager.js');
		
		AriDocumentHelper::includeCssFile($baseUri . 'addons/pager/jquery.tablesorter.pager.css');
		AriDocumentHelper::includeCssFile($baseUri . 'themes/' . $theme . '/style.css');
		
		AriDocumentHelper::includeJsFile($baseUri . 'addons/filter/tablesorter_filter.js');

		$loaded = true;
	}
	
	function initClientInstance($id, $params, $hiddenColumns)
	{
		$jsOptions = $this->getJsOptions($params, $hiddenColumns);
		$pagingJs = $this->getPagingJsDeclaration($id, $params);
		$filterJs = $this->getFilterJsDeclaration($id, $params);
		
		$document =& JFactory::getDocument();
		$document->addScriptDeclaration(sprintf('(window["ADTQuery"] || jQuery)(document).ready(function($) { $("#%1$s").addClass("tablesorter").tablesorter(%2$s)%3$s;if (typeof(AriDataTables) != "undefined") AriDataTables.hideLoadingPane("#%1$s_wrapper"); });',
			$id,
			!empty($jsOptions) ? AriJSONHelper::encode($jsOptions) : '',
			$pagingJs . $filterJs));
	}
	
	function getFilterJsDeclaration($id, $options)
	{
		$jsDec = '';
		if (empty($options['filter']) || !(bool)$options['filter']) return $jsDec;
		
		$defOptions = array(
			'filterContainer' => '', 
			'filterClearContainer' => '', 
			'filterCaseSensitive' => false,
			'filterWaitTime' => 500
		);
		
		$jsConfig = AriParametersHelper::getUniqueOverrideParameters($defOptions, $options['filtering']);
		
		$jsConfig['filterContainer'] = '#' . $id . '_filterbox';
		$jsConfig['filterClearContainer'] = '#' . $id . '_filterclear';
		
		$jsDec .= sprintf('.tablesorterFilter(%s)', AriJSONHelper::encode($jsConfig));
		
		return $jsDec;
	}
	
	function getPagingJsDeclaration($id, $options)
	{
		$jsDec = '';
		if (!(bool)$options['paging']) return $jsDec;
		
		$defOptions = array(
			'size' => 10, 
			'page' => 0, 
			'seperator' => '/'
		);
		
		$pagingPos = AriUtils::getParam($options, 'pagePos', 'bottom');
		$jsConfig = AriParametersHelper::getUniqueOverrideParameters($defOptions, $options['page']);
		$jsConfig['positionFixed'] = false;
		$jsConfig['container'] = sprintf('#%s_paging_' . $pagingPos, $id);
		
		$size = AriUtils::getParam($jsConfig, 'size', $defOptions['size']);
		$pageList = explode(',', $options['page']['list']);
		if (!in_array($size, $pageList))
			$jsConfig['size'] = $pageList[0]; 

		$jsDec .= sprintf('.tablesorterPager(%s)', AriJSONHelper::encode($jsConfig));

		return $jsDec;
	}
	
	function getJsOptions($options, $hiddenColumns)
	{
		$defOptions = array(
			'widthFixed' => false, 
			'textExtraction' => 'simple', 
			'debug' => false, 
			'cssAsc' => 'headerSortUp',
			'cssDesc' => 'headerSortDown',
			'cssHeader' => 'header',
			'dateFormat' => 'us'
		);

		$jsOptions = AriParametersHelper::getUniqueOverrideParameters($defOptions, $options);
		if ((bool)$options['zebra'])
		{
			if (!isset($jsOptions['widgets']))
				$jsOptions['widgets'] = array();
				
			$jsOptions['widgets'][] = 'zebra';
		}
		
		$notSortableCols = $this->getSorters($options, $hiddenColumns);
		if (count($notSortableCols) > 0) $jsOptions['headers'] = $notSortableCols;
		
		$sortList = $this->getDefaultSorting($options, $hiddenColumns);
		if (!empty($sortList)) $jsOptions['sortList'] = $sortList;		

		return $jsOptions;
	}
	
	function getSorters($options, $hiddenColumns)
	{
		$sorters = new stdClass();
		
		$false = false;
		$sorting = (bool)$options['sorting'];
		if (!$sorting)
		{
			$num = intval(AriUtils::getParam($options, 'colCount', 10), 15);
			for ($i = 0; $i < $num; $i++)
			{
				$idx = AriDataTablesHelper::recalculateIndex($i, $hiddenColumns);
				$sorters->$idx = array('sorter' => $false);
			}
		}

		$colSorters = trim($options['sorters']);
		if ($colSorters)
		{
			$colSorters = AriJSONHelper::decode($colSorters);
			if (is_array($colSorters))
			{
				foreach ($colSorters as $colSorter)
				{
					$index = @intval($colSorter['index'], 10);
					if ($index > -1)
					{
						$index = AriDataTablesHelper::recalculateIndex($index, $hiddenColumns);
						$sorter = $colSorter['sort'];
						$sorters->$index = array('sorter' => ((!$sorter || $sorter == 'none') ? $false : $sorter));
					}
				}
			}
		}
		
		return $sorters;
	}
	
	function getDefaultSorting($options, $hiddenColumns)
	{
		$defaultSorting = array();
		$sorting = trim($options['sorters']);
		if (empty($sorting) || !is_string($sorting)) return null;
		
		$fields = AriJSONHelper::decode($sorting);
		if (!is_array($fields)) return null;
		
		foreach ($fields as $field)
		{
			$fieldSortDir = $field['sortdir'];
			$fieldIndex = $field['index'];
			
			if (strlen($fieldIndex) == 0 || empty($fieldSortDir) || $fieldSortDir == 'none') continue ;

			$index = @intval($fieldIndex, 10);
			if ($index != $fieldIndex) continue ;

			$sortDir = strtolower($fieldSortDir);
			$sortIndex = ($sortDir == 'desc' ? 1 : 0);

			$index = AriDataTablesHelper::recalculateIndex($index, $hiddenColumns);
			$defaultSorting[$index] = array($index, $sortIndex);
		}
		
		return count($defaultSorting) > 0 ? array_values($defaultSorting) : null;
	}
}
?>