<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('DataTables.Models.DataTablesWidgetModel');
AriKernel::import('Document.DocumentHelper');

class AriDataTablesWidgetTinytableModel extends AriDataTablesWidgetModel
{
	var $_prefix = 'DataTablesWidget';
	
	function init($params, $mParams)
	{
		$this->includeAssets($params);
		$this->initClientInstance($mParams['table']['id'], $params);
	}
	
	function includeAssets($params)
	{
		static $loaded;

		if ($loaded)
			return ;

		$baseUri = JURI::root(true) . '/modules/mod_aridatatables/includes/js/widgets/tinytable/';

		AriDocumentHelper::includeJsFile($baseUri . 'tinytable.js');
		AriDocumentHelper::includeCssFile($baseUri . 'style.css');

		$loaded = true;
	}
	
	function initClientInstance($id, $params)
	{
		$loadType = AriUtils::getParam($params, 'loadType', 'domready');
		if ($loadType != 'onload')
			$loadType = 'domready';
		
		$jsOptions = $this->getJsOptions($id, $params['opt']);
		$jsOptions['pagesize'] = $this->getPageSize($jsOptions['pagesize'], $params['pagelist']);
		$sortCol = AriUtils::getParam($params, 'sortcol');
		if (is_null($sortCol) || strlen($sortCol) == 0)
			$sortCol = 'null';
		else 
			$sortCol = intval($sortCol, 10);

		$document =& JFactory::getDocument();
		$document->addScriptDeclaration(sprintf('TINY.%4$s(function() { window.var%1$s = new TINY.table.sorter(%2$s); window.var%1$s.init("%1$s", %3$s);if (typeof(AriDataTables) != "undefined") AriDataTables.hideLoadingPane("#%1$s_wrapper"); });',
			$id,
			!empty($jsOptions) ? AriJSONHelper::encode($jsOptions) : '{}',
			$sortCol,
			$loadType == 'domready' ? 'domReady' : 'onLoad'));
	}
	
	function getJsOptions($id, $options)
	{
		$defOptions = array(
			'pagesize' => 20,
			'paginate' => false,
			'head' => 'head',
			'asc' => 'asc',
			'desc' => 'desc',
			'even' => 'evenrow',
			'odd' => 'oddrow',
			'evensel' => 'evenselected',
			'oddsel' => 'oddselected'
		);

		$jsOptions = AriParametersHelper::getUniqueOverrideParameters($defOptions, $options);
		if (!empty($jsOptions['paginate']))
		{
			$jsOptions['currentid'] = $id . '_current';
			$jsOptions['limitid'] = $id . '_limit';
		}
		
		if (!isset($jsOptions['pagesize']))
			$jsOptions['pagesize'] = $defOptions['pagesize'];

		return $jsOptions;
	}
	
	function getPageSize($pageSize, $pageList)
	{
		$pageSize = intval($pageSize, 10);
		$pageList = AriDataTablesWidgetTinytableModel::getPageList($pageList);
		if (!in_array($pageSize, $pageList))
			$pageSize = $pageList[0];
			
		return $pageSize;
	}
	
	function getPageList($pageList)
	{
		$pageList = explode(',', $pageList);
		array_walk($pageList, 'intval');
		$pageList = array_unique($pageList);
		sort($pageList);
		
		return $pageList;
	}
}
?>