<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('DataTables.Models.DataTablesWidgetModel');
AriKernel::import('Document.DocumentHelper');

class AriDataTablesWidgetVisualizerModel extends AriDataTablesWidgetModel
{
	var $_availTypes = array('bar', 'area', 'pie', 'line');
	var $_availPieLabelPos = array('inside', 'outside');

	function init($params, $mParams)
	{
		$this->includeAssets($params, $mParams['_default']);
		$this->initClientInstance($mParams['table']['id'], $params);
	}
	
	function includeAssets($params, $cfg)
	{
		static $loaded;

		if ($loaded)
			return ;

		AriDataTablesHelper::includeJQuery((bool)$cfg['includeJQuery'], (bool)$cfg['noConflict'], $cfg);
		AriDataTablesHelper::includeExCanvas();

		$baseUri = JURI::root(true) . '/modules/mod_aridatatables/includes/js/widgets/datavisualizer/';

		AriDocumentHelper::includeJsFile($baseUri . 'visualize.jQuery.js');
		AriDocumentHelper::includeCssFile($baseUri . 'visualize.jQuery.css');

		$loaded = true;
	}
	
	function initClientInstance($id, $params)
	{
		$jsOptions = $this->getJsOptions($params);
		$document =& JFactory::getDocument();
		$document->addScriptDeclaration(sprintf('if (jQuery.browser.msie) jQuery(window).load(function() { (window["ADTQuery"] || jQuery)("#%1$s").visualize(%2$s); }); else (window["ADTQuery"] || jQuery)(document).ready(function($) { $("#%1$s").visualize(%2$s); });',
			$id,
			!empty($jsOptions) ? AriJSONHelper::encode($jsOptions) : ''));
	}
	
	function getJsOptions($options)
	{
		$defOptions = array(
			'type' => 'bar',
			'srcType' => 'int',
			'floatDelim' => '.',
			'width' => 0,
			'height' => 0,
			'appendTitle' => true,
			'title' => '',
			'appendKey' => true,
			'colors' => '#be1e2d,#666699,#92d5ea,#ee8310,#8d10ee,#5a3b16,#26a4ed,#f45a90,#e9e744',
			'textColors' => '',
			'parseDirection' => 'x',
			'pieMargin' => 20,
			'pieLabelPos' => 'inside',
			'pieShowValues' => false,
			'lineWeight' => 4,
			'barGroupMargin' => 10,
			'barMargin' => 1,
			'prefix' => '',
			'postfix' => ''
		);

		$jsOptions = AriParametersHelper::getUniqueOverrideParameters($defOptions, $options);
		
		if (isset($jsOptions['type'])) $jsOptions['type'] = $this->getCorrectType($jsOptions['type']);
		if (isset($jsOptions['pieLabelPos'])) $jsOptions['pieLabelPos'] = $this->getCorrectPieLabelPos($jsOptions['pieLabelPos']);
		if (isset($jsOptions['colors'])) $jsOptions['colors'] = $this->getCorrectColors($jsOptions['colors']);
		if (isset($jsOptions['textColors'])) $jsOptions['textColors'] = $this->getCorrectColors($jsOptions['textColors']);

		return $jsOptions;
	}
	
	function getCorrectColors($colors)
	{
		if (is_string($colors)) $colors = explode(',', $colors);
		
		return $colors;
	}
	
	function getCorrectType($type)
	{
		if (is_string($type)) $type = strtolower($type);
		if (!in_array($type, $this->_availTypes)) $type = 'bar';
		
		return $type;
	}
	
	function getCorrectPieLabelPos($labelPos)
	{
		if (is_string($labelPos)) $labelPos = strtolower($labelPos);
		if (!in_array($labelPos, $this->_availPieLabelPos)) $labelPos = 'inside';
		
		return $labelPos;
	}
	
	function fixColumnsSettings($columnsSettings)
	{
		if (is_null($columnsSettings))
			$columnsSettings = array();
			
		if (!isset($columnsSettings[0]))
			$columnsSettings[0] = array();
			
		$columnsSettings[0]['celltag'] = 'th';
		$columnsSettings[0]['headtag'] = 'td';
		
		return $columnsSettings;
	}
}
?>