<?php
/*
 * ARI Data Tables
 *
 * @package		ARI Data Tables
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Utils.Utils2');
AriKernel::import('Document.DocumentHelper');
AriKernel::import('Web.JSON.JSONHelper');
jimport('joomla.filter.filterinput');
jimport('joomla.utilities.arrayhelper');

class AriDataTablesHelper
{
	function getProtocol()
	{
		static $protocol;
		
		if (is_null($protocol))
		{
			$uri =& JURI::getInstance();
			$protocol = $uri->toString(array('scheme'));
		}
		
		return $protocol;
	}

	function includeExCanvas()
	{
		static $loaded;

		if ($loaded)
			return ;
		
		AriDocumentHelper::includeCustomHeadTag(
			'<!--[if IE]><script src="' . JURI::root(true) . '/modules/mod_aridatatables/includes/js/excanvas-compressed.js" type="text/javascript"></script><![endif]-->');
		
		$loaded = true;
	}
	
	function includeMooTools()
	{
		static $loaded;

		if ($loaded)
			return ;
			
		AriDocumentHelper::includeJsFile(AriDataTablesHelper::getProtocol() . 'ajax.googleapis.com/ajax/libs/mootools/1.1.2/mootools-yui-compressed.js');

		$loaded = true;
	}
	
	function includeGoogleAPI()
	{
		static $loaded;
		
		if ($loaded)
			return ;
			
		AriDocumentHelper::includeJsFile(AriDataTablesHelper::getProtocol() . 'www.google.com/jsapi');
			
		$loaded = true;
	}
	
	function includeJQuery($loadJQuery, $noConflict, $params)
	{
		static $loaded;

		if ($loaded)
			return ;

		$baseUri = JURI::root(true) . '/modules/mod_aridatatables/includes/js/';
		if ($loadJQuery) 
		{
			$loadJQueryMethod = AriUtils::getParam($params, 'loadJQueryMethod', 'google_cdn');
		
			if ($loadJQueryMethod == 'local')
			{
				if (J3_0)
					JHtml::_('jquery.framework', $noConflict);
				else
					AriDocumentHelper::includeJsFile($baseUri . 'jquery.min.js');
			}
			else
			{
				AriDocumentHelper::includeJsFile(AriDataTablesHelper::getProtocol() . 'ajax.googleapis.com/ajax/libs/jquery/1.8.2/jquery.min.js');
			}
			
			if ($noConflict)
			{
				if (!J3_0 || $loadJQueryMethod != 'local')
					AriDocumentHelper::includeJsFile($baseUri . 'jquery.noconflict.js');
			}
			
			$loaded = true;
		}
	}
	
	function includeJQueryUITheme($theme = 'smoothness')
	{
		static $loaded;

		if ($loaded)
			return ;

		$filter =& JFilterInput::getInstance();
		$theme = $filter->clean($theme, 'WORD');
		if (empty($theme))
			$theme = 'smoothness';

		$themeUri = JURI::root(true) . '/modules/mod_aridatatables/includes/assets/jui/' . $theme . '/style.css';
		AriDocumentHelper::includeCssFile($themeUri);
			
		$loaded = true;
	}
	
	function includeAssets()
	{
		static $loaded;

		if ($loaded)
			return ;
			
		$baseUri = JURI::root(true) . '/modules/mod_aridatatables/includes/assets/';
		AriDocumentHelper::includeCssFile($baseUri . 'style.css');
		AriDocumentHelper::includeJsFile($baseUri . 'common.js');
		
		$loaded = true;
	}

	function includeSpecificStyles($params, $columnsSettings)
	{
		$tableParams = $params['table'];
		$id = $tableParams['id'];
		
		$cssStyles = sprintf('#%1$s { width: %2$s; border-collapse: %3$s; border-width: %4$spx; }',
			$id,
			!empty($tableParams['width']) ? $tableParams['width'] : '100%',
			!empty($tableParams['borderCollapse']) ? $tableParams['borderCollapse'] : 'separate',
			intval($tableParams['border'], 10));
			
		if ($columnsSettings)
		{
			foreach ($columnsSettings as $colParam)
			{
				$colStyle = '';
				if (!empty($colParam['align']) && $colParam['align'] != 'none')
					$colStyle .= 'text-align:' . $colParam['align'] . ';';
					
				if (!empty($colParam['width']))
				{
					$width = $colParam['width'];
					if ($width == '' . intval($width, 10))
						$width .= 'px';
					$colStyle .= 'width:' . $width . ';';
				}

				if ($colStyle)
				{
					$colClassName = AriDataTablesHelper::getColClass($colParam['id']);
					$cssStyles .= sprintf('#%1$s TBODY TD.%2$s,#%1$s TBODY TD DIV.%2$s {%3$s}',
						$id,
						$colClassName,
						$colStyle);
				}
			}
		}
			
		$customStyles = $params['_default']['styles'];
		if ($customStyles)
			$cssStyles .= ' ' . str_replace('{$id}', '#' . $id, $customStyles);
		
		$document =& JFactory::getDocument();
		$document->addStyleDeclaration($cssStyles);
	}
	
	function getColumnsSettings($colParams)
	{
		$colParams = $colParams ? AriJSONHelper::decode($colParams) : null;
		if (is_null($colParams))
			return $colParams;
			
		$fixedParams = array();
		foreach ($colParams as $colParam)
		{
			if (!isset($colParam['id']) || is_null($colParam['id']) || strlen($colParam['id']) == 0)
				continue ;
				
			$id = $colParam['id'];
			if ($id == '' . intval($id, 10))
				$colParam['id'] = intval($id, 10);

			if (empty($colParam['class']))
				$colParam['class'] = '';
				
			if (!empty($colParam['format']))
				$colParam['format'] = trim($colParam['format']);

			$colParam['class'] .= ' ' . AriDataTablesHelper::getColClass($colParam['id']);
			$fixedParams[$colParam['id']] = $colParam;
		}
		
		return count($fixedParams) > 0 ? $fixedParams : null;
	}
	
	function getHiddenColumns($dataRow, $columnsSettings)
	{
		$hiddenColumns = array();
		
		if (empty($dataRow) || empty($columnsSettings))
			return $hiddenColumns;

		$keys = array_keys($dataRow);
		foreach ($columnsSettings as $key => $colSettings)
		{
			if (empty($colSettings['hidden']))
				continue ;

			if (is_int($key))
				$hiddenColumns[] = $key;
			else 
			{
				$idx = array_search($key, $keys);
				if ($idx !== false)
					$hiddenColumns[] = $idx;
			}
		}
		
		$hiddenColumns = array_unique($hiddenColumns);
		sort($hiddenColumns);
		
		return $hiddenColumns;
	}
	
	function recalculateIndex($index, $hiddenColumns)
	{
		if (!is_array($hiddenColumns) || count($hiddenColumns) == 0)
			return $index;

		$diff = 0;
		foreach ($hiddenColumns as $hiddenColumn)
		{
			if ($hiddenColumn >= $index)
				break ;

			++$diff;
		}
		
		return $index - $diff;
	}
	
	function getColumnSettings($columnsSettings, $key, $idx)
	{
		$colSettings = null;
		if (empty($columnsSettings))
			return $colSettings;

		if (array_key_exists($key, $columnsSettings))
			$colSettings = $columnsSettings[$key];
		else if (array_key_exists($idx, $columnsSettings))
			$colSettings = $columnsSettings[$idx];
			
		return $colSettings;
	}
	
	function getColClass($id)
	{
		return 'col_' . substr(md5($id), 0, 8);
	}
	
	function getDataModel($model, $params)
	{
		return AriDataTablesHelper::getModel($model, $params, 'DataTablesData');
	}
	
	function getWidgetModel($model, $params)
	{
		return AriDataTablesHelper::getModel($model, $params, 'DataTablesWidget');
	}
	
	function getModel($model, $params, $prefix = 'DataTables')
	{
		$modelInstance = null;
		$modelParams = isset($params[$model]) ? $params[$model] : null;
		$filter =& JFilterInput::getInstance();
		$model = ucfirst($filter->clean($model, 'WORD'));
		if (empty($model))
		{
			return $modelInstance;
		}
		else
		{
			$modelName = $prefix . $model . 'Model';
			$modelPath = dirname(__FILE__) . DS . 'Models' . DS . 'class.' . $modelName . '.php';

			if (!@file_exists($modelPath))
			{
				return $modelInstance;
			}
			else
			{
				AriKernel::import('DataTables.Models.' . $modelName);

				$modelClass = 'Ari' . $modelName;
				$modelInstance = new $modelClass();
			}
		}
		
		return $modelInstance;
	}
	
	function getCacheFolder()
	{
		$cacheDir = JPATH_ROOT . DS . 'cache' . DS . 'mod_aridatatables';
		$extCacheDir = $cacheDir . DS . 'mod_aridatatables';
		
		return (@file_exists($extCacheDir) && is_dir($extCacheDir))
			? $extCacheDir
			: $cacheDir;
	}
	
	function cleanValue($value)
	{
		if (strpos($value, '_') === false)
			return $value;
			
		$value = explode('_', $value);
		return $value[count($value) - 1];
	}
}
?>