<?php
defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

AriKernel::import('Components.AriSmartContent.Plugins.PluginBase');

class AriJTableSorterPlugin extends AriSmartContentPluginBase
{
	var $_idList = array();

	function __construct()
	{
		parent::__construct('arijtablesorter');
	}
	
	function processContent($published, &$row, &$params, $page = 0)
	{
		parent::processContent($published, $row, $params, $page);
		
		$this->runtimeJsDeclaration();
	}

	function replaceCallback($attrs, $content = '')
	{
		$config = array('js' => 
			array(
				'dateFormat' => 'us',
				'widthFixed' => false, 
				'textExtraction' => 'simple', 
				'debug' => false, 
				'cssAsc' => 'headerSortUp',
				'cssDesc' => 'headerSortDown',
				'cssHeader' => 'header'), 
			'paging' => null);

		$jsConfig = array();
		foreach ($config['js'] as $key => $defValue)
		{
			$val = $this->getValueByDefSample($key, $attrs, $defValue);
			if ($val != $defValue)
			{
				$jsConfig[$key] = $val;
			}
		}
		
		$config['js'] = $jsConfig;	
		$params = $this->getParameters();
		$zebra = AriUtils::getParam($attrs, 'zebra', null);
		if (is_null($zebra)) $zebra = $params->get('zebra', null);
		$zebra = AriUtils::parseValueBySample($zebra, true);
		if ($zebra) $config['js']['widgets'] = array('zebra');

		$notSortableCols = $this->getSorters($attrs);
		if (count($notSortableCols) > 0) $config['js']['headers'] = $notSortableCols;
		
		$sortList = $this->getDefaultSorting($attrs);
		if (!empty($sortList)) $config['js']['sortList'] = $sortList;
		$config['paging'] = $this->getPagingConfig($attrs, $params);

		if (!is_null($config['paging']))
		{
			$config['paging']['id'] = array();
			$pos = $config['paging']['position'];
			$pageId = uniqid('ajtbp_', false);
			$config['paging']['id'][] = $pageId;
			$pageTemplate = $this->getPagingTemplate($pageId, $config['paging']);
			if ($pos == 'top')
			{
				$content = $pageTemplate . $content;
			}
			else if ($pos == 'bottom')
			{
				$content .= $pageTemplate;
			}
		}

		$id = uniqid('jts_', false);
		$this->_idList[$id] = $config;

		return sprintf('<div id="%2$s">%1$s</div>',
			$content,
			$id);
	}
	
	function getSorters($attrs)
	{
		$sorters = new stdClass();
		
		$false = false;
		$sorting = $this->getValueByDefSample('sorting', $attrs, true);
		$notSortableCols = $this->getValueByDefSample('notSortableCols', $attrs, '');
		if (!$sorting)
		{
			$num = $this->getValueByDefSample('colCount', $attrs, 10);
			for ($i = 0; $i < $num; $i++)
			{
				$sorters->$i = array('sorter' => $false);
			}
		}
		else if ($notSortableCols != null && strlen($notSortableCols) > 0)
		{
			$indexList = explode(',', $notSortableCols);
			foreach ($indexList as $index)
			{
				$index = @intval($index, 10);
				if ($index > -1) $sorters->$index = array('sorter' => $false);
			}
		}
		
		// sorters="0:none,1:currency"
		$colSorters = $this->getValueByDefSample('sorters', $attrs, '');
		if ($colSorters)
		{
			$colSorters = trim($colSorters);
			$colSorters = explode(',', $colSorters);
			
			foreach ($colSorters as $colSorter)
			{
				$colSorterInfo = explode(':', $colSorter);
				$index = @intval($colSorterInfo[0], 10);
				if ($index > -1)
				{
					if (count($colSorterInfo) > 1) $sorter = trim($colSorterInfo[1]);
					$sorters->$index = array('sorter' => ((!$sorter || $sorter == 'none') ? $false : $sorter));
				}
			}
		}
		
		return $sorters;
	}
	
	function getPagingTemplate($id, $config)
	{
		$template = sprintf('<div id="%1$s" class="tablesorterPager">', $id);
		if ($config['showFirst']) $template .= '<input type="button" class="first" value=" " onfocus="this.blur();" />';
		if ($config['showPrev']) $template .= '<input type="button" class="prev" value=" " onfocus="this.blur();" />';
		if ($config['showPageInfo']) $template .= '<input class="pagedisplay" readonly type="text" />';
		if ($config['showNext']) $template .= '<input type="button" class="next" value=" " onfocus="this.blur();" />';
		if ($config['showLast']) $template .= '<input class="last" type="button" value=" " onfocus="this.blur();" />';
		$template .= $this->getTemplatePageList($config['pageList'], $config['js']['size']);
		$template .= '</div>';
		
		return $template;
	}
	
	function getTemplatePageList($pageList, $pageSize)
	{
		$template = '<select class="pagesize">';
		foreach ($pageList as $pageListItem)
		{
			$template .= sprintf('<option value="%1$d"%2$s>%1$d</option>',
				$pageListItem,
				$pageListItem == $pageSize ? ' selected' : '');
		}
		$template .= '</select>';
		
		return $template;
	}
	
	function getDefaultSorting($attrs)
	{
		$defaultSorting = array();
		$sorting = AriUtils::getParam($attrs, 'defaultSorting', null);
		if (empty($sorting) || !is_string($sorting)) return null;
		
		$fields = explode(',', $sorting);
		foreach ($fields as $field)
		{
			$fieldInfo = explode(':', $field);
			$fieldIndex = trim($fieldInfo[0]);
			
			if (strlen($fieldIndex) == 0) continue ;
			
			$index = @intval($fieldIndex, 10);
			if ($index != $fieldIndex) continue ;
			
			$sortDir = 'asc';
			if (isset($fieldInfo[1])) $sortDir = strtolower($fieldInfo[1]);
			$sortIndex = ($sortDir == 'desc' ? 1 : 0);

			$defaultSorting[$index] = array($index, $sortIndex);
		}
		
		return count($defaultSorting) > 0 ? array_values($defaultSorting) : null;
	}
	
	function getPagingConfig($attrs, $params)
	{
		$config = null;
		
		$paging = $this->getValueByDefSample('paging', $attrs, false);
		if (!$paging) return $config;
		
		$config = array('js' => array('positionFixed' => false), 'position' => 'bottom');
		$config['js']['size'] = $this->getValueByDefSample('pageSize', $attrs, 10);
		$config['js']['page'] = $this->getValueByDefSample('pageNum', $attrs, 0);
		$config['js']['seperator'] = $this->getValueByDefSample('pageSeparator', $attrs, '/');
		
		$config['showFirst'] = $this->getValueByDefSample('showFirst', $attrs, true);
		$config['showPrev'] = $this->getValueByDefSample('showPrev', $attrs, true);
		$config['showPageInfo'] = $this->getValueByDefSample('showPageInfo', $attrs, true);
		$config['showNext'] = $this->getValueByDefSample('showNext', $attrs, true);
		$config['showLast'] = $this->getValueByDefSample('showLast', $attrs, true);
		$cfgPageList = $this->getValueByDefSample('pageList', $attrs, '');
		$cfgPageList = explode(',', $cfgPageList);
		if (!is_array($cfgPageList))
		{
			$cfgPageList = array($cfgPageList);
		}
		
		$pageList = array();
		foreach ($cfgPageList as $pageListItem)
		{
			$val = @intval($pageListItem, 10);
			if ($val > 0) $pageList[] = $val;
		}
		
		if (count($pageList) == 0)
		{
			$pageList = array(10, 20, 30, 40);
		}
		
		$pageList = array_unique($pageList);
		sort($pageList);
		if (!in_array($config['js']['size'], $pageList))
		{
			$config['js']['size'] = $pageList[0];
		}
		
		$config['pageList'] = $pageList;
		
		$availPositions = array('bottom', 'top');
		$position = $this->getValueByDefSample('pagingPos', $attrs, 'bottom');
		if (!in_array($position, $availPositions)) $position = 'bottom';
		$config['position'] = $position;
		
		return $config;
	}

	function jsDeclaration()
	{
		$params = $this->getParameters();
		$theme = $params->get('theme', 'blue');

		$this->loadTheme($theme);
		
		$jqueryLoader = new AriJQueryLoaderPlugin();
		$jqueryLoader->load();
		$rootPath = $this->getRootPath();
		
		AriDocumentHelper::includeJsFile(AriUtils::absPath2Url($rootPath . 'jquery.tablesorter.min.js'));
		AriDocumentHelper::includeCssFile(AriUtils::absPath2Url($rootPath . 'addons/pager/jquery.tablesorter.pager.css'));
		AriDocumentHelper::includeJsFile(AriUtils::absPath2Url($rootPath . 'addons/pager/jquery.tablesorter.pager.js'));
	}
	
	function loadTheme($theme = 'blue')
	{
		$rootPath = $this->getRootPath();
		$themeFile = $rootPath . 'themes/' . $theme . '/style.css';
		if (@file_exists($themeFile))
		{
			AriDocumentHelper::includeCssFile(AriUtils::absPath2Url($themeFile));
		}
	}
	
	function runtimeJsDeclaration()
	{
		$idList = $this->_idList;
		if (empty($idList)) return ;
		
		$script = '<script type="text/javascript">';
		foreach ($idList as $id => $config)
		{
			$jsConfig = AriUtils::getParam($config, 'js', null);
			$pagingJs = $this->getPagingJsDeclaration(AriUtils::getParam($config, 'paging', null));

			$script .= sprintf('jQuery(document).ready(function($) { $("#%1$s > table").addClass("tablesorter").tablesorter(%2$s)%3$s });', 
				$id,
				!empty($jsConfig)? AriJSONHelper::encode($jsConfig) : 'null',
				$pagingJs ? '.' . $pagingJs : '');
		}
		$script .= '</script>';
		
		AriDocumentHelper::includeCustomHeadTag($script);
	}
	
	function getPagingJsDeclaration($config)
	{
		$jsDec = '';
		if (is_null($config)) return $jsDec;

		$jsConfig = $config['js'];
		$jsDec = array();
		foreach ($config['id'] as $id)
		{
			$jsConfig['container'] = sprintf('#%s', $id);
			$jsDec[] = sprintf('tablesorterPager(%s)', AriJSONHelper::encode($jsConfig));
		}
		
		$jsDec = join('.', $jsDec);
		
		return $jsDec;
	}
	
	function getRootPath()
	{
		return dirname(__FILE__) . '/js/';
	}
}
?>