<?php
defined('_JEXEC') or die( 'Restricted access' );

class RFPROAdminModelCategories extends JModel{

	function __construct($config=array()){
		parent::__construct($config);
	}
	
	/**
	 * recursive function that returns the categories as a tree
	 *
	 * @param int The parent id
	 * @param bool Include the parent in the array (default false)
	 * @param bool Reset the result (static variable); set this flag true when you call the function twice in a script
	 *
	 * @return array $treeCat (static variable!)
	 */
	function getChildrenTree($categoryId,$includeParent=false,$maxLevels=0){
		static $instances = array();

		$key = $categoryId.($includeParent?'1':'0').$maxLevels;
		if(isset($instances[$key])){
			return $instances[$key];
		}
		$tablePath = RSS_FACTORY_COMPONENT_ADMIN_PATH.DS.'tables';
		JTable::addIncludePath($tablePath);
		$category =& JTable::getInstance('rfpronestedset');
		$instances[$key] = $category->getSubTree($categoryId,$includeParent,$maxLevels=0);

		return $instances[$key];
	}

	//this function checks if a category has children; it returns the children's number
	function has_children($id)
	{
	
		$database = &JFactory::getDBO();
	
		$database->setQuery("select count(*) from #__rssfactory_categories where parent='".$id."'");
		$count = $database->loadResult();
	
		return $count;
	}
	//this function checks if a category has published rss sources; it returns the sources' number
	function has_sources($id)
	{
	
		$database = &JFactory::getDBO();
	
		$database->setQuery("SELECT COUNT(r.id) FROM #__rssfactory r WHERE r.cat='".$id."' AND r.published='1'");
		$count = $database->loadResult();
	
		return $count;
	
	}
	
	function saveCategory()
	{
		$database =& JFactory::getDBO();
		$category =& JTable::getInstance('rfpronestedset');

		$nodeId = JRequest::getVar('id',0,'default','int');
    	$nodeName = JRequest::getVar('name','','default','string');

    	$rN = $category->getRootNode();

    	$parentId = JRequest::getVar('parentId',$category->getRootNode()->id,'default','int');

		if($nodeId)
		{
			if($parentId!=JRequest::getVar('oldParentId',$category->getRootNode()->id,'default','int'))
			{
				$category->moveNode($nodeId,$parentId);
			}
		}
		else
		{
			$nodeId = $category->insertChildNode($nodeName,$parentId);
		}

		$category->id = $nodeId;
		$category->name = JRequest::getVar('name');
		$category->description = JRequest::getVar('description');

		if(!$category->store()){
			return false;
		}

		//save related categories
		$database->setQuery("DELETE FROM `#__rssfactory_related_categories_map` WHERE `categoryId`='".$category->id."' OR `relatedId`='".$category->id."'");
		$database->query();
		$relatedCategories = JRequest::getVar('relatedcats',array(),'default','array');

		$relation =& JTable::getInstance('rfprorelatedcategorymap');
		foreach($relatedCategories as $key=>$relcat)
		{
			//don't relate the category to itself
			if( $relcat == $category->id )
			{
				unset($category->relatedcats[$key]);
				continue;
			}

			//direct relation 
			$relation->id = null;
			$relation->categoryId = $category->id;
			$relation->relatedId = intval($relcat);
			$relation->store();

			//backward relation
			$relation->id = null;
			$relation->categoryId = intval($relcat);
			$relation->relatedId = $category->id;
			$relation->store();
		}

		return true;
	}

	function saveMultipleCategories(){
		global $mainframe,$option;
		$database =& JFactory::getDBO();
		$big_parent_id = JRequest::getVar('parentCategory');

		$separator = PHP_EOL;
		$textcats = explode($separator,JRequest::getVar('textcats'));

		$stack = array($big_parent_id);

		$i=0;
		foreach($textcats as $key=>$cat){
			$prevcat = isset($textcats[$i-1]) ? $textcats[$i-1] : '';
			$prevcat_spaces = RFProAdminHelper::getFirstSpaces($prevcat);
			$cat_spaces = RFProAdminHelper::getFirstSpaces($cat);

			if($cat_spaces > $prevcat_spaces ){

				//if that moron user puts too many spaces in front of the child category, here we make things right
				if($cat_spaces-$prevcat_spaces>1){
					$new_cat_spaces = '';
					for($j=0;$j<$prevcat_spaces+1;$j++){
						$new_cat_spaces .= ' ';
					}
					$textcats[$key] = $new_cat_spaces.trim($cat);
				}
				
				array_push($stack,$database->insertid());

			}
			if($cat_spaces < $prevcat_spaces){
				$diff_level = $prevcat_spaces-$cat_spaces;
				for($j=0;$j<$diff_level;$j++){
					array_pop($stack);
				}
			}
			$query = "INSERT INTO `#__rssfactory_categories` (`title`,`parent`) VALUES ('". $database->getEscaped( trim($cat) )."','".$database->getEscaped( end($stack) )."')";
			$database->setQuery($query);
			if(!$database->query())
			{
				JError::raiseError(1,$database->getErrorMsg());
				return false;
			}

			$i++;
		}
		
		$this->reIndexOrdering($big_parent_id);

		return $i;
	}

	function reIndexOrdering($parentId)
	{
		$database =& JFactory::getDBO();

		$database->setQuery("SELECT MAX(`ordering`) FROM `#__rssfactory_categories` WHERE `parent`='".$database->getEscaped($parentId)."'");
		$maxOrdering = $database->loadResult();
		
		$database->setQuery("SELECT `id`, `ordering` FROM `#__rssfactory_categories` WHERE `parent`='".$database->getEscaped($parentId)."'");
		$categories = $database->loadObjectList();

		foreach($categories as $category)
		{
			if(!$category->ordering)
			{
				$database->setQuery("UPDATE `#__rssfactory_categories` SET `ordering`='".($maxOrdering+1)."' WHERE `id`='".$category->id."'");
				$database->query();
				$maxOrdering++;
			}
			$this->reIndexOrdering($category->id);
		}
	}

	function deleteCategories()
	{
		$database =& JFactory::getDBO();
		$cid = JRequest::getVar('cid',array(),'default','array');

		if(!count($cid))
		{
			JError::raiseWarning(1,'You have to select  a category to delete!');
			return false;
		}

		$category =& JTable::getInstance('rfpronestedset');
		foreach($cid as $categoryId)
		{
			$category->deleteNode($categoryId);
		}

		return true;
	}

	function removeFromRelatedCats($categoryId)
	{
		$database =& JFactory::getDBO();
		$id = $database->getEscaped($categoryId);
		$database->setQuery("DELETE FROM `#__rssfactory_related_categories_map` WHERE `categoryId`='".$id."' OR `relatedId`='".$id."'");
		$database->query();
		
	}

	function getRelatedCategories($id)
	{
		$this->db->setQuery(
			"SELECT `relatedid` FROM `#__rssfactory_related_categories_map`
				WHERE `categoryId`='".$database->getEscaped($id)."'
			UNION
			SELECT `categoryId` FROM `#__rssfactory_related_categories_map`
				WHERE `relatedid`='".$database->getEscaped($id)."'"
		);
		return $this->db->loadResultArray();
	}
}
?>