<?php
/**
 * A class to use nested sets easiely
 * @author Mark Stuppacher <mark@happynet.at>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License version 3 (GPLv3)
 * @version 0.6
 * @
 *
 */
class JTableRfpronestedset extends JTable
{
	/*Properties*/

	var $id=null;
	var $name = null;
	var $description = null;

	/*Methods*/

	/**
	 * Stores a Mysqli object for further use
	 * @param object $mysqli Mysqli object
	 * @return boolean true
	 */
	public function __construct($db){
		//$this->_db =& JFactory::getDBO();

		parent::__construct(APP_CATEGORY_TABLE, APP_CATEGORY_PRIMARY_KEY, $db);
	}

	/**
	 * Creates the root node
	 * @param string $name Name of the new node
	 * @return boolean true
	 */
	function createRootNode($name) {
		$this->_db->setQuery("LOCK TABLES `" . $this->_tbl . "` WRITE");
		$this->_db->query();

		$sql = "SELECT `rgt` FROM `" . $this->_tbl . "` ORDER BY `rgt` DESC LIMIT 1";
		$this->_db->setQuery($sql);
		$result = $this->_db->loadResult();

		if ($this->_db->getAffectedRows() == 0)
		{
			$lft = 1;
			$rgt = 2;
		}
		else
		{
			$lft = $result + 1;
			$rgt = $lft + 1;
		}
		$sql = "INSERT INTO " . $this->_tbl . " (" . APP_CATEGORY_FIELD_NAME . ", lft, rgt) VALUES ('" . $this->_db->getEscaped($name) . "', " . $lft . ", " . $rgt . ");";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$this->_db->setQuery("UNLOCK TABLES");
		$this->_db->query();
	}

	function getRootNode()
	{
		static $rN=null;

		if(!$rN){
			$sql = "SELECT * FROM `" . $this->_tbl . "` ORDER BY `lft` ASC LIMIT 1";
			$this->_db->setQuery($sql);

			$rN = $this->_db->loadObject();

			if(!$rN)
			{
				$this->createRootNode('RFPRO_WORD4ROOT');
				$rN = $this->getRootNode();
			}
		}

		return $rN;
	}


	/**
	 * Creates a new node
	 * @param string $name name of the new node
	 * @param integer $lft lft of parent node
	 * @param integer $rgt	rgt of parent node
	 * @return boolean	true
	 */
	function insertNode($name, $lft, $rgt)
	{
		$sql = "UPDATE " . $this->_tbl . " SET rgt = rgt + 2 WHERE rgt >= " . $rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "UPDATE " . $this->_tbl . " SET lft = lft + 2 WHERE lft > " . $rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "INSERT INTO " . $this->_tbl . " (" . APP_CATEGORY_FIELD_NAME . ", lft, rgt) VALUES ('" . $this->_db->getEscaped($name) . "', " . $rgt . ", " . $rgt . "+1);";

		$this->_db->setQuery($sql);
		$this->_db->query();
	}


	/**
	 * Gets an object with all data of a node
	 * @param integer $id id of the node
	 * @return object object with node-data (id, lft, rgt,...)
	 */
	function getNode($id)
	{
		$id = intval($id);

		$this->_db->setQuery( "SELECT * FROM " . $this->_tbl . " WHERE " . $this->_tbl_key . " = " . $this->_db->getEscaped($id)  );
		$node = $this->_db->loadObject();
		if($this->_db->getAffectedRows() == 0)
		{
			return false;
		}
		return $node;
	}

	/**
	 * Creates a new child node of the node with the given id
	 * @param string $name name of the new node
	 * @param integer $parent id of the parent node
	 * @return boolean true
	 */
	function insertChildNode($name, $parent)
	{
		$p_node = $this->getNode($parent);

		if(!$p_node)
		{
			$rootNode = $this->getRootNode();
			if(!$rootNode)
			{
				$this->createRootNode('ROOT');
			}
			$p_node = $this->getRootNode();
		}

		$this->insertNode($name, $p_node->lft, $p_node->rgt);
		return $this->_db->insertid();
	}

	/**
	 * Creates a multi-dimensional array of the whole tree
	 * @return array multi-dimenssional array of the whole tree
	 */
	function getTree($parentId)
	{
		$parentCondition = '';
		if(intval($parentId))
		{
			$this->_db->setQuery("SELECT * FROM `".$this->_tbl."` WHERE `id`='".$parentId."'");
			$parent = $this->_db->loadObject();

			if($parent->{$this->_tbl_key}==$parentId)
			{
				$parentCondition = ' AND p.lft>'.$parent->lft.' AND p.rgt<'.$parent->rgt.' ';
			}
		}

		$sql = "SELECT n." . $this->_tbl_key . ", n." . APP_CATEGORY_FIELD_NAME . ", n.*, COUNT(*)-1 AS level FROM " . $this->_tbl . " AS n, " . $this->_tbl . " AS p WHERE n.lft BETWEEN p.lft AND p.rgt $parentCondition GROUP BY n.lft ORDER BY n.lft";
		$this->_db->setQuery($sql);

		return $this->_db->loadObjectList($this->_tbl_key);
	}
	
	function getParent($id,$fullDetails=false)
	{
		$path = $this->getPath($id);
		array_pop($path);

		if(!$fullDetails)
		{
			return array_pop($path);
		}
		else
		{
			$partParent = array_pop($path);
			return $this->getNode($partParent->{$this->_tbl_key});
		}
	}

	
	/**
	 * Get the HTML code for an unordered list of the tree
	 * @return string HTML code for an unordered list of the whole tree
	 */
	function treeAsHtml() {
		$tree = $this->getTree();
		$html = "<ul>\n";
		for ($i=0; $i<count($tree); $i++) {
			$html .= "<li>" . $tree[$i][APP_CATEGORY_FIELD_NAME];
			if ($tree[$i]['level'] < $tree[$i+1]['level']) {
				$html .= "\n<ul>\n";
			} elseif ($tree[$i]['level'] == $tree[$i+1]['level']) {
				$html .= "</li>\n";
			} else {
				$diff = $tree[$i]['level'] - $tree[$i+1]['level'];
				$html .= str_repeat("</li>\n</ul>\n", $diff) . "</li>\n";
			}
		}
		$html .= "</ul>\n";
		return $html;
	}

	/**
	 * Moves a node and its children to a new parent
	 * @param integer $id id of the node
	 * @param integer $parentId id of the new parent
	 * @return boolean true
	 */
	function moveNode($id,$parentId)
	{
		$id = $this->_db->getEscaped($id);
		$parentId = $this->_db->getEscaped($parentId);

		$node = $this->getNode($id);
		$newParent = $this->getNode($parentId);
		$iSize = $node->rgt - $node->lft + 1;

		$this->_db->setQuery("LOCK tables " . $this->_tbl . " WRITE;");
		$this->_db->query();

		// step 1: temporary "remove" moving node
		$query = 'UPDATE `'.$this->_tbl.'` SET `lft` = 0-(`lft`), `rgt` = 0-(`rgt`) WHERE `lft` >= "'.$node->lft.'" AND `rgt` <= "'.$node->rgt.'"';
    	$this->_db->setQuery($query);
    	$this->_db->query();

		// step 2: decrease left and/or right position values of currently 'lower' items (and parents)
    	$query = 'UPDATE `'.$this->_tbl.'` SET `lft` = `lft` - '.$iSize.' WHERE `lft` > "'.$node->rgt.'"';
    	$this->_db->setQuery($query);
    	$this->_db->query();
    	$query = 'UPDATE `'.$this->_tbl.'` SET `rgt` = `rgt` - '.$iSize.' WHERE `rgt` > "'.$node->rgt.'"';
    	$this->_db->setQuery($query);
    	$this->_db->query();

		// step 3: increase left and/or right position values of future 'lower' items (and parents)
		$query = 'UPDATE `'.$this->_tbl.'` SET `lft` = `lft` + '.$iSize.' WHERE `lft` >= "'.($newParent->rgt > $node->rgt ? $newParent->rgt - $iSize : $newParent->rgt).'"';
		$this->_db->setQuery($query);
    	$this->_db->query();
    	$query = 'UPDATE `'.$this->_tbl.'` SET `rgt` = `rgt` + '.$iSize.' WHERE `rgt` >= "'.($newParent->rgt > $node->rgt ? $newParent->rgt - $iSize : $newParent->rgt).'"';
		$this->_db->setQuery($query);
    	$this->_db->query();

		// step 4: move node (ant it's subnodes) and update it's parent item id
    	$query = 'UPDATE `'.$this->_tbl.'` SET `lft` = 0-(`lft`)+'.($newParent->rgt > $node->rgt ? $newParent->rgt - $node->rgt - 1 : $newParent->rgt - $node->rgt - 1 + $iSize).', `rgt` = 0-(`rgt`)+'.($newParent->rgt > $node->rgt ? $newParent->rgt - $node->rgt - 1 : $newParent->rgt - $node->rgt - 1 + $iSize).' WHERE `lft` <= "'.(0-$node->lft).'" AND `rgt` >= "'.(0-$node->rgt).'"';
		$this->_db->setQuery($query);
    	$this->_db->query();
//		$query = 'UPDATE `'.$this->_tbl.'` SET `parentId` = "'.$newParent->id.'" WHERE `id`="'.$node->id.'"';
//		$this->_db->setQuery($query);
//    	$this->_db->query();
    	
		$this->_db->setQuery("UNLOCK TABLES;");
		$this->_db->query();
	}
	
	
	/**
	 * Deletes a node and all it's children
	 * @param integer $id id of the node to delete
	 * @return boolean true
	 */
	function deleteNode($id) {
		
		$node = $this->getNode($id);
		if(!$node){
			return false;
		}

		$sql = "DELETE FROM " . $this->_tbl . " WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();
		$sql = "UPDATE " . $this->_tbl . " SET lft = lft - ROUND((" . $node->rgt . " - " . $node->lft . " + 1)) WHERE lft > " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();
		$sql = "UPDATE " . $this->_tbl . " SET rgt = rgt - ROUND((" . $node->rgt . " - " . $node->lft . " + 1)) WHERE rgt > " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		return true;
	}

	/**
	 * Deletes a node and increases the level of all children by one
	 * @param integer $id id of the node to delete
	 * @return boolean true
	 */
	function deleteSingleNode($id) {
		$this->_db->setQuery("LOCK tables " . $this->_tbl . " WRITE;");
		$this->_db->query();

		$node = $this->getNode($id);

		$sql = "DELETE FROM " . $this->_tbl . " WHERE lft = " . $node->lft . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "UPDATE " . $this->_tbl . " SET lft = lft - 1, rgt = rgt - 1 WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "UPDATE " . $this->_tbl . " SET lft = lft - 2 WHERE lft > " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "UPDATE " . $this->_tbl . " SET rgt = rgt - 2 WHERE rgt > " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$this->_db->setQuery("UNLOCK TABLES;");
		$this->_db->query();

		return true;
	}
	
	
	/**
	 * Gets a multidimensional array containing the path to defined node
	 * @param integer $id id of the node to which the path should point
	 * @return array multidimensional array with the data of the nodes in the tree
	 */
	function getPath($id)
	{
		$sql = "SELECT p." . $this->_tbl_key . ", p." . APP_CATEGORY_FIELD_NAME . ", p." . APP_CATEGORY_FIELD_DESCRIPTION . " FROM " . $this->_tbl . " n, " . $this->_tbl . " p WHERE n.lft BETWEEN p.lft AND p.rgt AND n." . $this->_tbl_key ." = " . $this->_db->getEscaped($id) . " ORDER BY p.lft;";
		$this->_db->setQuery($sql);

		return $this->_db->loadObjectList($this->_tbl_key);
	}

	function getSubTree($id,$includeParent=false,$maxLevels=0,$countItems=false)
	{
		if(!$id){
			$rN = $this->getRootNode();
			$id = $rN->id;
		}
		$sql = "SELECT node.*,
					   (COUNT(DISTINCT parent.".APP_CATEGORY_FIELD_NAME.") - (sub_tree.depth + 1)) AS depth,
					   COUNT(DISTINCT `feed`.`id`) AS nrFeeds,
					   COUNT(DISTINCT `story`.`id`) AS nrStories					   
				FROM ".$this->_tbl." AS node
					LEFT JOIN `#__rssfactory` AS feed
						ON `node`.`id`=`feed`.`cat`
					LEFT JOIN `#__rssfactory_cache` AS story
					   ON `feed`.`id`=`story`.`rssid`,
					".$this->_tbl." AS parent,
					".$this->_tbl." AS sub_parent,					
					(
						SELECT node.".$this->_tbl_key.", (COUNT(parent.".APP_CATEGORY_FIELD_NAME.") - 1) AS depth
						FROM ".$this->_tbl." AS node,
						".$this->_tbl." AS parent
						WHERE node.lft BETWEEN parent.lft AND parent.rgt
						AND node.".$this->_tbl_key." = '".$this->_db->getEscaped($id)."'
						GROUP BY node.".$this->_tbl_key."
						ORDER BY node.lft
					)AS sub_tree

				WHERE node.lft BETWEEN parent.lft AND parent.rgt
					AND node.lft BETWEEN sub_parent.lft AND sub_parent.rgt
					AND sub_parent.".$this->_tbl_key." = sub_tree.".$this->_tbl_key."					
				GROUP BY `node`.`".$this->_tbl_key.'` '.
				($maxLevels ? (' HAVING `depth`<='.$this->_db->getEscaped($maxLevels).' ') : '' ).
				" ORDER BY node.lft ";
		$this->_db->setQuery($sql);
		//vardump($this->_db->_sql,1);

		$rows = $this->_db->loadObjectList();
		//vardump($rows,1);
		if(!$includeParent)
		{
			array_shift($rows);
		}

		return $rows;
	}
	
	
	/**
	 * Gets the id of a node depending on it's rgt value
	 * @param integer $rgt rgt value of the node
	 * @return integer id of the node
	 */
	function getIdRgt($rgt)
	{
		$this->_db->setQuery( "SELECT " . $this->_tbl_key . " FROM " . $this->_tbl . " WHERE rgt = " . $this->_db->getEscaped($rgt) );
		return $this->_db->loadResult();
	}
	
	
	/**
	 * Moves a node one position to the left staying in the same level
	 * @param $nodeId id of the node to move
	 * @return boolean true
	 */
	function moveLft($nodeId)
	{
		$node = $this->getNode($nodeId);
		$brotherId = $this->getIdRgt($node->lft-1);
		if (!$brotherId)
		{
			//node can't be moved left
			JError::raiseWarning(1,'Node can not be moved left!');
			return;
		}
		$brother = $this->getNode($brotherId);
		
		$nodeSize = $node->rgt - $node->lft + 1;
		$brotherSize = $brother->rgt - $brother->lft + 1;

		$sql = "SELECT " . $this->_tbl_key . " FROM " . $this->_tbl . " WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$res = $this->_db->loadObjectList();

		$idsNotToMove = array();
		foreach ($res as $n)
		{
			$idsNotToMove[] = $n->{$this->_tbl_key};
		}

		$sql = "UPDATE " . $this->_tbl . " SET lft = lft - " . $brotherSize . ", rgt = rgt - " . $brotherSize . " WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();

		$sql = "UPDATE " . $this->_tbl . " SET lft = lft + " . $nodeSize . ", rgt = rgt + " . $nodeSize . " WHERE lft BETWEEN " . $brother->lft . " AND " . $brother->rgt;
		for ($i = 0; $i < count($idsNotToMove); $i++)
		{
			$sql .= " AND " . $this->_tbl_key . " != " . $idsNotToMove[$i];
		}
		$this->_db->setQuery($sql);
		$this->_db->query();
		
		return true;
	}
	
	
	/**
	 * Gets the id of a node depending on it's lft value
	 * @param integer $lft lft value of the node
	 * @return integer id of the node
	 */
	function getIdLft($lft)
	{
		$this->_db->setQuery("SELECT " . $this->_tbl_key . " FROM " . $this->_tbl . " WHERE lft = " . $this->_db->getEscaped($lft) );
		return $this->_db->loadResult();
	}

	/**
	 * Moves a node one position to the right staying in the same level
	 * @param $nodeId id of the node to move
	 * @return boolean true
	 */
	function moveRgt($nodeId)
	{
		$node = $this->getNode($nodeId);
		$brotherId = $this->getIdLft($node->rgt+1);
		if ($brotherId == false)
		{
			//node can't be moved right
			JError::raiseWarning(1,'Node can not be moved right!');
			return;
		}
		$brother = $this->getNode($brotherId);

		$nodeSize = $node->rgt - $node->lft + 1;
		$brotherSize = $brother->rgt - $brother->lft + 1;
		
		
		$this->_db->setQuery("SELECT " . $this->_tbl_key . " FROM " . $this->_tbl . " WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt);
		$result = $this->_db->loadObjectList();

		$idsNotToMove = array();
		foreach($result as $r)
		{
			$idsNotToMove[] = $r->{$this->_tbl_key};
		}
		
		$sql = "UPDATE " . $this->_tbl . " SET lft = lft + " . $brotherSize . ", rgt = rgt + " . $brotherSize . " WHERE lft BETWEEN " . $node->lft . " AND " . $node->rgt . ";";
		$this->_db->setQuery($sql);
		$this->_db->query();
			
		$sql = "UPDATE " . $this->_tbl . " SET lft = lft - " . $nodeSize . ", rgt = rgt - " . $nodeSize . " WHERE lft BETWEEN " . $brother->lft . " AND " . $brother->rgt;
		for ($i = 0; $i < count($idsNotToMove); $i++) {
			$sql .= " AND " . $this->_tbl_key . " != " . $idsNotToMove[$i];
		}		
		$this->_db->setQuery($sql);
		$this->_db->query();

		return true;
	}
	
	
	/**
	 * Prints a error message
	 * @param int $id array-key of the message
	 * @param boolean $continue continue script or not
	 * @return void
	 */
	function error($id, $continue = false) {
		$errors = array ();
		$errors[] = 'There is no node with the given id!';
		$errors[] = 'No entries!';
		$errors[] = 'Node can\'t be moved to the right!';
		$errors[] = 'Node can\'t be moved to the left!';
		echo $errors[$id];
		if ($continue == false) {
			exit;
		}
	}

	function getRelatedCategories()
	{
		$database =& JFactory::getDBO();
		$database->setQuery(
			"SELECT `relatedid` FROM `#__rssfactory_related_categories_map` 
				WHERE `categoryId`='".$database->getEscaped($this->id)."'
			UNION
			SELECT `categoryId` FROM `#__rssfactory_related_categories_map` 
				WHERE `relatedid`='".$database->getEscaped($this->id)."'"
		);
		return $database->loadResultArray();
	}	
}
?>