<?php
/**
* @package rssFactory
* @version 1.5
* @copyright www.thefactory.ro
* @license commercial
*/
defined( '_JEXEC' ) or die( 'Direct Access to this location is not allowed.' );

jimport('phputf8.utils.validation');

class JRSSFactoryProParser{

	var $parserName = null;
	var $parser = null;
	var $xmlEncoding = '';

	function JRSSFactoryProParser($parserName){

		$parserName = strtolower($parserName);
		$this->parserName = $parserName;

		$parserDir = RSS_FACTORY_COMPONENT_PATH.DS.'parsers'.DS.$this->parserName;

		if(!file_exists($parserDir)){
			JError::raiseWarning('500','Parser '.strtoupper($this->parserName).' not found! No feed refreshed!');
			return;
		}

		RFProHelper::loadLanguage();

		switch($parserName)
		{
			case 'simplepie':
				$oldER=error_reporting();
				error_reporting(0);
				require_once($parserDir.DS.'simplepie.inc');
				require_once($parserDir.DS.'idn'.DS.'idna_convert.class.php');
				error_reporting($oldER);
				$this->parser = new SimplePie();
				$this->parser->enable_cache(false);
			break;
			case 'domit':
				require_once($parserDir. DS . 'xml_domit_rss.php');
				$this->parser = new xml_domit_rss_document();
			break;
			case 'magpie':
				require_once($parserDir. DS . 'rss_fetch.inc');
				define('MAGPIE_CACHE_ON',false);
				define('MAGPIE_OUTPUT_ENCODING', 'UTF-8');
				$this->parser = new MagpieRSSParser;
			break;

		}

		return $this->parser;
	}
	
	function &getInstance($parserName)
	{
		static $parser;
		
		if(!is_object($parser) || (is_object($parser) && $parser->parserName!=$parserName)){
			$parser = new JRSSFactoryProParser($parserName);
		}
		return $parser;
	}
	
	function fetchURL($url){
		if(!$this->parser){
			return;
		}

		switch($this->parserName){
			case 'simplepie':
				$this->parser->set_feed_url($url);
				if(!$this->parser->init()){
					return false;
				}
			break;
			case 'domit':
				if(!$this->parser->loadRSS($url)){
					return false;
				}
			break;
			case 'magpie':
				if(!$this->parser->fetch_rss($url)){
					return false;
				}
			break;
		}
		
		$this->setEncoding();

		return true;
	}
	
	function setEncoding(){
		switch($this->parserName){
			case 'simplepie':
				$this->xmlEncoding = $this->parser->get_encoding();
			break;
			case 'domit':
				//domit doesn't seem to have a method to retreive xml's encoding
				$this->xmlEncoding = 'UTF-8';
			break;
			case 'magpie':
				$this->xmlEncoding = 'UTF-8';
			break;
		}
	}
	
	function getEncoding()
	{
		return $this->xmlEncoding;
	}

	function parse2cache(&$rssSource)
	{
		if(!$this->parser){
			return;
		}

		$database = &JFactory::getDBO();
		
		$config =& RFProHelper::getConfig();

		$query = "DELETE FROM `#__rssfactory_cache` WHERE `rssid`='".$rssSource->id."' AND `archived`='1' AND DATE_SUB(NOW(), INTERVAL ".intval($config->archivedeleteinterval*7)." DAY)>=`date`";
		$database->setQuery($query);
		$database->query();

		$query = "UPDATE `#__rssfactory_cache` SET `archived`='1' where rssid='".$rssSource->id."' ";
		$database->setQuery( $query );
		$database->query();

		if(!$this->fetchURL($rssSource->url)){
			return false;
		}

		$method = ($this->parserName).'2cache';
		return  $this->$method($rssSource);
	}

	function &getParsedObject(&$rssSource){
		if(!$this->parser){
			return;
		}

		$database = &JFactory::getDBO();

		$config =& RFProHelper::getConfig();

		$this->fetchURL($rssSource->url);

		switch($this->parserName){
			case 'simplepie':
				//vardump($this->parser,1);
			break;
			case 'domit':
				$this->parser->loadRSS($url);
			break;
			case 'magpie':
				$this->parser->fetch_rss($url);
			break;
		}
	}

	function isDuplicateStory(&$row)
	{
		$database = &JFactory::getDBO();

//		/** check if item with same title exists from same source */
//		$query1 = "SELECT `id` FROM `#__rssfactory_cache` WHERE `rssid`='".$row->rssid."' AND `item_title`='".$database->getEscaped($row->item_title)."'";
//		$database->setQuery($query1);
//		$found_id = $database->loadResult();
//		
//		if($found_id)
//		{
//			return $found_id;
//		}

		/** check if item with same hash exists in cache */
		//$row->item_hash = sha1( strip_tags($row->item_title.$row->item_description) );
		$row->item_hash = sha1( strip_tags($row->item_title) );

		$query2 = "SELECT `id` FROM `#__rssfactory_cache` WHERE `rssid`='".$row->rssid."' AND `item_hash`='".$row->item_hash."'";
		$database->setQuery($query2);
		$found_id = $database->loadResult();

		if($found_id)
		{
			return $found_id;
		}

		return false;
	}

	function simplepie2cache(&$rssSource)
	{
		global $mainframe;

		$database = &JFactory::getDBO();
		$config =& RFProHelper::getConfig();

		$limit = $this->parser->get_item_quantity($rssSource->nrfeeds);
		$items = $this->parser->get_items(0,$limit);

		$row =& JTable::getInstance('rfproitem');

		$row->channel_title = $this->convert2utf8($this->parser->get_title());
		$row->channel_description = $this->convert2utf8($this->parser->get_description());

		$row->channel_link = $this->parser->get_link();

		$nrCachedItems = 0;
		foreach($items as $item)
		{
			$row->id=null;

	        $row->item_title = $this->convert2utf8($item->get_title());
	        $row->item_description = $this->convert2utf8($item->get_description());
	        $row->item_date = gmdate('Y-m-d H:i:s',$item->get_date('U'));

	        $row->rssid=$rssSource->id;
	        $row->rssurl=$rssSource->url;

			if($found_id=$this->isDuplicateStory($row))
			{
				$query = "UPDATE `#__rssfactory_cache` SET `archived`='0',`item_date`='".$row->item_date."' WHERE `id`='".$found_id."'";
				$database->setQuery($query);
				$database->query();
				continue;
			}

			$row->item_description = preg_replace('/<a /','<a target="_blank" rel="nofollow" ',$row->item_description);
			$row->item_description = preg_replace('/&gt;a /','&gt;a target="_blank" rel="nofollow" ',$row->item_description);

			//apply word filters
			if(!$row->preStoreFilter($config))
			{
				continue;
			}

			if(!$row->item_date)
			{
				$row->item_date = gmdate("Y-m-d H:i:s" );
			}

			//a little trick to make sure all the characters in the query are URL encoded
			$uri =& JURI::getInstance( $item->get_link() );
			$query = $uri->getQuery(true);
			$uri->setQuery(null);
			$uri->setQuery($query);
			$row->item_link = $uri->toString();			

			$feed = $item->get_feed();
			$row->item_source = $feed->get_title();

			$enclosures = $item->get_enclosures();
			unset($e);
			$e = array();

			if($enclosures){
	        	foreach($enclosures as $enclosure){
	        		$e[] = JArrayHelper::fromObject($enclosure);
	        	}
	        }
	        $row->item_enclosure = base64_encode(serialize($e));
			
			//VERY important!!!
			$row->date = gmdate('Y-m-d H:i:s');

			$row->store();

			$this->storeArticle($rssSource,$row);

			$nrCachedItems++;
		}

		return $nrCachedItems;
	}
	
	function domit2cache(&$rssSource){

		$database = &JFactory::getDBO();
		$config =& RFProHelper::getConfig();

		//DOMIT knows how to parse feeds with multiple channels, but this is deprecated; so we take only the first channel
		$currChannel =& $this->parser->getChannel(0);
		$nrItems = $currChannel->getItemCount();

		$limit = $rssSource->nrfeeds;
		if($limit==0 || $limit>$nrItems){
			$limit = $nrItems;
		}

		$row=new JRSSFactoryPRO_Cache($database);
    	$now = date( "Y-m-d H:i:s", time());

		$row->channel_title = $this->convert2utf8($currChannel->getTitle());
		$row->channel_title = htmlentities($row->channel_title,ENT_QUOTES,'UTF-8');

		$row->channel_description = $this->convert2utf8($currChannel->getDescription());
		$row->channel_description = htmlentities($row->channel_description,ENT_QUOTES,'UTF-8');

		$row->channel_link = $currChannel->getLink();


		$nrCachedItems = 0;
		for($i=0;$i<$limit;$i++){

			$currItem =& $currChannel->getItem($i);
			
			$row->id=null;
	        
	        //prepare item title
			$row->item_title = $this->convert2utf8($currItem->getTitle());
			$row->item_title = htmlentities($row->item_title,ENT_QUOTES,'UTF-8');

			//prepare item description
			$row->item_description = $this->convert2utf8($currItem->getDescription());
			$row->item_description = preg_replace('/<a /','<a target="_blank" rel="nofollow" ',$row->item_description);
			$row->item_description = preg_replace('/&gt;a /','&gt;a target="_blank" rel="nofollow" ',$row->item_description);
			//apply word filters
			if(!$row->preStoreFilter($config)){
				continue;
			}
			

			/** check if item already exists in cache */
			$itemResume = $row->item_title.$row->item_description;
			$row->item_hash = sha1( $this->prepareFeedSHA1($itemResume) );

			$query = "SELECT `id` FROM `#__rssfactory_cache` WHERE `rssid`='".$rssSource->id."' AND `item_hash`='".$row->item_hash."'";
			$database->setQuery($query);
			$found_id = $database->loadResult();
			if($found_id){
				$query = "UPDATE `#__rssfactory_cache` SET `archived`='0' WHERE `id`='".$found_id."'";
				$database->setQuery($query);
				$database->query();
				continue;
			}
			/** */
			
			$row->rssid=$rssSource->id;
	        $row->rssurl=$rssSource->url; 

			$publishDate = $currItem->getPubDate('Y-m-d H:i:s');
			if(!$publishDate){
				$publishDate = time();
			}
			$row->item_date = date('Y-m-d H:i:s', strtotime($publishDate));
			
			$row->item_link = $currItem->getLink();

			if ($currChannel->hasElement('source')){
	            $row->item_source=$currChannel->getElementText('source');
	        }
	        
	        $enclosure = $currItem->getEnclosure();
			unset($e);
			$e = array();
			if($enclosure){
	        	$e[0] = array('url'=>$enclosure->getUrl(),
							'length'=>$enclosure->getLength(),
							'type'=>$enclosure->getType()
							);
	        }
	        $row->item_enclosure = base64_encode(serialize($e));

       	 	//VERY important!!!
			$row->date = gmdate('Y-m-d H:i:s');

			$row->store();

			$this->storeArticle($rssSource,$row);

			$nrCachedItems++;
		}
		return $nrCachedItems;
	}
	
	function magpie2cache(&$rssSource){

		$database = &JFactory::getDBO();
		$config =& RFProHelper::getConfig();

		$resp =& $this->parser->resp;
		$items = $resp->items;

	    $nrItems = count($items);
		$limit = $rssSource->nrfeeds;
		if($limit==0 || $limit>$nrItems){
			$limit = $nrItems;
		}

	    $row=new JRSSFactoryPRO_Cache($database);
	    $now = date( "Y-m-d H:i:s", time() );

		$row->channel_title =isset($resp->channel['title'])?($resp->channel['title']):'';
        $row->channel_title = $this->convert2utf8($row->channel_title);

		$row->channel_description=isset($resp->channel['description'])?($resp->channel['description']):'';
        $row->channel_description = $this->convert2utf8($row->channel_description);

		$row->channel_link=isset($resp->channel['link'])?$resp->channel['link']:'';

		$nrCachedItems = 0;
		for($i=0; $i<$limit; $i++){
			$item = $items[$i];

			$row->id=null;

			//prepare item title
			$row->item_title = isset($item['title'])?($item['title']):'';
			$row->item_title = $this->convert2utf8($row->item_title);
			$row->item_title = htmlentities($row->item_title,ENT_QUOTES,'UTF-8');

			//prepare item description
			$row->item_description = isset($item['description'])?($item['description']):'';
			$row->item_description = $this->convert2utf8($row->item_description);
			$row->item_description = preg_replace('/<a /','<a target="_blank" rel="nofollow" ',$row->item_description);
			$row->item_description = preg_replace('/&gt;a /','&gt;a target="_blank" rel="nofollow" ',$row->item_description);
			//apply word filters
			if(!$row->preStoreFilter($config)){
				continue;
			}

			/** check if item already exists in cache */
			$itemResume = $row->item_title.$row->item_description;
			$row->item_hash = sha1( $this->prepareFeedSHA1($itemResume) );

			$query = "SELECT `id` FROM `#__rssfactory_cache` WHERE `rssid`='".$rssSource->id."' AND `item_hash`='".$row->item_hash."'";
			$database->setQuery($query);
			$found_id = $database->loadResult();
			if($found_id){
				$query = "UPDATE `#__rssfactory_cache` SET `archived`='0' WHERE `id`='".$found_id."'";
				$database->setQuery($query);
				$database->query();
				continue;
			}
			/** */

			$row->rssid = $rssSource->id;
			$row->rssurl = $rssSource->url;

			$publishDate = ( isset( $item['pubDate'] )?$item['pubDate'] : 
								( isset( $item['dc']['date'] ) ? $item['dc']['date'] : 
									( isset($item['dc:date'] ) ? $item['dc:date'] : 
										null ) ) 
						   );

			$publishDate = strtotime($publishDate);
			if(!$publishDate){
				$publishDate = time();
			}
			$row->item_date = date( 'Y-m-d H:i:s', $publishDate );

			$row->item_link = isset($item['link']) ? $item['link'] : '';
            $row->item_source = isset($item['source']) ? $item['source'] : '';

            $enclosure = isset($item['enclosure']) ? $item['enclosure'] : array();
            $row->item_enclosure = base64_encode(serialize($enclosure));

			//VERY important!!!
			$row->date = gmdate('Y-m-d H:i:s');

			$row->store();

			$this->storeArticle($rssSource,$row);

			$nrCachedItems++;
		}

	    return $nrCachedItems;
	}

	function convert2utf8(&$string){
		if(!utf8_is_valid($string)){
			$string = utf8_encode($string);
		}
		return $string;
	}

	function storeArticle(&$rssSource,&$row)
	{
		global $option;
		$database = &JFactory::getDBO();
		$config =& RFProHelper::getConfig();

		if(!$config->enable_import2content)
		{
			return;
		}

		if(!$rssSource->i2c_enabled)
		{
			return;
		}

		if($rssSource->i2c_full_article)
		{
			$row->item_description = RFProAdminHelper::parseFullArticle($row);
		}

		if($config->enablei2cwordfilter)
		{
			static $allowedWords=array(),$bannedWords=array();
			if(!count($allowedWords) && trim($config->i2callowedwords)!='')
			{
				$allowedWords = explode(',',trim($config->i2callowedwords));
			}
			if(!count($bannedWords) && trim($config->i2cbannedwords)!='')
			{
				$bannedWords = explode(',',trim($config->i2cbannedwords));
			}

			$allowArticle = false;

			foreach($allowedWords as $aw)
			{
				if(stripos($row->item_title.$row->item_description,$aw)!==false)
				{
					$allowArticle = true;
					continue;
				}
			}
			
			$bannArticle = false;
			
			foreach($bannedWords as $bw)
			{
				if(stripos($row->item_title.$row->item_description,$aw)!==false)
				{
					$bannArticle = false;
					continue;
				}
			}

			if(!$allowArticle || $bannArticle)
			{
				return false;
			}
		}

		$article =& JTable::getInstance('content');

		$article->sectionid = (int) $rssSource->i2c_sectionid ? $rssSource->i2c_sectionid : $config->i2c_default_sectionid;
		$article->catid = (int) $rssSource->i2c_catid ? $rssSource->i2c_catid : $config->i2c_default_catid;

		//$article->created = JHTML::date($row->item_date,'%Y-%m-%d %H-%M-%S');
		$article->created = $row->item_date;
		$article->created_by = $rssSource->i2c_author ? $rssSource->i2c_author : $config->i2c_default_author;

		$article->state = $config->i2c_default_article_state;
        $article->state = $rssSource->i2c_published;

		$publishing_period = $rssSource->i2c_publishing_period ? $rssSource->i2c_publishing_period : $config->i2c_default_publishing_period;

		$jconfig =& JFactory::getConfig();
		$publish_date =& JFactory::getDate();
		$publish_date->setOffset($jconfig->getValue('config.offset'));
		$article->publish_up = $publish_date->toFormat('%Y-%m-%d');
		$publish_date->setOffset($jconfig->getValue('config.offset') + $publishing_period*24);
		$article->publish_down = $publish_date->toFormat('%Y-%m-%d');

		$prepend_text = $rssSource->i2c_prepend ? $rssSource->i2c_prepend : $config->i2c_default_prepend_text;
		$append_text = $rssSource->i2c_append ? $rssSource->i2c_append : $config->i2c_default_append_text;
		$content = $prepend_text . $row->item_description . $append_text;

		$article->title = html_entity_decode($row->item_title,ENT_COMPAT,'UTF-8');

		if($config->i2c_add_source_link)
		{
			$content .= '<br />'.JHTML::link($row->item_link,RFPRO_WORD4NEWSSOURCE);
		}
		
		if($config->i2c_add_read_more)
		{
			$lang =& JFactory::getLanguage();
			$lang->load('joomla',JPATH_SITE);

			if($config->i2c_readmore_options==1){
				$readmore_link = JRoute::_('index.php?option='.$option.'&task=visit&id='.$row->id);
			}
			else{
				$readmore_link = $row->item_link;
			}
			$aTarget = $config->output_iframe == 'popup_original_site' ? 'target="_blank"' : '';
			$read_more = '<br><p class="rsspro_readmore"><a '.$aTarget.' href='.$readmore_link.'>'.JText::_('READ MORE...').'</a></p>';
			$content .= $read_more;
		}

		$article->introtext = $content;

		$this->prepareSaveArticle($article);

		$article->store();

		if ($rssSource->i2c_frontpage)
		{
            $query="UPDATE `#__content_frontpage` SET `ordering`=`ordering`+1";
			$database->setQuery($query);
			$database->query();

            $query="INSERT INTO `#__content_frontpage` (`content_id`,`ordering`) VALUES ('".$article->id."','1');";
			$database->setQuery($query);
			$database->query();
		}
	}

	function prepareSaveArticle( &$row )
	{
		// Filter settings
		jimport( 'joomla.application.component.helper' );
		$config	= JComponentHelper::getParams( 'com_content' );
		$user	= &JFactory::getUser();
		$gid	= $user->get( 'gid' );

		$filterGroups	=  $config->get( 'filter_groups' );
		
		// convert to array if one group selected
		if ( (!is_array($filterGroups) && (int) $filterGroups > 0) ) { 
			$filterGroups = array($filterGroups);
		}

		if (is_array($filterGroups) && in_array( $gid, $filterGroups ))
		{
			$filterType		= $config->get( 'filter_type' );
			$filterTags		= preg_split( '#[,\s]+#', trim( $config->get( 'filter_tags' ) ) );
			$filterAttrs	= preg_split( '#[,\s]+#', trim( $config->get( 'filter_attritbutes' ) ) );
			switch ($filterType)
			{
				case 'NH':
					$filter	= new JFilterInput();
					break;
				case 'WL':
					$filter	= new JFilterInput( $filterTags, $filterAttrs, 0, 0, 0);  // turn off xss auto clean
					break;
				case 'BL':
				default:
					$filter	= new JFilterInput( $filterTags, $filterAttrs, 1, 1 );
					break;
			}
			$row->introtext	= $filter->clean( $row->introtext );
			$row->fulltext	= $filter->clean( $row->fulltext );
		} elseif(empty($filterGroups) && $gid != '25') { // no default filtering for super admin (gid=25)
			$filter = new JFilterInput( array(), array(), 1, 1 );
			$row->introtext	= $filter->clean( $row->introtext );
			$row->fulltext	= $filter->clean( $row->fulltext );
		}
		return true;
	}
}
?>